/****************************************************************************
** Meta object code from reading C++ file 'ddciiconplayer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/util/ddciiconplayer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ddciiconplayer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DDciIconImagePlayer::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DDciIconImagePlayer",
        "started",
        "",
        "updated",
        "finished",
        "stateChanged",
        "start",
        "speed",
        "Gui::DDciIconImagePlayer::Flags",
        "flags",
        "stop",
        "state",
        "Gui::DDciIconImagePlayer::State",
        "State",
        "NotRunning",
        "WaitingRead",
        "Running"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updated'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'start'
        QtMocHelpers::SlotData<bool(float, Gui::DDciIconImagePlayer::Flags)>(6, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Float, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<bool(float)>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Float, 7 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<bool()>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'state'
        QtMocHelpers::PropertyData<Gui::DDciIconImagePlayer::State>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(13, 13, QMC::EnumFlags{}).add({
            {   14, State::NotRunning },
            {   15, State::WaitingRead },
            {   16, State::Running },
        }),
    };
    return QtMocHelpers::metaObjectData<DDciIconImagePlayer, qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DDciIconImagePlayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DDciIconImagePlayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DDciIconImagePlayer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->updated(); break;
        case 2: _t->finished(); break;
        case 3: _t->stateChanged(); break;
        case 4: { bool _r = _t->start((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Gui::DDciIconImagePlayer::Flags>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->start((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->start();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->stop(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DDciIconImagePlayer::*)()>(_a, &DDciIconImagePlayer::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDciIconImagePlayer::*)()>(_a, &DDciIconImagePlayer::updated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDciIconImagePlayer::*)()>(_a, &DDciIconImagePlayer::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDciIconImagePlayer::*)()>(_a, &DDciIconImagePlayer::stateChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Gui::DDciIconImagePlayer::State*>(_v) = _t->state(); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DDciIconImagePlayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DDciIconImagePlayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui19DDciIconImagePlayerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DDciIconImagePlayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DDciIconImagePlayer::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DDciIconImagePlayer::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DDciIconImagePlayer::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Dtk::Gui::DDciIconImagePlayer::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DDciIconPlayer::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DDciIconPlayer",
        "stateChanged",
        "",
        "updated",
        "modeChanged",
        "DDciIcon::Mode",
        "oldMode",
        "newMode",
        "_q_playFromQueue",
        "State",
        "Idle",
        "Busy"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updated'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void(DDciIcon::Mode, DDciIcon::Mode)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 5, 7 },
        }}),
        // Slot '_q_playFromQueue'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(9, 9, QMC::EnumFlags{}).add({
            {   10, State::Idle },
            {   11, State::Busy },
        }),
    };
    return QtMocHelpers::metaObjectData<DDciIconPlayer, qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DDciIconPlayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DDciIconPlayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DDciIconPlayer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged(); break;
        case 1: _t->updated(); break;
        case 2: _t->modeChanged((*reinterpret_cast<std::add_pointer_t<DDciIcon::Mode>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DDciIcon::Mode>>(_a[2]))); break;
        case 3: _t->d_func()->_q_playFromQueue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DDciIconPlayer::*)()>(_a, &DDciIconPlayer::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDciIconPlayer::*)()>(_a, &DDciIconPlayer::updated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDciIconPlayer::*)(DDciIcon::Mode , DDciIcon::Mode )>(_a, &DDciIconPlayer::modeChanged, 2))
            return;
    }
}

const QMetaObject *Dtk::Gui::DDciIconPlayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DDciIconPlayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DDciIconPlayerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DDciIconPlayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DDciIconPlayer::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DDciIconPlayer::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DDciIconPlayer::modeChanged(DDciIcon::Mode _t1, DDciIcon::Mode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
