/****************************************************************************
** Meta object code from reading C++ file 'dregionmonitor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/kernel/dregionmonitor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dregionmonitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DRegionMonitor::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DRegionMonitor",
        "buttonPress",
        "",
        "QPoint",
        "p",
        "flag",
        "buttonRelease",
        "cursorMove",
        "cursorEnter",
        "cursorLeave",
        "keyPress",
        "keyname",
        "keyRelease",
        "registerdFlagsChanged",
        "RegisterdFlags",
        "flags",
        "coordinateTypeChanged",
        "CoordinateType",
        "type",
        "registerRegion",
        "QRegion",
        "region",
        "unregisterRegion",
        "setWatchedRegion",
        "setRegisterFlags",
        "setCoordinateType",
        "_q_ButtonPress",
        "_q_ButtonRelease",
        "_q_CursorMove",
        "_q_CursorEnter",
        "_q_CursorLeave",
        "_q_KeyPress",
        "_q_KeyRelease",
        "coordinateType",
        "registerdFlags",
        "ScaleRatio",
        "Original"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'buttonPress'
        QtMocHelpers::SignalData<void(const QPoint &, const int) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'buttonRelease'
        QtMocHelpers::SignalData<void(const QPoint &, const int) const>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'cursorMove'
        QtMocHelpers::SignalData<void(const QPoint &) const>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'cursorEnter'
        QtMocHelpers::SignalData<void(const QPoint &) const>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'cursorLeave'
        QtMocHelpers::SignalData<void(const QPoint &) const>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'keyPress'
        QtMocHelpers::SignalData<void(const QString &) const>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'keyRelease'
        QtMocHelpers::SignalData<void(const QString &) const>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'registerdFlagsChanged'
        QtMocHelpers::SignalData<void(RegisterdFlags) const>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'coordinateTypeChanged'
        QtMocHelpers::SignalData<void(enum CoordinateType) const>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'registerRegion'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'registerRegion'
        QtMocHelpers::SlotData<void(const QRegion &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'unregisterRegion'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setWatchedRegion'
        QtMocHelpers::SlotData<void(const QRegion &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'setRegisterFlags'
        QtMocHelpers::SlotData<void(RegisterdFlags)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'setCoordinateType'
        QtMocHelpers::SlotData<void(enum CoordinateType)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot '_q_ButtonPress'
        QtMocHelpers::SlotData<void(const int, const int, const int, const QString &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot '_q_ButtonRelease'
        QtMocHelpers::SlotData<void(const int, const int, const int, const QString &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot '_q_CursorMove'
        QtMocHelpers::SlotData<void(const int, const int, const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot '_q_CursorEnter'
        QtMocHelpers::SlotData<void(const int, const int, const QString &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot '_q_CursorLeave'
        QtMocHelpers::SlotData<void(const int, const int, const QString &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot '_q_KeyPress'
        QtMocHelpers::SlotData<void(const QString &, const int, const int, const QString &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot '_q_KeyRelease'
        QtMocHelpers::SlotData<void(const QString &, const int, const int, const QString &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'coordinateType'
        QtMocHelpers::PropertyData<enum CoordinateType>(33, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'registerdFlags'
        QtMocHelpers::PropertyData<RegisterdFlags>(34, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CoordinateType'
        QtMocHelpers::EnumData<enum CoordinateType>(17, 17, QMC::EnumFlags{}).add({
            {   35, CoordinateType::ScaleRatio },
            {   36, CoordinateType::Original },
        }),
    };
    return QtMocHelpers::metaObjectData<DRegionMonitor, qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DRegionMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DRegionMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DRegionMonitor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->buttonPress((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->buttonRelease((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->cursorMove((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->cursorEnter((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->cursorLeave((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->keyPress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->keyRelease((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->registerdFlagsChanged((*reinterpret_cast<std::add_pointer_t<RegisterdFlags>>(_a[1]))); break;
        case 8: _t->coordinateTypeChanged((*reinterpret_cast<std::add_pointer_t<enum CoordinateType>>(_a[1]))); break;
        case 9: _t->registerRegion(); break;
        case 10: _t->registerRegion((*reinterpret_cast<std::add_pointer_t<QRegion>>(_a[1]))); break;
        case 11: _t->unregisterRegion(); break;
        case 12: _t->setWatchedRegion((*reinterpret_cast<std::add_pointer_t<QRegion>>(_a[1]))); break;
        case 13: _t->setRegisterFlags((*reinterpret_cast<std::add_pointer_t<RegisterdFlags>>(_a[1]))); break;
        case 14: _t->setCoordinateType((*reinterpret_cast<std::add_pointer_t<enum CoordinateType>>(_a[1]))); break;
        case 15: _t->d_func()->_q_ButtonPress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 16: _t->d_func()->_q_ButtonRelease((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 17: _t->d_func()->_q_CursorMove((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 18: _t->d_func()->_q_CursorEnter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 19: _t->d_func()->_q_CursorLeave((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 20: _t->d_func()->_q_KeyPress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 21: _t->d_func()->_q_KeyRelease((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QPoint & , const int ) const>(_a, &DRegionMonitor::buttonPress, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QPoint & , const int ) const>(_a, &DRegionMonitor::buttonRelease, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QPoint & ) const>(_a, &DRegionMonitor::cursorMove, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QPoint & ) const>(_a, &DRegionMonitor::cursorEnter, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QPoint & ) const>(_a, &DRegionMonitor::cursorLeave, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QString & ) const>(_a, &DRegionMonitor::keyPress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(const QString & ) const>(_a, &DRegionMonitor::keyRelease, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(RegisterdFlags ) const>(_a, &DRegionMonitor::registerdFlagsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DRegionMonitor::*)(CoordinateType ) const>(_a, &DRegionMonitor::coordinateTypeChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum CoordinateType*>(_v) = _t->coordinateType(); break;
        case 1: *reinterpret_cast<RegisterdFlags*>(_v) = _t->registerFlags(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCoordinateType(*reinterpret_cast<enum CoordinateType*>(_v)); break;
        case 1: _t->setRegisterFlags(*reinterpret_cast<RegisterdFlags*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DRegionMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DRegionMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DRegionMonitorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DRegionMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DRegionMonitor::buttonPress(const QPoint & _t1, const int _t2)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Dtk::Gui::DRegionMonitor::buttonRelease(const QPoint & _t1, const int _t2)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Dtk::Gui::DRegionMonitor::cursorMove(const QPoint & _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Dtk::Gui::DRegionMonitor::cursorEnter(const QPoint & _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Dtk::Gui::DRegionMonitor::cursorLeave(const QPoint & _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Dtk::Gui::DRegionMonitor::keyPress(const QString & _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Dtk::Gui::DRegionMonitor::keyRelease(const QString & _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Dtk::Gui::DRegionMonitor::registerdFlagsChanged(RegisterdFlags _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Dtk::Gui::DRegionMonitor::coordinateTypeChanged(CoordinateType _t1)const
{
    QMetaObject::activate<void>(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
