/****************************************************************************
** Meta object code from reading C++ file 'dfontmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/util/dfontmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfontmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DFontManager_t {
    QByteArrayData data[35];
    char stringdata0[196];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DFontManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DFontManager_t qt_meta_stringdata_Dtk__Gui__DFontManager = {
    {
QT_MOC_LITERAL(0, 0, 22), // "Dtk::Gui::DFontManager"
QT_MOC_LITERAL(1, 23, 11), // "fontChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 13), // "fontPixelSize"
QT_MOC_LITERAL(4, 50, 8), // "SizeType"
QT_MOC_LITERAL(5, 59, 4), // "type"
QT_MOC_LITERAL(6, 64, 16), // "setFontPixelSize"
QT_MOC_LITERAL(7, 81, 4), // "size"
QT_MOC_LITERAL(8, 86, 4), // "font"
QT_MOC_LITERAL(9, 91, 3), // "get"
QT_MOC_LITERAL(10, 95, 9), // "pixelSize"
QT_MOC_LITERAL(11, 105, 4), // "base"
QT_MOC_LITERAL(12, 110, 2), // "t1"
QT_MOC_LITERAL(13, 113, 2), // "t2"
QT_MOC_LITERAL(14, 116, 2), // "t3"
QT_MOC_LITERAL(15, 119, 2), // "t4"
QT_MOC_LITERAL(16, 122, 2), // "t5"
QT_MOC_LITERAL(17, 125, 2), // "t6"
QT_MOC_LITERAL(18, 128, 2), // "t7"
QT_MOC_LITERAL(19, 131, 2), // "t8"
QT_MOC_LITERAL(20, 134, 2), // "t9"
QT_MOC_LITERAL(21, 137, 3), // "t10"
QT_MOC_LITERAL(22, 141, 8), // "baseFont"
QT_MOC_LITERAL(23, 150, 2), // "T1"
QT_MOC_LITERAL(24, 153, 2), // "T2"
QT_MOC_LITERAL(25, 156, 2), // "T3"
QT_MOC_LITERAL(26, 159, 2), // "T4"
QT_MOC_LITERAL(27, 162, 2), // "T5"
QT_MOC_LITERAL(28, 165, 2), // "T6"
QT_MOC_LITERAL(29, 168, 2), // "T7"
QT_MOC_LITERAL(30, 171, 2), // "T8"
QT_MOC_LITERAL(31, 174, 2), // "T9"
QT_MOC_LITERAL(32, 177, 3), // "T10"
QT_MOC_LITERAL(33, 181, 3), // "T11"
QT_MOC_LITERAL(34, 185, 10) // "NSizeTypes"

    },
    "Dtk::Gui::DFontManager\0fontChanged\0\0"
    "fontPixelSize\0SizeType\0type\0"
    "setFontPixelSize\0size\0font\0get\0pixelSize\0"
    "base\0t1\0t2\0t3\0t4\0t5\0t6\0t7\0t8\0t9\0t10\0"
    "baseFont\0T1\0T2\0T3\0T4\0T5\0T6\0T7\0T8\0T9\0"
    "T10\0T11\0NSizeTypes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DFontManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
      11,   56, // properties
       1,  100, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       3,    1,   40,    2, 0x02 /* Public */,
       6,    2,   43,    2, 0x02 /* Public */,
       3,    1,   48,    2, 0x02 /* Public */,
       9,    2,   51,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Int, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int,    5,    7,
    QMetaType::Int, QMetaType::QFont,    8,
    QMetaType::QFont, QMetaType::Int, QMetaType::QFont,   10,   11,

 // properties: name, type, flags
      12, QMetaType::QFont, 0x00495001,
      13, QMetaType::QFont, 0x00495001,
      14, QMetaType::QFont, 0x00495001,
      15, QMetaType::QFont, 0x00495001,
      16, QMetaType::QFont, 0x00495001,
      17, QMetaType::QFont, 0x00495001,
      18, QMetaType::QFont, 0x00495001,
      19, QMetaType::QFont, 0x00495001,
      20, QMetaType::QFont, 0x00495001,
      21, QMetaType::QFont, 0x00495001,
      22, QMetaType::QFont, 0x00495107,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // enums: name, alias, flags, count, data
       4,    4, 0x0,   12,  105,

 // enum data: key, value
      23, uint(Dtk::Gui::DFontManager::T1),
      24, uint(Dtk::Gui::DFontManager::T2),
      25, uint(Dtk::Gui::DFontManager::T3),
      26, uint(Dtk::Gui::DFontManager::T4),
      27, uint(Dtk::Gui::DFontManager::T5),
      28, uint(Dtk::Gui::DFontManager::T6),
      29, uint(Dtk::Gui::DFontManager::T7),
      30, uint(Dtk::Gui::DFontManager::T8),
      31, uint(Dtk::Gui::DFontManager::T9),
      32, uint(Dtk::Gui::DFontManager::T10),
      33, uint(Dtk::Gui::DFontManager::T11),
      34, uint(Dtk::Gui::DFontManager::NSizeTypes),

       0        // eod
};

void Dtk::Gui::DFontManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DFontManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fontChanged(); break;
        case 1: { int _r = _t->fontPixelSize((*reinterpret_cast< SizeType(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->setFontPixelSize((*reinterpret_cast< SizeType(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: { int _r = _t->fontPixelSize((*reinterpret_cast< const QFont(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 4: { QFont _r = _t->get((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QFont(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QFont*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DFontManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFontManager::fontChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DFontManager *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QFont*>(_v) = _t->t1(); break;
        case 1: *reinterpret_cast< QFont*>(_v) = _t->t2(); break;
        case 2: *reinterpret_cast< QFont*>(_v) = _t->t3(); break;
        case 3: *reinterpret_cast< QFont*>(_v) = _t->t4(); break;
        case 4: *reinterpret_cast< QFont*>(_v) = _t->t5(); break;
        case 5: *reinterpret_cast< QFont*>(_v) = _t->t6(); break;
        case 6: *reinterpret_cast< QFont*>(_v) = _t->t7(); break;
        case 7: *reinterpret_cast< QFont*>(_v) = _t->t8(); break;
        case 8: *reinterpret_cast< QFont*>(_v) = _t->t9(); break;
        case 9: *reinterpret_cast< QFont*>(_v) = _t->t10(); break;
        case 10: *reinterpret_cast< QFont*>(_v) = _t->baseFont(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DFontManager *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 10: _t->setBaseFont(*reinterpret_cast< QFont*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        DFontManager *_t = static_cast<DFontManager *>(_o);
        (void)_t;
        switch (_id) {
        case 10: _t->resetBaseFont(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DFontManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DFontManager.data,
    qt_meta_data_Dtk__Gui__DFontManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DFontManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DFontManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DFontManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DFontManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DFontManager::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
