/****************************************************************************
** Meta object code from reading C++ file 'dplatformhandle.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/kernel/dplatformhandle.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dplatformhandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DPlatformHandle::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DPlatformHandle",
        "frameMarginsChanged",
        "",
        "windowRadiusChanged",
        "borderWidthChanged",
        "borderColorChanged",
        "shadowRadiusChanged",
        "shadowOffsetChanged",
        "shadowColorChanged",
        "windowEffectChanged",
        "windowStartUpEffectChanged",
        "clipPathChanged",
        "frameMaskChanged",
        "translucentBackgroundChanged",
        "enableSystemResizeChanged",
        "enableSystemMoveChanged",
        "enableBlurWindowChanged",
        "autoInputMaskByClipPathChanged",
        "setWindowRadius",
        "windowRadius",
        "setBorderWidth",
        "borderWidth",
        "setBorderColor",
        "QColor",
        "borderColor",
        "setWindowEffect",
        "EffectScenes",
        "effectScene",
        "setWindowStartUpEffect",
        "EffectTypes",
        "effectType",
        "setShadowRadius",
        "shadowRadius",
        "setShadowOffset",
        "QPoint",
        "shadowOffset",
        "setShadowColor",
        "shadowColor",
        "setClipPath",
        "QPainterPath",
        "clipPath",
        "setFrameMask",
        "QRegion",
        "frameMask",
        "setTranslucentBackground",
        "translucentBackground",
        "setEnableSystemResize",
        "enableSystemResize",
        "setEnableSystemMove",
        "enableSystemMove",
        "setEnableBlurWindow",
        "enableBlurWindow",
        "setAutoInputMaskByClipPath",
        "autoInputMaskByClipPath",
        "windowEffect",
        "EffectScene",
        "windowStartUpEffect",
        "EffectType",
        "frameMargins",
        "QMargins",
        "realWindowId",
        "WId",
        "EffectNoRadius",
        "EffectNoShadow",
        "EffectNoBorder",
        "EffectNoStart",
        "EffectNoClose",
        "EffectNoMaximize",
        "EffectNoMinimize",
        "EffectNormal",
        "EffectCursor",
        "EffectTop",
        "EffectBottom",
        "EffectOut"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'frameMarginsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowRadiusChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderWidthChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowRadiusChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowOffsetChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowColorChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowEffectChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowStartUpEffectChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clipPathChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'frameMaskChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'translucentBackgroundChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableSystemResizeChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableSystemMoveChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableBlurWindowChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoInputMaskByClipPathChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setWindowRadius'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'setBorderWidth'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'setBorderColor'
        QtMocHelpers::SlotData<void(const QColor &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'setWindowEffect'
        QtMocHelpers::SlotData<void(EffectScenes)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'setWindowStartUpEffect'
        QtMocHelpers::SlotData<void(EffectTypes)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'setShadowRadius'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
        // Slot 'setShadowOffset'
        QtMocHelpers::SlotData<void(const QPoint &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'setShadowColor'
        QtMocHelpers::SlotData<void(const QColor &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 37 },
        }}),
        // Slot 'setClipPath'
        QtMocHelpers::SlotData<void(const QPainterPath &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
        // Slot 'setFrameMask'
        QtMocHelpers::SlotData<void(const QRegion &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Slot 'setTranslucentBackground'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 45 },
        }}),
        // Slot 'setEnableSystemResize'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 47 },
        }}),
        // Slot 'setEnableSystemMove'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 49 },
        }}),
        // Slot 'setEnableBlurWindow'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'setAutoInputMaskByClipPath'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 53 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'windowRadius'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'borderWidth'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(24, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'shadowRadius'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'shadowOffset'
        QtMocHelpers::PropertyData<QPoint>(35, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'shadowColor'
        QtMocHelpers::PropertyData<QColor>(37, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'windowEffect'
        QtMocHelpers::PropertyData<enum EffectScene>(54, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'windowStartUpEffect'
        QtMocHelpers::PropertyData<enum EffectType>(56, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'clipPath'
        QtMocHelpers::PropertyData<QPainterPath>(40, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'frameMask'
        QtMocHelpers::PropertyData<QRegion>(43, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'frameMargins'
        QtMocHelpers::PropertyData<QMargins>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'translucentBackground'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'enableSystemResize'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'enableSystemMove'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'enableBlurWindow'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'autoInputMaskByClipPath'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'realWindowId'
        QtMocHelpers::PropertyData<WId>(60, 0x80000000 | 61, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EffectScene'
        QtMocHelpers::EnumData<enum EffectScene>(55, 55, QMC::EnumFlags{}).add({
            {   62, EffectScene::EffectNoRadius },
            {   63, EffectScene::EffectNoShadow },
            {   64, EffectScene::EffectNoBorder },
            {   65, EffectScene::EffectNoStart },
            {   66, EffectScene::EffectNoClose },
            {   67, EffectScene::EffectNoMaximize },
            {   68, EffectScene::EffectNoMinimize },
        }),
        // enum 'EffectScenes'
        QtMocHelpers::EnumData<EffectScenes>(26, 55, QMC::EnumFlags{}).add({
            {   62, EffectScene::EffectNoRadius },
            {   63, EffectScene::EffectNoShadow },
            {   64, EffectScene::EffectNoBorder },
            {   65, EffectScene::EffectNoStart },
            {   66, EffectScene::EffectNoClose },
            {   67, EffectScene::EffectNoMaximize },
            {   68, EffectScene::EffectNoMinimize },
        }),
        // enum 'EffectType'
        QtMocHelpers::EnumData<enum EffectType>(57, 57, QMC::EnumFlags{}).add({
            {   69, EffectType::EffectNormal },
            {   70, EffectType::EffectCursor },
            {   71, EffectType::EffectTop },
            {   72, EffectType::EffectBottom },
            {   73, EffectType::EffectOut },
        }),
        // enum 'EffectTypes'
        QtMocHelpers::EnumData<EffectTypes>(29, 57, QMC::EnumFlags{}).add({
            {   69, EffectType::EffectNormal },
            {   70, EffectType::EffectCursor },
            {   71, EffectType::EffectTop },
            {   72, EffectType::EffectBottom },
            {   73, EffectType::EffectOut },
        }),
    };
    return QtMocHelpers::metaObjectData<DPlatformHandle, qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DPlatformHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DPlatformHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DPlatformHandle *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->frameMarginsChanged(); break;
        case 1: _t->windowRadiusChanged(); break;
        case 2: _t->borderWidthChanged(); break;
        case 3: _t->borderColorChanged(); break;
        case 4: _t->shadowRadiusChanged(); break;
        case 5: _t->shadowOffsetChanged(); break;
        case 6: _t->shadowColorChanged(); break;
        case 7: _t->windowEffectChanged(); break;
        case 8: _t->windowStartUpEffectChanged(); break;
        case 9: _t->clipPathChanged(); break;
        case 10: _t->frameMaskChanged(); break;
        case 11: _t->translucentBackgroundChanged(); break;
        case 12: _t->enableSystemResizeChanged(); break;
        case 13: _t->enableSystemMoveChanged(); break;
        case 14: _t->enableBlurWindowChanged(); break;
        case 15: _t->autoInputMaskByClipPathChanged(); break;
        case 16: _t->setWindowRadius((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->setBorderWidth((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->setBorderColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 19: _t->setWindowEffect((*reinterpret_cast<std::add_pointer_t<EffectScenes>>(_a[1]))); break;
        case 20: _t->setWindowStartUpEffect((*reinterpret_cast<std::add_pointer_t<EffectTypes>>(_a[1]))); break;
        case 21: _t->setShadowRadius((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->setShadowOffset((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 23: _t->setShadowColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 24: _t->setClipPath((*reinterpret_cast<std::add_pointer_t<QPainterPath>>(_a[1]))); break;
        case 25: _t->setFrameMask((*reinterpret_cast<std::add_pointer_t<QRegion>>(_a[1]))); break;
        case 26: _t->setTranslucentBackground((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->setEnableSystemResize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->setEnableSystemMove((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->setEnableBlurWindow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->setAutoInputMaskByClipPath((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRegion >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::frameMarginsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::windowRadiusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::borderWidthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::borderColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::shadowRadiusChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::shadowOffsetChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::shadowColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::windowEffectChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::windowStartUpEffectChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::clipPathChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::frameMaskChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::translucentBackgroundChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::enableSystemResizeChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::enableSystemMoveChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::enableBlurWindowChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformHandle::*)()>(_a, &DPlatformHandle::autoInputMaskByClipPathChanged, 15))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QRegion >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->windowRadius(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->borderWidth(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->shadowRadius(); break;
        case 4: *reinterpret_cast<QPoint*>(_v) = _t->shadowOffset(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->shadowColor(); break;
        case 6: *reinterpret_cast<enum EffectScene*>(_v) = _t->windowEffect(); break;
        case 7: *reinterpret_cast<enum EffectType*>(_v) = _t->windowStartUpEffect(); break;
        case 8: *reinterpret_cast<QPainterPath*>(_v) = _t->clipPath(); break;
        case 9: *reinterpret_cast<QRegion*>(_v) = _t->frameMask(); break;
        case 10: *reinterpret_cast<QMargins*>(_v) = _t->frameMargins(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->translucentBackground(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->enableSystemResize(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->enableSystemMove(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->enableBlurWindow(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->autoInputMaskByClipPath(); break;
        case 16: *reinterpret_cast<WId*>(_v) = _t->realWindowId(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWindowRadius(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setBorderWidth(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setShadowRadius(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setShadowOffset(*reinterpret_cast<QPoint*>(_v)); break;
        case 5: _t->setShadowColor(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->setWindowEffect(*reinterpret_cast<enum EffectScene*>(_v)); break;
        case 7: _t->setWindowStartUpEffect(*reinterpret_cast<enum EffectType*>(_v)); break;
        case 8: _t->setClipPath(*reinterpret_cast<QPainterPath*>(_v)); break;
        case 9: _t->setFrameMask(*reinterpret_cast<QRegion*>(_v)); break;
        case 11: _t->setTranslucentBackground(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setEnableSystemResize(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setEnableSystemMove(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setEnableBlurWindow(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setAutoInputMaskByClipPath(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DPlatformHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DPlatformHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui15DPlatformHandleE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DPlatformHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DPlatformHandle::frameMarginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DPlatformHandle::windowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DPlatformHandle::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Dtk::Gui::DPlatformHandle::borderColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Dtk::Gui::DPlatformHandle::shadowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Dtk::Gui::DPlatformHandle::shadowOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Dtk::Gui::DPlatformHandle::shadowColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Dtk::Gui::DPlatformHandle::windowEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Dtk::Gui::DPlatformHandle::windowStartUpEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Dtk::Gui::DPlatformHandle::clipPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Dtk::Gui::DPlatformHandle::frameMaskChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Dtk::Gui::DPlatformHandle::translucentBackgroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Dtk::Gui::DPlatformHandle::enableSystemResizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void Dtk::Gui::DPlatformHandle::enableSystemMoveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Dtk::Gui::DPlatformHandle::enableBlurWindowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Dtk::Gui::DPlatformHandle::autoInputMaskByClipPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
