/****************************************************************************
** Meta object code from reading C++ file 'orgdeepindtkpreference.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../orgdeepindtkpreference.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'orgdeepindtkpreference.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgDeepinDTKPreference::qt_create_metaobjectdata<qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgDeepinDTKPreference",
        "DConfigKeyList",
        "autoDisplayFeature;colorMode;defaultColorMode;disableInWindowBlur;"
        "enableDtkAnimations;featureUpdated;keyboardsearchDisabled;rules;sc"
        "rollBarPolicy;sizeMode;themeType;titlebarHeight;underlineShortcut",
        "DConfigFileName",
        "org.deepin.dtk.preference",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "autoDisplayFeatureChanged",
        "colorModeChanged",
        "defaultColorModeChanged",
        "disableInWindowBlurChanged",
        "enableDtkAnimationsChanged",
        "featureUpdatedChanged",
        "keyboardsearchDisabledChanged",
        "rulesChanged",
        "scrollBarPolicyChanged",
        "sizeModeChanged",
        "themeTypeChanged",
        "titlebarHeightChanged",
        "underlineShortcutChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "autoDisplayFeatureIsDefaultValue",
        "colorModeIsDefaultValue",
        "defaultColorModeIsDefaultValue",
        "disableInWindowBlurIsDefaultValue",
        "enableDtkAnimationsIsDefaultValue",
        "featureUpdatedIsDefaultValue",
        "keyboardsearchDisabledIsDefaultValue",
        "rulesIsDefaultValue",
        "scrollBarPolicyIsDefaultValue",
        "sizeModeIsDefaultValue",
        "themeTypeIsDefaultValue",
        "titlebarHeightIsDefaultValue",
        "underlineShortcutIsDefaultValue",
        "autoDisplayFeature",
        "colorMode",
        "defaultColorMode",
        "disableInWindowBlur",
        "enableDtkAnimations",
        "featureUpdated",
        "keyboardsearchDisabled",
        "rules",
        "scrollBarPolicy",
        "sizeMode",
        "themeType",
        "titlebarHeight",
        "underlineShortcut"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'autoDisplayFeatureChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorModeChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultColorModeChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disableInWindowBlurChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableDtkAnimationsChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'featureUpdatedChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyboardsearchDisabledChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rulesChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scrollBarPolicyChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sizeModeChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeTypeChanged'
        QtMocHelpers::SignalData<void()>(26, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titlebarHeightChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'underlineShortcutChanged'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(29, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(30, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(31, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(32, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(33, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(34, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'autoDisplayFeatureIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(35, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'colorModeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(36, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'defaultColorModeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(37, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'disableInWindowBlurIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(38, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'enableDtkAnimationsIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(39, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'featureUpdatedIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(40, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyboardsearchDisabledIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(41, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'rulesIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(42, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'scrollBarPolicyIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(43, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'sizeModeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(44, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'themeTypeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(45, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'titlebarHeightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(46, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'underlineShortcutIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(47, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'autoDisplayFeature'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'colorMode'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'defaultColorMode'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
        // property 'disableInWindowBlur'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'enableDtkAnimations'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7),
        // property 'featureUpdated'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 8),
        // property 'keyboardsearchDisabled'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 9),
        // property 'rules'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10),
        // property 'scrollBarPolicy'
        QtMocHelpers::PropertyData<qlonglong>(56, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 11),
        // property 'sizeMode'
        QtMocHelpers::PropertyData<qlonglong>(57, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 12),
        // property 'themeType'
        QtMocHelpers::PropertyData<qlonglong>(58, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 13),
        // property 'titlebarHeight'
        QtMocHelpers::PropertyData<qlonglong>(59, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 14),
        // property 'underlineShortcut'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 15),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<OrgDeepinDTKPreference, qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OrgDeepinDTKPreference::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t>.metaTypes,
    nullptr
} };

void OrgDeepinDTKPreference::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgDeepinDTKPreference *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->autoDisplayFeatureChanged(); break;
        case 4: _t->colorModeChanged(); break;
        case 5: _t->defaultColorModeChanged(); break;
        case 6: _t->disableInWindowBlurChanged(); break;
        case 7: _t->enableDtkAnimationsChanged(); break;
        case 8: _t->featureUpdatedChanged(); break;
        case 9: _t->keyboardsearchDisabledChanged(); break;
        case 10: _t->rulesChanged(); break;
        case 11: _t->scrollBarPolicyChanged(); break;
        case 12: _t->sizeModeChanged(); break;
        case 13: _t->themeTypeChanged(); break;
        case 14: _t->titlebarHeightChanged(); break;
        case 15: _t->underlineShortcutChanged(); break;
        case 16: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->autoDisplayFeatureIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->colorModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->defaultColorModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->disableInWindowBlurIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->enableDtkAnimationsIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->featureUpdatedIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->keyboardsearchDisabledIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->rulesIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->scrollBarPolicyIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->sizeModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->themeTypeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->titlebarHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->underlineShortcutIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)(Dtk::Core::DConfig * )>(_a, &OrgDeepinDTKPreference::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)(const QString & , const QVariant & )>(_a, &OrgDeepinDTKPreference::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::autoDisplayFeatureChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::colorModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::defaultColorModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::disableInWindowBlurChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::enableDtkAnimationsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::featureUpdatedChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::keyboardsearchDisabledChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::rulesChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::scrollBarPolicyChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::sizeModeChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::themeTypeChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::titlebarHeightChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinDTKPreference::*)()>(_a, &OrgDeepinDTKPreference::underlineShortcutChanged, 15))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->autoDisplayFeature(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->colorMode(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->defaultColorMode(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->disableInWindowBlur(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->enableDtkAnimations(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->featureUpdated(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->keyboardsearchDisabled(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->rules(); break;
        case 8: *reinterpret_cast<qlonglong*>(_v) = _t->scrollBarPolicy(); break;
        case 9: *reinterpret_cast<qlonglong*>(_v) = _t->sizeMode(); break;
        case 10: *reinterpret_cast<qlonglong*>(_v) = _t->themeType(); break;
        case 11: *reinterpret_cast<qlonglong*>(_v) = _t->titlebarHeight(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->underlineShortcut(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAutoDisplayFeature(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setColorMode(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setDefaultColorMode(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setDisableInWindowBlur(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setEnableDtkAnimations(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFeatureUpdated(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setKeyboardsearchDisabled(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setRules(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setScrollBarPolicy(*reinterpret_cast<qlonglong*>(_v)); break;
        case 9: _t->setSizeMode(*reinterpret_cast<qlonglong*>(_v)); break;
        case 10: _t->setThemeType(*reinterpret_cast<qlonglong*>(_v)); break;
        case 11: _t->setTitlebarHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 12: _t->setUnderlineShortcut(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetAutoDisplayFeature(); break;
        case 1: _t->resetColorMode(); break;
        case 2: _t->resetDefaultColorMode(); break;
        case 3: _t->resetDisableInWindowBlur(); break;
        case 4: _t->resetEnableDtkAnimations(); break;
        case 5: _t->resetFeatureUpdated(); break;
        case 6: _t->resetKeyboardsearchDisabled(); break;
        case 7: _t->resetRules(); break;
        case 8: _t->resetScrollBarPolicy(); break;
        case 9: _t->resetSizeMode(); break;
        case 10: _t->resetThemeType(); break;
        case 11: _t->resetTitlebarHeight(); break;
        case 12: _t->resetUnderlineShortcut(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgDeepinDTKPreference::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgDeepinDTKPreference::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22OrgDeepinDTKPreferenceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OrgDeepinDTKPreference::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void OrgDeepinDTKPreference::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgDeepinDTKPreference::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgDeepinDTKPreference::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OrgDeepinDTKPreference::autoDisplayFeatureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OrgDeepinDTKPreference::colorModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OrgDeepinDTKPreference::defaultColorModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OrgDeepinDTKPreference::disableInWindowBlurChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OrgDeepinDTKPreference::enableDtkAnimationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OrgDeepinDTKPreference::featureUpdatedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OrgDeepinDTKPreference::keyboardsearchDisabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void OrgDeepinDTKPreference::rulesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OrgDeepinDTKPreference::scrollBarPolicyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void OrgDeepinDTKPreference::sizeModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void OrgDeepinDTKPreference::themeTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void OrgDeepinDTKPreference::titlebarHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void OrgDeepinDTKPreference::underlineShortcutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
