/****************************************************************************
** Meta object code from reading C++ file 'dplatformtheme.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/kernel/dplatformtheme.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dplatformtheme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DPlatformTheme_t {
    QByteArrayData data[145];
    char stringdata0[2089];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DPlatformTheme_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DPlatformTheme_t qt_meta_stringdata_Dtk__Gui__DPlatformTheme = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Dtk::Gui::DPlatformTheme"
QT_MOC_LITERAL(1, 25, 22), // "cursorBlinkTimeChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 15), // "cursorBlinkTime"
QT_MOC_LITERAL(4, 65, 25), // "cursorBlinkTimeoutChanged"
QT_MOC_LITERAL(5, 91, 18), // "cursorBlinkTimeout"
QT_MOC_LITERAL(6, 110, 18), // "cursorBlinkChanged"
QT_MOC_LITERAL(7, 129, 11), // "cursorBlink"
QT_MOC_LITERAL(8, 141, 26), // "doubleClickDistanceChanged"
QT_MOC_LITERAL(9, 168, 19), // "doubleClickDistance"
QT_MOC_LITERAL(10, 188, 22), // "doubleClickTimeChanged"
QT_MOC_LITERAL(11, 211, 15), // "doubleClickTime"
QT_MOC_LITERAL(12, 227, 23), // "dndDragThresholdChanged"
QT_MOC_LITERAL(13, 251, 16), // "dndDragThreshold"
QT_MOC_LITERAL(14, 268, 16), // "themeNameChanged"
QT_MOC_LITERAL(15, 285, 9), // "themeName"
QT_MOC_LITERAL(16, 295, 20), // "iconThemeNameChanged"
QT_MOC_LITERAL(17, 316, 13), // "iconThemeName"
QT_MOC_LITERAL(18, 330, 21), // "soundThemeNameChanged"
QT_MOC_LITERAL(19, 352, 14), // "soundThemeName"
QT_MOC_LITERAL(20, 367, 15), // "fontNameChanged"
QT_MOC_LITERAL(21, 383, 8), // "fontName"
QT_MOC_LITERAL(22, 392, 19), // "monoFontNameChanged"
QT_MOC_LITERAL(23, 412, 12), // "monoFontName"
QT_MOC_LITERAL(24, 425, 20), // "fontPointSizeChanged"
QT_MOC_LITERAL(25, 446, 13), // "fontPointSize"
QT_MOC_LITERAL(26, 460, 18), // "gtkFontNameChanged"
QT_MOC_LITERAL(27, 479, 18), // "activeColorChanged"
QT_MOC_LITERAL(28, 498, 11), // "activeColor"
QT_MOC_LITERAL(29, 510, 22), // "darkActiveColorChanged"
QT_MOC_LITERAL(30, 533, 14), // "paletteChanged"
QT_MOC_LITERAL(31, 548, 8), // "DPalette"
QT_MOC_LITERAL(32, 557, 7), // "palette"
QT_MOC_LITERAL(33, 565, 13), // "windowChanged"
QT_MOC_LITERAL(34, 579, 6), // "window"
QT_MOC_LITERAL(35, 586, 17), // "windowTextChanged"
QT_MOC_LITERAL(36, 604, 10), // "windowText"
QT_MOC_LITERAL(37, 615, 11), // "baseChanged"
QT_MOC_LITERAL(38, 627, 4), // "base"
QT_MOC_LITERAL(39, 632, 20), // "alternateBaseChanged"
QT_MOC_LITERAL(40, 653, 13), // "alternateBase"
QT_MOC_LITERAL(41, 667, 18), // "toolTipBaseChanged"
QT_MOC_LITERAL(42, 686, 11), // "toolTipBase"
QT_MOC_LITERAL(43, 698, 18), // "toolTipTextChanged"
QT_MOC_LITERAL(44, 717, 11), // "toolTipText"
QT_MOC_LITERAL(45, 729, 11), // "textChanged"
QT_MOC_LITERAL(46, 741, 4), // "text"
QT_MOC_LITERAL(47, 746, 13), // "buttonChanged"
QT_MOC_LITERAL(48, 760, 6), // "button"
QT_MOC_LITERAL(49, 767, 17), // "buttonTextChanged"
QT_MOC_LITERAL(50, 785, 10), // "buttonText"
QT_MOC_LITERAL(51, 796, 17), // "brightTextChanged"
QT_MOC_LITERAL(52, 814, 10), // "brightText"
QT_MOC_LITERAL(53, 825, 12), // "lightChanged"
QT_MOC_LITERAL(54, 838, 5), // "light"
QT_MOC_LITERAL(55, 844, 15), // "midlightChanged"
QT_MOC_LITERAL(56, 860, 8), // "midlight"
QT_MOC_LITERAL(57, 869, 11), // "darkChanged"
QT_MOC_LITERAL(58, 881, 4), // "dark"
QT_MOC_LITERAL(59, 886, 10), // "midChanged"
QT_MOC_LITERAL(60, 897, 3), // "mid"
QT_MOC_LITERAL(61, 901, 13), // "shadowChanged"
QT_MOC_LITERAL(62, 915, 6), // "shadow"
QT_MOC_LITERAL(63, 922, 16), // "highlightChanged"
QT_MOC_LITERAL(64, 939, 9), // "highlight"
QT_MOC_LITERAL(65, 949, 22), // "highlightedTextChanged"
QT_MOC_LITERAL(66, 972, 15), // "highlightedText"
QT_MOC_LITERAL(67, 988, 11), // "linkChanged"
QT_MOC_LITERAL(68, 1000, 4), // "link"
QT_MOC_LITERAL(69, 1005, 18), // "linkVisitedChanged"
QT_MOC_LITERAL(70, 1024, 11), // "linkVisited"
QT_MOC_LITERAL(71, 1036, 21), // "itemBackgroundChanged"
QT_MOC_LITERAL(72, 1058, 14), // "itemBackground"
QT_MOC_LITERAL(73, 1073, 16), // "textTitleChanged"
QT_MOC_LITERAL(74, 1090, 9), // "textTitle"
QT_MOC_LITERAL(75, 1100, 15), // "textTipsChanged"
QT_MOC_LITERAL(76, 1116, 8), // "textTips"
QT_MOC_LITERAL(77, 1125, 18), // "textWarningChanged"
QT_MOC_LITERAL(78, 1144, 11), // "textWarning"
QT_MOC_LITERAL(79, 1156, 17), // "textLivelyChanged"
QT_MOC_LITERAL(80, 1174, 10), // "textLively"
QT_MOC_LITERAL(81, 1185, 18), // "lightLivelyChanged"
QT_MOC_LITERAL(82, 1204, 11), // "lightLively"
QT_MOC_LITERAL(83, 1216, 17), // "darkLivelyChanged"
QT_MOC_LITERAL(84, 1234, 10), // "darkLively"
QT_MOC_LITERAL(85, 1245, 18), // "frameBorderChanged"
QT_MOC_LITERAL(86, 1264, 11), // "frameBorder"
QT_MOC_LITERAL(87, 1276, 18), // "dotsPerInchChanged"
QT_MOC_LITERAL(88, 1295, 6), // "screen"
QT_MOC_LITERAL(89, 1302, 3), // "dpi"
QT_MOC_LITERAL(90, 1306, 19), // "windowRadiusChanged"
QT_MOC_LITERAL(91, 1326, 1), // "r"
QT_MOC_LITERAL(92, 1328, 15), // "sizeModeChanged"
QT_MOC_LITERAL(93, 1344, 8), // "sizeMode"
QT_MOC_LITERAL(94, 1353, 22), // "scrollBarPolicyChanged"
QT_MOC_LITERAL(95, 1376, 15), // "scrollBarPolicy"
QT_MOC_LITERAL(96, 1392, 18), // "setCursorBlinkTime"
QT_MOC_LITERAL(97, 1411, 21), // "setCursorBlinkTimeout"
QT_MOC_LITERAL(98, 1433, 14), // "setCursorBlink"
QT_MOC_LITERAL(99, 1448, 22), // "setDoubleClickDistance"
QT_MOC_LITERAL(100, 1471, 18), // "setDoubleClickTime"
QT_MOC_LITERAL(101, 1490, 19), // "setDndDragThreshold"
QT_MOC_LITERAL(102, 1510, 12), // "setThemeName"
QT_MOC_LITERAL(103, 1523, 16), // "setIconThemeName"
QT_MOC_LITERAL(104, 1540, 17), // "setSoundThemeName"
QT_MOC_LITERAL(105, 1558, 11), // "setFontName"
QT_MOC_LITERAL(106, 1570, 15), // "setMonoFontName"
QT_MOC_LITERAL(107, 1586, 16), // "setFontPointSize"
QT_MOC_LITERAL(108, 1603, 14), // "setGtkFontName"
QT_MOC_LITERAL(109, 1618, 14), // "setActiveColor"
QT_MOC_LITERAL(110, 1633, 18), // "setDarkActiveColor"
QT_MOC_LITERAL(111, 1652, 9), // "setWindow"
QT_MOC_LITERAL(112, 1662, 13), // "setWindowText"
QT_MOC_LITERAL(113, 1676, 7), // "setBase"
QT_MOC_LITERAL(114, 1684, 16), // "setAlternateBase"
QT_MOC_LITERAL(115, 1701, 14), // "setToolTipBase"
QT_MOC_LITERAL(116, 1716, 14), // "setToolTipText"
QT_MOC_LITERAL(117, 1731, 7), // "setText"
QT_MOC_LITERAL(118, 1739, 9), // "setButton"
QT_MOC_LITERAL(119, 1749, 13), // "setButtonText"
QT_MOC_LITERAL(120, 1763, 13), // "setBrightText"
QT_MOC_LITERAL(121, 1777, 8), // "setLight"
QT_MOC_LITERAL(122, 1786, 11), // "setMidlight"
QT_MOC_LITERAL(123, 1798, 7), // "setDark"
QT_MOC_LITERAL(124, 1806, 6), // "setMid"
QT_MOC_LITERAL(125, 1813, 9), // "setShadow"
QT_MOC_LITERAL(126, 1823, 12), // "setHighlight"
QT_MOC_LITERAL(127, 1836, 18), // "setHighlightedText"
QT_MOC_LITERAL(128, 1855, 7), // "setLink"
QT_MOC_LITERAL(129, 1863, 14), // "setLinkVisited"
QT_MOC_LITERAL(130, 1878, 17), // "setItemBackground"
QT_MOC_LITERAL(131, 1896, 12), // "setTextTitle"
QT_MOC_LITERAL(132, 1909, 11), // "setTextTips"
QT_MOC_LITERAL(133, 1921, 14), // "setTextWarning"
QT_MOC_LITERAL(134, 1936, 13), // "setTextLively"
QT_MOC_LITERAL(135, 1950, 14), // "setLightLively"
QT_MOC_LITERAL(136, 1965, 13), // "setDarkLively"
QT_MOC_LITERAL(137, 1979, 14), // "setFrameBorder"
QT_MOC_LITERAL(138, 1994, 11), // "dotsPerInch"
QT_MOC_LITERAL(139, 2006, 10), // "screenName"
QT_MOC_LITERAL(140, 2017, 14), // "setDotsPerInch"
QT_MOC_LITERAL(141, 2032, 15), // "setWindowRadius"
QT_MOC_LITERAL(142, 2048, 12), // "windowRadius"
QT_MOC_LITERAL(143, 2061, 11), // "gtkFontName"
QT_MOC_LITERAL(144, 2073, 15) // "darkActiveColor"

    },
    "Dtk::Gui::DPlatformTheme\0"
    "cursorBlinkTimeChanged\0\0cursorBlinkTime\0"
    "cursorBlinkTimeoutChanged\0cursorBlinkTimeout\0"
    "cursorBlinkChanged\0cursorBlink\0"
    "doubleClickDistanceChanged\0"
    "doubleClickDistance\0doubleClickTimeChanged\0"
    "doubleClickTime\0dndDragThresholdChanged\0"
    "dndDragThreshold\0themeNameChanged\0"
    "themeName\0iconThemeNameChanged\0"
    "iconThemeName\0soundThemeNameChanged\0"
    "soundThemeName\0fontNameChanged\0fontName\0"
    "monoFontNameChanged\0monoFontName\0"
    "fontPointSizeChanged\0fontPointSize\0"
    "gtkFontNameChanged\0activeColorChanged\0"
    "activeColor\0darkActiveColorChanged\0"
    "paletteChanged\0DPalette\0palette\0"
    "windowChanged\0window\0windowTextChanged\0"
    "windowText\0baseChanged\0base\0"
    "alternateBaseChanged\0alternateBase\0"
    "toolTipBaseChanged\0toolTipBase\0"
    "toolTipTextChanged\0toolTipText\0"
    "textChanged\0text\0buttonChanged\0button\0"
    "buttonTextChanged\0buttonText\0"
    "brightTextChanged\0brightText\0lightChanged\0"
    "light\0midlightChanged\0midlight\0"
    "darkChanged\0dark\0midChanged\0mid\0"
    "shadowChanged\0shadow\0highlightChanged\0"
    "highlight\0highlightedTextChanged\0"
    "highlightedText\0linkChanged\0link\0"
    "linkVisitedChanged\0linkVisited\0"
    "itemBackgroundChanged\0itemBackground\0"
    "textTitleChanged\0textTitle\0textTipsChanged\0"
    "textTips\0textWarningChanged\0textWarning\0"
    "textLivelyChanged\0textLively\0"
    "lightLivelyChanged\0lightLively\0"
    "darkLivelyChanged\0darkLively\0"
    "frameBorderChanged\0frameBorder\0"
    "dotsPerInchChanged\0screen\0dpi\0"
    "windowRadiusChanged\0r\0sizeModeChanged\0"
    "sizeMode\0scrollBarPolicyChanged\0"
    "scrollBarPolicy\0setCursorBlinkTime\0"
    "setCursorBlinkTimeout\0setCursorBlink\0"
    "setDoubleClickDistance\0setDoubleClickTime\0"
    "setDndDragThreshold\0setThemeName\0"
    "setIconThemeName\0setSoundThemeName\0"
    "setFontName\0setMonoFontName\0"
    "setFontPointSize\0setGtkFontName\0"
    "setActiveColor\0setDarkActiveColor\0"
    "setWindow\0setWindowText\0setBase\0"
    "setAlternateBase\0setToolTipBase\0"
    "setToolTipText\0setText\0setButton\0"
    "setButtonText\0setBrightText\0setLight\0"
    "setMidlight\0setDark\0setMid\0setShadow\0"
    "setHighlight\0setHighlightedText\0setLink\0"
    "setLinkVisited\0setItemBackground\0"
    "setTextTitle\0setTextTips\0setTextWarning\0"
    "setTextLively\0setLightLively\0setDarkLively\0"
    "setFrameBorder\0dotsPerInch\0screenName\0"
    "setDotsPerInch\0setWindowRadius\0"
    "windowRadius\0gtkFontName\0darkActiveColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DPlatformTheme[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      93,   14, // methods
      45,  760, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      47,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  479,    2, 0x06 /* Public */,
       4,    1,  482,    2, 0x06 /* Public */,
       6,    1,  485,    2, 0x06 /* Public */,
       8,    1,  488,    2, 0x06 /* Public */,
      10,    1,  491,    2, 0x06 /* Public */,
      12,    1,  494,    2, 0x06 /* Public */,
      14,    1,  497,    2, 0x06 /* Public */,
      16,    1,  500,    2, 0x06 /* Public */,
      18,    1,  503,    2, 0x06 /* Public */,
      20,    1,  506,    2, 0x06 /* Public */,
      22,    1,  509,    2, 0x06 /* Public */,
      24,    1,  512,    2, 0x06 /* Public */,
      26,    1,  515,    2, 0x06 /* Public */,
      27,    1,  518,    2, 0x06 /* Public */,
      29,    1,  521,    2, 0x06 /* Public */,
      30,    1,  524,    2, 0x06 /* Public */,
      33,    1,  527,    2, 0x06 /* Public */,
      35,    1,  530,    2, 0x06 /* Public */,
      37,    1,  533,    2, 0x06 /* Public */,
      39,    1,  536,    2, 0x06 /* Public */,
      41,    1,  539,    2, 0x06 /* Public */,
      43,    1,  542,    2, 0x06 /* Public */,
      45,    1,  545,    2, 0x06 /* Public */,
      47,    1,  548,    2, 0x06 /* Public */,
      49,    1,  551,    2, 0x06 /* Public */,
      51,    1,  554,    2, 0x06 /* Public */,
      53,    1,  557,    2, 0x06 /* Public */,
      55,    1,  560,    2, 0x06 /* Public */,
      57,    1,  563,    2, 0x06 /* Public */,
      59,    1,  566,    2, 0x06 /* Public */,
      61,    1,  569,    2, 0x06 /* Public */,
      63,    1,  572,    2, 0x06 /* Public */,
      65,    1,  575,    2, 0x06 /* Public */,
      67,    1,  578,    2, 0x06 /* Public */,
      69,    1,  581,    2, 0x06 /* Public */,
      71,    1,  584,    2, 0x06 /* Public */,
      73,    1,  587,    2, 0x06 /* Public */,
      75,    1,  590,    2, 0x06 /* Public */,
      77,    1,  593,    2, 0x06 /* Public */,
      79,    1,  596,    2, 0x06 /* Public */,
      81,    1,  599,    2, 0x06 /* Public */,
      83,    1,  602,    2, 0x06 /* Public */,
      85,    1,  605,    2, 0x06 /* Public */,
      87,    2,  608,    2, 0x06 /* Public */,
      90,    1,  613,    2, 0x06 /* Public */,
      92,    1,  616,    2, 0x06 /* Public */,
      94,    1,  619,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      96,    1,  622,    2, 0x0a /* Public */,
      97,    1,  625,    2, 0x0a /* Public */,
      98,    1,  628,    2, 0x0a /* Public */,
      99,    1,  631,    2, 0x0a /* Public */,
     100,    1,  634,    2, 0x0a /* Public */,
     101,    1,  637,    2, 0x0a /* Public */,
     102,    1,  640,    2, 0x0a /* Public */,
     103,    1,  643,    2, 0x0a /* Public */,
     104,    1,  646,    2, 0x0a /* Public */,
     105,    1,  649,    2, 0x0a /* Public */,
     106,    1,  652,    2, 0x0a /* Public */,
     107,    1,  655,    2, 0x0a /* Public */,
     108,    1,  658,    2, 0x0a /* Public */,
     109,    1,  661,    2, 0x0a /* Public */,
     110,    1,  664,    2, 0x0a /* Public */,
     111,    1,  667,    2, 0x0a /* Public */,
     112,    1,  670,    2, 0x0a /* Public */,
     113,    1,  673,    2, 0x0a /* Public */,
     114,    1,  676,    2, 0x0a /* Public */,
     115,    1,  679,    2, 0x0a /* Public */,
     116,    1,  682,    2, 0x0a /* Public */,
     117,    1,  685,    2, 0x0a /* Public */,
     118,    1,  688,    2, 0x0a /* Public */,
     119,    1,  691,    2, 0x0a /* Public */,
     120,    1,  694,    2, 0x0a /* Public */,
     121,    1,  697,    2, 0x0a /* Public */,
     122,    1,  700,    2, 0x0a /* Public */,
     123,    1,  703,    2, 0x0a /* Public */,
     124,    1,  706,    2, 0x0a /* Public */,
     125,    1,  709,    2, 0x0a /* Public */,
     126,    1,  712,    2, 0x0a /* Public */,
     127,    1,  715,    2, 0x0a /* Public */,
     128,    1,  718,    2, 0x0a /* Public */,
     129,    1,  721,    2, 0x0a /* Public */,
     130,    1,  724,    2, 0x0a /* Public */,
     131,    1,  727,    2, 0x0a /* Public */,
     132,    1,  730,    2, 0x0a /* Public */,
     133,    1,  733,    2, 0x0a /* Public */,
     134,    1,  736,    2, 0x0a /* Public */,
     135,    1,  739,    2, 0x0a /* Public */,
     136,    1,  742,    2, 0x0a /* Public */,
     137,    1,  745,    2, 0x0a /* Public */,
     138,    1,  748,    2, 0x0a /* Public */,
     138,    0,  751,    2, 0x2a /* Public | MethodCloned */,
     140,    2,  752,    2, 0x0a /* Public */,
     141,    1,  757,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::QByteArray,   15,
    QMetaType::Void, QMetaType::QByteArray,   17,
    QMetaType::Void, QMetaType::QByteArray,   19,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::QByteArray,   23,
    QMetaType::Void, QMetaType::QReal,   25,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::QColor,   28,
    QMetaType::Void, QMetaType::QColor,   28,
    QMetaType::Void, 0x80000000 | 31,   32,
    QMetaType::Void, QMetaType::QColor,   34,
    QMetaType::Void, QMetaType::QColor,   36,
    QMetaType::Void, QMetaType::QColor,   38,
    QMetaType::Void, QMetaType::QColor,   40,
    QMetaType::Void, QMetaType::QColor,   42,
    QMetaType::Void, QMetaType::QColor,   44,
    QMetaType::Void, QMetaType::QColor,   46,
    QMetaType::Void, QMetaType::QColor,   48,
    QMetaType::Void, QMetaType::QColor,   50,
    QMetaType::Void, QMetaType::QColor,   52,
    QMetaType::Void, QMetaType::QColor,   54,
    QMetaType::Void, QMetaType::QColor,   56,
    QMetaType::Void, QMetaType::QColor,   58,
    QMetaType::Void, QMetaType::QColor,   60,
    QMetaType::Void, QMetaType::QColor,   62,
    QMetaType::Void, QMetaType::QColor,   64,
    QMetaType::Void, QMetaType::QColor,   66,
    QMetaType::Void, QMetaType::QColor,   68,
    QMetaType::Void, QMetaType::QColor,   70,
    QMetaType::Void, QMetaType::QColor,   72,
    QMetaType::Void, QMetaType::QColor,   74,
    QMetaType::Void, QMetaType::QColor,   76,
    QMetaType::Void, QMetaType::QColor,   78,
    QMetaType::Void, QMetaType::QColor,   80,
    QMetaType::Void, QMetaType::QColor,   82,
    QMetaType::Void, QMetaType::QColor,   84,
    QMetaType::Void, QMetaType::QColor,   86,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   88,   89,
    QMetaType::Void, QMetaType::Int,   91,
    QMetaType::Void, QMetaType::Int,   93,
    QMetaType::Void, QMetaType::Int,   95,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::QByteArray,   15,
    QMetaType::Void, QMetaType::QByteArray,   17,
    QMetaType::Void, QMetaType::QByteArray,   19,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::QByteArray,   23,
    QMetaType::Void, QMetaType::QReal,   25,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::QColor,   28,
    QMetaType::Void, QMetaType::QColor,   28,
    QMetaType::Void, QMetaType::QColor,   34,
    QMetaType::Void, QMetaType::QColor,   36,
    QMetaType::Void, QMetaType::QColor,   38,
    QMetaType::Void, QMetaType::QColor,   40,
    QMetaType::Void, QMetaType::QColor,   42,
    QMetaType::Void, QMetaType::QColor,   44,
    QMetaType::Void, QMetaType::QColor,   46,
    QMetaType::Void, QMetaType::QColor,   48,
    QMetaType::Void, QMetaType::QColor,   50,
    QMetaType::Void, QMetaType::QColor,   52,
    QMetaType::Void, QMetaType::QColor,   54,
    QMetaType::Void, QMetaType::QColor,   56,
    QMetaType::Void, QMetaType::QColor,   58,
    QMetaType::Void, QMetaType::QColor,   60,
    QMetaType::Void, QMetaType::QColor,   62,
    QMetaType::Void, QMetaType::QColor,   64,
    QMetaType::Void, QMetaType::QColor,   66,
    QMetaType::Void, QMetaType::QColor,   68,
    QMetaType::Void, QMetaType::QColor,   70,
    QMetaType::Void, QMetaType::QColor,   72,
    QMetaType::Void, QMetaType::QColor,   74,
    QMetaType::Void, QMetaType::QColor,   76,
    QMetaType::Void, QMetaType::QColor,   78,
    QMetaType::Void, QMetaType::QColor,   80,
    QMetaType::Void, QMetaType::QColor,   82,
    QMetaType::Void, QMetaType::QColor,   84,
    QMetaType::Void, QMetaType::QColor,   86,
    QMetaType::Int, QMetaType::QString,  139,
    QMetaType::Int,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,  139,   89,
    QMetaType::Void, QMetaType::Int,  142,

 // properties: name, type, flags
       3, QMetaType::Int, 0x00495103,
       5, QMetaType::Int, 0x00495103,
       7, QMetaType::Bool, 0x00495103,
       9, QMetaType::Int, 0x00495103,
      11, QMetaType::Int, 0x00495103,
      13, QMetaType::Int, 0x00495103,
     142, QMetaType::Int, 0x00495103,
      15, QMetaType::QByteArray, 0x00495103,
      17, QMetaType::QByteArray, 0x00495103,
      19, QMetaType::QByteArray, 0x00495103,
      21, QMetaType::QByteArray, 0x00495103,
      23, QMetaType::QByteArray, 0x00495103,
      25, QMetaType::QReal, 0x00495103,
     143, QMetaType::QByteArray, 0x00495103,
      28, QMetaType::QColor, 0x00495103,
     144, QMetaType::QColor, 0x00495103,
      34, QMetaType::QColor, 0x00495103,
      36, QMetaType::QColor, 0x00495103,
      38, QMetaType::QColor, 0x00495103,
      40, QMetaType::QColor, 0x00495103,
      42, QMetaType::QColor, 0x00495103,
      44, QMetaType::QColor, 0x00495103,
      46, QMetaType::QColor, 0x00495103,
      48, QMetaType::QColor, 0x00495103,
      50, QMetaType::QColor, 0x00495103,
      52, QMetaType::QColor, 0x00495103,
      54, QMetaType::QColor, 0x00495103,
      56, QMetaType::QColor, 0x00495103,
      58, QMetaType::QColor, 0x00495103,
      60, QMetaType::QColor, 0x00495103,
      62, QMetaType::QColor, 0x00495103,
      64, QMetaType::QColor, 0x00495103,
      66, QMetaType::QColor, 0x00495103,
      68, QMetaType::QColor, 0x00495103,
      70, QMetaType::QColor, 0x00495103,
      72, QMetaType::QColor, 0x00495103,
      74, QMetaType::QColor, 0x00495103,
      76, QMetaType::QColor, 0x00495103,
      78, QMetaType::QColor, 0x00495103,
      80, QMetaType::QColor, 0x00495103,
      82, QMetaType::QColor, 0x00495103,
      84, QMetaType::QColor, 0x00495103,
      86, QMetaType::QColor, 0x00495103,
      93, QMetaType::Int, 0x00495001,
      95, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
      44,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      16,
      17,
      18,
      19,
      20,
      21,
      22,
      23,
      24,
      25,
      26,
      27,
      28,
      29,
      30,
      31,
      32,
      33,
      34,
      35,
      36,
      37,
      38,
      39,
      40,
      41,
      42,
      45,
      46,

       0        // eod
};

void Dtk::Gui::DPlatformTheme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DPlatformTheme *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cursorBlinkTimeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->cursorBlinkTimeoutChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->cursorBlinkChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->doubleClickDistanceChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->doubleClickTimeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->dndDragThresholdChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->themeNameChanged((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 7: _t->iconThemeNameChanged((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 8: _t->soundThemeNameChanged((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 9: _t->fontNameChanged((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 10: _t->monoFontNameChanged((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 11: _t->fontPointSizeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 12: _t->gtkFontNameChanged((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 13: _t->activeColorChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 14: _t->darkActiveColorChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 15: _t->paletteChanged((*reinterpret_cast< DPalette(*)>(_a[1]))); break;
        case 16: _t->windowChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 17: _t->windowTextChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 18: _t->baseChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 19: _t->alternateBaseChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 20: _t->toolTipBaseChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 21: _t->toolTipTextChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 22: _t->textChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 23: _t->buttonChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 24: _t->buttonTextChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 25: _t->brightTextChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 26: _t->lightChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 27: _t->midlightChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 28: _t->darkChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 29: _t->midChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 30: _t->shadowChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 31: _t->highlightChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 32: _t->highlightedTextChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 33: _t->linkChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 34: _t->linkVisitedChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 35: _t->itemBackgroundChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 36: _t->textTitleChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 37: _t->textTipsChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 38: _t->textWarningChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 39: _t->textLivelyChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 40: _t->lightLivelyChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 41: _t->darkLivelyChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 42: _t->frameBorderChanged((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 43: _t->dotsPerInchChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 44: _t->windowRadiusChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 45: _t->sizeModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 46: _t->scrollBarPolicyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 47: _t->setCursorBlinkTime((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 48: _t->setCursorBlinkTimeout((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 49: _t->setCursorBlink((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 50: _t->setDoubleClickDistance((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 51: _t->setDoubleClickTime((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 52: _t->setDndDragThreshold((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 53: _t->setThemeName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 54: _t->setIconThemeName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 55: _t->setSoundThemeName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 56: _t->setFontName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 57: _t->setMonoFontName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 58: _t->setFontPointSize((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 59: _t->setGtkFontName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 60: _t->setActiveColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 61: _t->setDarkActiveColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 62: _t->setWindow((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 63: _t->setWindowText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 64: _t->setBase((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 65: _t->setAlternateBase((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 66: _t->setToolTipBase((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 67: _t->setToolTipText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 68: _t->setText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 69: _t->setButton((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 70: _t->setButtonText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 71: _t->setBrightText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 72: _t->setLight((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 73: _t->setMidlight((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 74: _t->setDark((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 75: _t->setMid((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 76: _t->setShadow((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 77: _t->setHighlight((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 78: _t->setHighlightedText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 79: _t->setLink((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 80: _t->setLinkVisited((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 81: _t->setItemBackground((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 82: _t->setTextTitle((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 83: _t->setTextTips((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 84: _t->setTextWarning((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 85: _t->setTextLively((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 86: _t->setLightLively((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 87: _t->setDarkLively((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 88: _t->setFrameBorder((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 89: { int _r = _t->dotsPerInch((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 90: { int _r = _t->dotsPerInch();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 91: _t->setDotsPerInch((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 92: _t->setWindowRadius((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::cursorBlinkTimeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::cursorBlinkTimeoutChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::cursorBlinkChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::doubleClickDistanceChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::doubleClickTimeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::dndDragThresholdChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::themeNameChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::iconThemeNameChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::soundThemeNameChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::fontNameChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::monoFontNameChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::fontPointSizeChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QByteArray );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::gtkFontNameChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::activeColorChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::darkActiveColorChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(DPalette );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::paletteChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::windowChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::windowTextChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::baseChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::alternateBaseChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::toolTipBaseChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::toolTipTextChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::textChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::buttonChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::buttonTextChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::brightTextChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::lightChanged)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::midlightChanged)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::darkChanged)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::midChanged)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::shadowChanged)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::highlightChanged)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::highlightedTextChanged)) {
                *result = 32;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::linkChanged)) {
                *result = 33;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::linkVisitedChanged)) {
                *result = 34;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::itemBackgroundChanged)) {
                *result = 35;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::textTitleChanged)) {
                *result = 36;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::textTipsChanged)) {
                *result = 37;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::textWarningChanged)) {
                *result = 38;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::textLivelyChanged)) {
                *result = 39;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::lightLivelyChanged)) {
                *result = 40;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::darkLivelyChanged)) {
                *result = 41;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(QColor );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::frameBorderChanged)) {
                *result = 42;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::dotsPerInchChanged)) {
                *result = 43;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::windowRadiusChanged)) {
                *result = 44;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::sizeModeChanged)) {
                *result = 45;
                return;
            }
        }
        {
            using _t = void (DPlatformTheme::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformTheme::scrollBarPolicyChanged)) {
                *result = 46;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DPlatformTheme *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->cursorBlinkTime(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->cursorBlinkTimeout(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->cursorBlink(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->doubleClickDistance(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->doubleClickTime(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->dndDragThreshold(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->windowRadius(); break;
        case 7: *reinterpret_cast< QByteArray*>(_v) = _t->themeName(); break;
        case 8: *reinterpret_cast< QByteArray*>(_v) = _t->iconThemeName(); break;
        case 9: *reinterpret_cast< QByteArray*>(_v) = _t->soundThemeName(); break;
        case 10: *reinterpret_cast< QByteArray*>(_v) = _t->fontName(); break;
        case 11: *reinterpret_cast< QByteArray*>(_v) = _t->monoFontName(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->fontPointSize(); break;
        case 13: *reinterpret_cast< QByteArray*>(_v) = _t->gtkFontName(); break;
        case 14: *reinterpret_cast< QColor*>(_v) = _t->activeColor(); break;
        case 15: *reinterpret_cast< QColor*>(_v) = _t->darkActiveColor(); break;
        case 16: *reinterpret_cast< QColor*>(_v) = _t->window(); break;
        case 17: *reinterpret_cast< QColor*>(_v) = _t->windowText(); break;
        case 18: *reinterpret_cast< QColor*>(_v) = _t->base(); break;
        case 19: *reinterpret_cast< QColor*>(_v) = _t->alternateBase(); break;
        case 20: *reinterpret_cast< QColor*>(_v) = _t->toolTipBase(); break;
        case 21: *reinterpret_cast< QColor*>(_v) = _t->toolTipText(); break;
        case 22: *reinterpret_cast< QColor*>(_v) = _t->text(); break;
        case 23: *reinterpret_cast< QColor*>(_v) = _t->button(); break;
        case 24: *reinterpret_cast< QColor*>(_v) = _t->buttonText(); break;
        case 25: *reinterpret_cast< QColor*>(_v) = _t->brightText(); break;
        case 26: *reinterpret_cast< QColor*>(_v) = _t->light(); break;
        case 27: *reinterpret_cast< QColor*>(_v) = _t->midlight(); break;
        case 28: *reinterpret_cast< QColor*>(_v) = _t->dark(); break;
        case 29: *reinterpret_cast< QColor*>(_v) = _t->mid(); break;
        case 30: *reinterpret_cast< QColor*>(_v) = _t->shadow(); break;
        case 31: *reinterpret_cast< QColor*>(_v) = _t->highlight(); break;
        case 32: *reinterpret_cast< QColor*>(_v) = _t->highlightedText(); break;
        case 33: *reinterpret_cast< QColor*>(_v) = _t->link(); break;
        case 34: *reinterpret_cast< QColor*>(_v) = _t->linkVisited(); break;
        case 35: *reinterpret_cast< QColor*>(_v) = _t->itemBackground(); break;
        case 36: *reinterpret_cast< QColor*>(_v) = _t->textTitle(); break;
        case 37: *reinterpret_cast< QColor*>(_v) = _t->textTips(); break;
        case 38: *reinterpret_cast< QColor*>(_v) = _t->textWarning(); break;
        case 39: *reinterpret_cast< QColor*>(_v) = _t->textLively(); break;
        case 40: *reinterpret_cast< QColor*>(_v) = _t->lightLively(); break;
        case 41: *reinterpret_cast< QColor*>(_v) = _t->darkLively(); break;
        case 42: *reinterpret_cast< QColor*>(_v) = _t->frameBorder(); break;
        case 43: *reinterpret_cast< int*>(_v) = _t->sizeMode(); break;
        case 44: *reinterpret_cast< int*>(_v) = _t->scrollBarPolicy(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DPlatformTheme *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCursorBlinkTime(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setCursorBlinkTimeout(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setCursorBlink(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setDoubleClickDistance(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setDoubleClickTime(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setDndDragThreshold(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setWindowRadius(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setThemeName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 8: _t->setIconThemeName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 9: _t->setSoundThemeName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 10: _t->setFontName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 11: _t->setMonoFontName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 12: _t->setFontPointSize(*reinterpret_cast< qreal*>(_v)); break;
        case 13: _t->setGtkFontName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 14: _t->setActiveColor(*reinterpret_cast< QColor*>(_v)); break;
        case 15: _t->setDarkActiveColor(*reinterpret_cast< QColor*>(_v)); break;
        case 16: _t->setWindow(*reinterpret_cast< QColor*>(_v)); break;
        case 17: _t->setWindowText(*reinterpret_cast< QColor*>(_v)); break;
        case 18: _t->setBase(*reinterpret_cast< QColor*>(_v)); break;
        case 19: _t->setAlternateBase(*reinterpret_cast< QColor*>(_v)); break;
        case 20: _t->setToolTipBase(*reinterpret_cast< QColor*>(_v)); break;
        case 21: _t->setToolTipText(*reinterpret_cast< QColor*>(_v)); break;
        case 22: _t->setText(*reinterpret_cast< QColor*>(_v)); break;
        case 23: _t->setButton(*reinterpret_cast< QColor*>(_v)); break;
        case 24: _t->setButtonText(*reinterpret_cast< QColor*>(_v)); break;
        case 25: _t->setBrightText(*reinterpret_cast< QColor*>(_v)); break;
        case 26: _t->setLight(*reinterpret_cast< QColor*>(_v)); break;
        case 27: _t->setMidlight(*reinterpret_cast< QColor*>(_v)); break;
        case 28: _t->setDark(*reinterpret_cast< QColor*>(_v)); break;
        case 29: _t->setMid(*reinterpret_cast< QColor*>(_v)); break;
        case 30: _t->setShadow(*reinterpret_cast< QColor*>(_v)); break;
        case 31: _t->setHighlight(*reinterpret_cast< QColor*>(_v)); break;
        case 32: _t->setHighlightedText(*reinterpret_cast< QColor*>(_v)); break;
        case 33: _t->setLink(*reinterpret_cast< QColor*>(_v)); break;
        case 34: _t->setLinkVisited(*reinterpret_cast< QColor*>(_v)); break;
        case 35: _t->setItemBackground(*reinterpret_cast< QColor*>(_v)); break;
        case 36: _t->setTextTitle(*reinterpret_cast< QColor*>(_v)); break;
        case 37: _t->setTextTips(*reinterpret_cast< QColor*>(_v)); break;
        case 38: _t->setTextWarning(*reinterpret_cast< QColor*>(_v)); break;
        case 39: _t->setTextLively(*reinterpret_cast< QColor*>(_v)); break;
        case 40: _t->setLightLively(*reinterpret_cast< QColor*>(_v)); break;
        case 41: _t->setDarkLively(*reinterpret_cast< QColor*>(_v)); break;
        case 42: _t->setFrameBorder(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DPlatformTheme::staticMetaObject = { {
    QMetaObject::SuperData::link<DNativeSettings::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DPlatformTheme.data,
    qt_meta_data_Dtk__Gui__DPlatformTheme,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DPlatformTheme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DPlatformTheme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DPlatformTheme.stringdata0))
        return static_cast<void*>(this);
    return DNativeSettings::qt_metacast(_clname);
}

int Dtk::Gui::DPlatformTheme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DNativeSettings::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 93)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 93;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 93)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 93;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 45;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 45;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 45;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 45;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 45;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DPlatformTheme::cursorBlinkTimeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Dtk::Gui::DPlatformTheme::cursorBlinkTimeoutChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Dtk::Gui::DPlatformTheme::cursorBlinkChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Dtk::Gui::DPlatformTheme::doubleClickDistanceChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Dtk::Gui::DPlatformTheme::doubleClickTimeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Dtk::Gui::DPlatformTheme::dndDragThresholdChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Dtk::Gui::DPlatformTheme::themeNameChanged(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Dtk::Gui::DPlatformTheme::iconThemeNameChanged(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Dtk::Gui::DPlatformTheme::soundThemeNameChanged(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Dtk::Gui::DPlatformTheme::fontNameChanged(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void Dtk::Gui::DPlatformTheme::monoFontNameChanged(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void Dtk::Gui::DPlatformTheme::fontPointSizeChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void Dtk::Gui::DPlatformTheme::gtkFontNameChanged(QByteArray _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Dtk::Gui::DPlatformTheme::activeColorChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void Dtk::Gui::DPlatformTheme::darkActiveColorChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void Dtk::Gui::DPlatformTheme::paletteChanged(DPalette _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void Dtk::Gui::DPlatformTheme::windowChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void Dtk::Gui::DPlatformTheme::windowTextChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void Dtk::Gui::DPlatformTheme::baseChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void Dtk::Gui::DPlatformTheme::alternateBaseChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void Dtk::Gui::DPlatformTheme::toolTipBaseChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void Dtk::Gui::DPlatformTheme::toolTipTextChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void Dtk::Gui::DPlatformTheme::textChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 22, _a);
}

// SIGNAL 23
void Dtk::Gui::DPlatformTheme::buttonChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void Dtk::Gui::DPlatformTheme::buttonTextChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void Dtk::Gui::DPlatformTheme::brightTextChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void Dtk::Gui::DPlatformTheme::lightChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void Dtk::Gui::DPlatformTheme::midlightChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 27, _a);
}

// SIGNAL 28
void Dtk::Gui::DPlatformTheme::darkChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 28, _a);
}

// SIGNAL 29
void Dtk::Gui::DPlatformTheme::midChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 29, _a);
}

// SIGNAL 30
void Dtk::Gui::DPlatformTheme::shadowChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 30, _a);
}

// SIGNAL 31
void Dtk::Gui::DPlatformTheme::highlightChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 31, _a);
}

// SIGNAL 32
void Dtk::Gui::DPlatformTheme::highlightedTextChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 32, _a);
}

// SIGNAL 33
void Dtk::Gui::DPlatformTheme::linkChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 33, _a);
}

// SIGNAL 34
void Dtk::Gui::DPlatformTheme::linkVisitedChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 34, _a);
}

// SIGNAL 35
void Dtk::Gui::DPlatformTheme::itemBackgroundChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 35, _a);
}

// SIGNAL 36
void Dtk::Gui::DPlatformTheme::textTitleChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 36, _a);
}

// SIGNAL 37
void Dtk::Gui::DPlatformTheme::textTipsChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 37, _a);
}

// SIGNAL 38
void Dtk::Gui::DPlatformTheme::textWarningChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 38, _a);
}

// SIGNAL 39
void Dtk::Gui::DPlatformTheme::textLivelyChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 39, _a);
}

// SIGNAL 40
void Dtk::Gui::DPlatformTheme::lightLivelyChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 40, _a);
}

// SIGNAL 41
void Dtk::Gui::DPlatformTheme::darkLivelyChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 41, _a);
}

// SIGNAL 42
void Dtk::Gui::DPlatformTheme::frameBorderChanged(QColor _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 42, _a);
}

// SIGNAL 43
void Dtk::Gui::DPlatformTheme::dotsPerInchChanged(const QString & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 43, _a);
}

// SIGNAL 44
void Dtk::Gui::DPlatformTheme::windowRadiusChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 44, _a);
}

// SIGNAL 45
void Dtk::Gui::DPlatformTheme::sizeModeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 45, _a);
}

// SIGNAL 46
void Dtk::Gui::DPlatformTheme::scrollBarPolicyChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 46, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
