// This file is autogenerated. Changes will be overwritten.
#include "EJRQKI7XPS/moc_orgdeepindtkpreference.cpp"
#include "IQY2TTQ4MX/moc_dfiledrag.cpp"
#include "IQY2TTQ4MX/moc_dfiledragclient.cpp"
#include "IQY2TTQ4MX/moc_dfiledragserver.cpp"
#include "WHM3T56WKK/moc_dforeignwindow.cpp"
#include "WHM3T56WKK/moc_dnativesettings.cpp"
#include "WHM3T56WKK/moc_dpalette.cpp"
#include "WHM3T56WKK/moc_dplatformhandle.cpp"
#include "WHM3T56WKK/moc_dwindowmanagerhelper.cpp"
#include "HMJSVG7GSJ/moc_ddciiconpalette.cpp"
#include "HMJSVG7GSJ/moc_dfontmanager.cpp"
#include "HMJSVG7GSJ/moc_dimagehandler.cpp"
#include "HMJSVG7GSJ/moc_dtaskbarcontrol.cpp"
#include "HMJSVG7GSJ/moc_dthumbnailprovider.cpp"
#include "KS3UX4TNHX/moc_xeventmonitor_interface.cpp"
#include "ZYXQ4BDYH7/moc_dtreelandplatforminterface.cpp"
#include "ZYXQ4BDYH7/moc_dtreelandplatformwindowinterface.cpp"
#include "ZYXQ4BDYH7/moc_personalizationwaylandclientextension.cpp"
#include "O3DQLMQXTT/moc_dxcbplatformwindowinterface.cpp"
