/****************************************************************************
** Meta object code from reading C++ file 'dwindowmanagerhelper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/kernel/dwindowmanagerhelper.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dwindowmanagerhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DWindowManagerHelper::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DWindowManagerHelper",
        "windowManagerChanged",
        "",
        "hasBlurWindowChanged",
        "hasCompositeChanged",
        "hasNoTitlebarChanged",
        "hasWallpaperEffectChanged",
        "windowListChanged",
        "windowMotifWMHintsChanged",
        "winId",
        "hasBlurWindow",
        "hasComposite",
        "hasNoTitlebar",
        "hasWallpaperEffect",
        "MotifFunction",
        "FUNC_RESIZE",
        "FUNC_MOVE",
        "FUNC_MINIMIZE",
        "FUNC_MAXIMIZE",
        "FUNC_CLOSE",
        "FUNC_ALL",
        "MotifFunctions",
        "MotifDecoration",
        "DECOR_BORDER",
        "DECOR_RESIZEH",
        "DECOR_TITLE",
        "DECOR_MENU",
        "DECOR_MINIMIZE",
        "DECOR_MAXIMIZE",
        "DECOR_ALL",
        "MotifDecorations",
        "WMName",
        "OtherWM",
        "DeepinWM",
        "KWinWM",
        "WmWindowType",
        "UnknowWindowType",
        "NormalType",
        "DesktopType",
        "DockType",
        "ToolbarType",
        "MenuType",
        "UtilityType",
        "SplashType",
        "DialogType",
        "DropDownMenuType",
        "PopupMenuType",
        "TooltipType",
        "NotificationType",
        "ComboType",
        "DndType",
        "KdeOverrideType",
        "WmWindowTypes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'windowManagerChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasBlurWindowChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasCompositeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasNoTitlebarChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasWallpaperEffectChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowListChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowMotifWMHintsChanged'
        QtMocHelpers::SignalData<void(quint32)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hasBlurWindow'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'hasComposite'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'hasNoTitlebar'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'hasWallpaperEffect'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MotifFunction'
        QtMocHelpers::EnumData<enum MotifFunction>(14, 14, QMC::EnumFlags{}).add({
            {   15, MotifFunction::FUNC_RESIZE },
            {   16, MotifFunction::FUNC_MOVE },
            {   17, MotifFunction::FUNC_MINIMIZE },
            {   18, MotifFunction::FUNC_MAXIMIZE },
            {   19, MotifFunction::FUNC_CLOSE },
            {   20, MotifFunction::FUNC_ALL },
        }),
        // flag 'MotifFunctions'
        QtMocHelpers::EnumData<MotifFunctions>(21, 14, QMC::EnumIsFlag).add({
            {   15, MotifFunction::FUNC_RESIZE },
            {   16, MotifFunction::FUNC_MOVE },
            {   17, MotifFunction::FUNC_MINIMIZE },
            {   18, MotifFunction::FUNC_MAXIMIZE },
            {   19, MotifFunction::FUNC_CLOSE },
            {   20, MotifFunction::FUNC_ALL },
        }),
        // enum 'MotifDecoration'
        QtMocHelpers::EnumData<enum MotifDecoration>(22, 22, QMC::EnumFlags{}).add({
            {   23, MotifDecoration::DECOR_BORDER },
            {   24, MotifDecoration::DECOR_RESIZEH },
            {   25, MotifDecoration::DECOR_TITLE },
            {   26, MotifDecoration::DECOR_MENU },
            {   27, MotifDecoration::DECOR_MINIMIZE },
            {   28, MotifDecoration::DECOR_MAXIMIZE },
            {   29, MotifDecoration::DECOR_ALL },
        }),
        // flag 'MotifDecorations'
        QtMocHelpers::EnumData<MotifDecorations>(30, 22, QMC::EnumIsFlag).add({
            {   23, MotifDecoration::DECOR_BORDER },
            {   24, MotifDecoration::DECOR_RESIZEH },
            {   25, MotifDecoration::DECOR_TITLE },
            {   26, MotifDecoration::DECOR_MENU },
            {   27, MotifDecoration::DECOR_MINIMIZE },
            {   28, MotifDecoration::DECOR_MAXIMIZE },
            {   29, MotifDecoration::DECOR_ALL },
        }),
        // enum 'WMName'
        QtMocHelpers::EnumData<enum WMName>(31, 31, QMC::EnumFlags{}).add({
            {   32, WMName::OtherWM },
            {   33, WMName::DeepinWM },
            {   34, WMName::KWinWM },
        }),
        // enum 'WmWindowType'
        QtMocHelpers::EnumData<enum WmWindowType>(35, 35, QMC::EnumFlags{}).add({
            {   36, WmWindowType::UnknowWindowType },
            {   37, WmWindowType::NormalType },
            {   38, WmWindowType::DesktopType },
            {   39, WmWindowType::DockType },
            {   40, WmWindowType::ToolbarType },
            {   41, WmWindowType::MenuType },
            {   42, WmWindowType::UtilityType },
            {   43, WmWindowType::SplashType },
            {   44, WmWindowType::DialogType },
            {   45, WmWindowType::DropDownMenuType },
            {   46, WmWindowType::PopupMenuType },
            {   47, WmWindowType::TooltipType },
            {   48, WmWindowType::NotificationType },
            {   49, WmWindowType::ComboType },
            {   50, WmWindowType::DndType },
            {   51, WmWindowType::KdeOverrideType },
        }),
        // flag 'WmWindowTypes'
        QtMocHelpers::EnumData<WmWindowTypes>(52, 35, QMC::EnumIsFlag).add({
            {   36, WmWindowType::UnknowWindowType },
            {   37, WmWindowType::NormalType },
            {   38, WmWindowType::DesktopType },
            {   39, WmWindowType::DockType },
            {   40, WmWindowType::ToolbarType },
            {   41, WmWindowType::MenuType },
            {   42, WmWindowType::UtilityType },
            {   43, WmWindowType::SplashType },
            {   44, WmWindowType::DialogType },
            {   45, WmWindowType::DropDownMenuType },
            {   46, WmWindowType::PopupMenuType },
            {   47, WmWindowType::TooltipType },
            {   48, WmWindowType::NotificationType },
            {   49, WmWindowType::ComboType },
            {   50, WmWindowType::DndType },
            {   51, WmWindowType::KdeOverrideType },
        }),
    };
    return QtMocHelpers::metaObjectData<DWindowManagerHelper, qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DWindowManagerHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DWindowManagerHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DWindowManagerHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->windowManagerChanged(); break;
        case 1: _t->hasBlurWindowChanged(); break;
        case 2: _t->hasCompositeChanged(); break;
        case 3: _t->hasNoTitlebarChanged(); break;
        case 4: _t->hasWallpaperEffectChanged(); break;
        case 5: _t->windowListChanged(); break;
        case 6: _t->windowMotifWMHintsChanged((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)()>(_a, &DWindowManagerHelper::windowManagerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)()>(_a, &DWindowManagerHelper::hasBlurWindowChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)()>(_a, &DWindowManagerHelper::hasCompositeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)()>(_a, &DWindowManagerHelper::hasNoTitlebarChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)()>(_a, &DWindowManagerHelper::hasWallpaperEffectChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)()>(_a, &DWindowManagerHelper::windowListChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DWindowManagerHelper::*)(quint32 )>(_a, &DWindowManagerHelper::windowMotifWMHintsChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hasBlurWindow(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->hasComposite(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasNoTitlebar(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasWallpaperEffect(); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DWindowManagerHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DWindowManagerHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui20DWindowManagerHelperE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DWindowManagerHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DWindowManagerHelper::windowManagerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DWindowManagerHelper::hasBlurWindowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DWindowManagerHelper::hasCompositeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Dtk::Gui::DWindowManagerHelper::hasNoTitlebarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Dtk::Gui::DWindowManagerHelper::hasWallpaperEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Dtk::Gui::DWindowManagerHelper::windowListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Dtk::Gui::DWindowManagerHelper::windowMotifWMHintsChanged(quint32 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
