/****************************************************************************
** Meta object code from reading C++ file 'xeventmonitor_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/dbus/xeventmonitor_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xeventmonitor_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ComDeepinApiXEventMonitorInterface::qt_create_metaobjectdata<qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ComDeepinApiXEventMonitorInterface",
        "ButtonPress",
        "",
        "in0",
        "in1",
        "in2",
        "in3",
        "ButtonRelease",
        "CancelAllArea",
        "CursorInto",
        "CursorMove",
        "CursorOut",
        "KeyPress",
        "KeyRelease",
        "RegisterArea",
        "QDBusPendingReply<QString>",
        "in4",
        "RegisterAreas",
        "AreaList",
        "RegisterFullScreen",
        "UnregisterArea",
        "QDBusPendingReply<bool>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ButtonPress'
        QtMocHelpers::SignalData<void(int, int, int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'ButtonRelease'
        QtMocHelpers::SignalData<void(int, int, int, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'CancelAllArea'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CursorInto'
        QtMocHelpers::SignalData<void(int, int, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'CursorMove'
        QtMocHelpers::SignalData<void(int, int, const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'CursorOut'
        QtMocHelpers::SignalData<void(int, int, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'KeyPress'
        QtMocHelpers::SignalData<void(const QString &, int, int, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'KeyRelease'
        QtMocHelpers::SignalData<void(const QString &, int, int, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'RegisterArea'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(int, int, int, int, int)>(14, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
            { QMetaType::Int, 16 },
        }}),
        // Slot 'RegisterAreas'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(AreaList, int)>(17, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 18, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'RegisterFullScreen'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(19, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'UnregisterArea'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(20, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComDeepinApiXEventMonitorInterface, qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ComDeepinApiXEventMonitorInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t>.metaTypes,
    nullptr
} };

void ComDeepinApiXEventMonitorInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComDeepinApiXEventMonitorInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ButtonPress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 1: _t->ButtonRelease((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 2: _t->CancelAllArea(); break;
        case 3: _t->CursorInto((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 4: _t->CursorMove((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 5: _t->CursorOut((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->KeyPress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 7: _t->KeyRelease((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 8: { QDBusPendingReply<QString> _r = _t->RegisterArea((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->RegisterAreas((*reinterpret_cast<std::add_pointer_t<AreaList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QString> _r = _t->RegisterFullScreen();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<bool> _r = _t->UnregisterArea((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(int , int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::ButtonPress, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(int , int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::ButtonRelease, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)()>(_a, &ComDeepinApiXEventMonitorInterface::CancelAllArea, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::CursorInto, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::CursorMove, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::CursorOut, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(const QString & , int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::KeyPress, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComDeepinApiXEventMonitorInterface::*)(const QString & , int , int , const QString & )>(_a, &ComDeepinApiXEventMonitorInterface::KeyRelease, 7))
            return;
    }
}

const QMetaObject *ComDeepinApiXEventMonitorInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComDeepinApiXEventMonitorInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34ComDeepinApiXEventMonitorInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int ComDeepinApiXEventMonitorInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ComDeepinApiXEventMonitorInterface::ButtonPress(int _t1, int _t2, int _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void ComDeepinApiXEventMonitorInterface::ButtonRelease(int _t1, int _t2, int _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void ComDeepinApiXEventMonitorInterface::CancelAllArea()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ComDeepinApiXEventMonitorInterface::CursorInto(int _t1, int _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void ComDeepinApiXEventMonitorInterface::CursorMove(int _t1, int _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void ComDeepinApiXEventMonitorInterface::CursorOut(int _t1, int _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void ComDeepinApiXEventMonitorInterface::KeyPress(const QString & _t1, int _t2, int _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 7
void ComDeepinApiXEventMonitorInterface::KeyRelease(const QString & _t1, int _t2, int _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
