/****************************************************************************
** Meta object code from reading C++ file 'dxcbplatforminterface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/plugins/platform/xcb/dxcbplatforminterface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dxcbplatforminterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DXCBPlatformInterface_t {
    QByteArrayData data[93];
    char stringdata0[1191];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DXCBPlatformInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DXCBPlatformInterface_t qt_meta_stringdata_Dtk__Gui__DXCBPlatformInterface = {
    {
QT_MOC_LITERAL(0, 0, 31), // "Dtk::Gui::DXCBPlatformInterface"
QT_MOC_LITERAL(1, 32, 18), // "setCursorBlinkTime"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 15), // "cursorBlinkTime"
QT_MOC_LITERAL(4, 68, 21), // "setCursorBlinkTimeout"
QT_MOC_LITERAL(5, 90, 18), // "cursorBlinkTimeout"
QT_MOC_LITERAL(6, 109, 14), // "setCursorBlink"
QT_MOC_LITERAL(7, 124, 11), // "cursorBlink"
QT_MOC_LITERAL(8, 136, 22), // "setDoubleClickDistance"
QT_MOC_LITERAL(9, 159, 19), // "doubleClickDistance"
QT_MOC_LITERAL(10, 179, 18), // "setDoubleClickTime"
QT_MOC_LITERAL(11, 198, 15), // "doubleClickTime"
QT_MOC_LITERAL(12, 214, 19), // "setDndDragThreshold"
QT_MOC_LITERAL(13, 234, 16), // "dndDragThreshold"
QT_MOC_LITERAL(14, 251, 12), // "setThemeName"
QT_MOC_LITERAL(15, 264, 9), // "themeName"
QT_MOC_LITERAL(16, 274, 16), // "setIconThemeName"
QT_MOC_LITERAL(17, 291, 13), // "iconThemeName"
QT_MOC_LITERAL(18, 305, 17), // "setSoundThemeName"
QT_MOC_LITERAL(19, 323, 14), // "soundThemeName"
QT_MOC_LITERAL(20, 338, 11), // "setFontName"
QT_MOC_LITERAL(21, 350, 8), // "fontName"
QT_MOC_LITERAL(22, 359, 15), // "setMonoFontName"
QT_MOC_LITERAL(23, 375, 12), // "monoFontName"
QT_MOC_LITERAL(24, 388, 16), // "setFontPointSize"
QT_MOC_LITERAL(25, 405, 13), // "fontPointSize"
QT_MOC_LITERAL(26, 419, 14), // "setGtkFontName"
QT_MOC_LITERAL(27, 434, 14), // "setActiveColor"
QT_MOC_LITERAL(28, 449, 11), // "activeColor"
QT_MOC_LITERAL(29, 461, 18), // "setDarkActiveColor"
QT_MOC_LITERAL(30, 480, 9), // "setWindow"
QT_MOC_LITERAL(31, 490, 6), // "window"
QT_MOC_LITERAL(32, 497, 13), // "setWindowText"
QT_MOC_LITERAL(33, 511, 10), // "windowText"
QT_MOC_LITERAL(34, 522, 7), // "setBase"
QT_MOC_LITERAL(35, 530, 4), // "base"
QT_MOC_LITERAL(36, 535, 16), // "setAlternateBase"
QT_MOC_LITERAL(37, 552, 13), // "alternateBase"
QT_MOC_LITERAL(38, 566, 14), // "setToolTipBase"
QT_MOC_LITERAL(39, 581, 11), // "toolTipBase"
QT_MOC_LITERAL(40, 593, 14), // "setToolTipText"
QT_MOC_LITERAL(41, 608, 11), // "toolTipText"
QT_MOC_LITERAL(42, 620, 7), // "setText"
QT_MOC_LITERAL(43, 628, 4), // "text"
QT_MOC_LITERAL(44, 633, 9), // "setButton"
QT_MOC_LITERAL(45, 643, 6), // "button"
QT_MOC_LITERAL(46, 650, 13), // "setButtonText"
QT_MOC_LITERAL(47, 664, 10), // "buttonText"
QT_MOC_LITERAL(48, 675, 13), // "setBrightText"
QT_MOC_LITERAL(49, 689, 10), // "brightText"
QT_MOC_LITERAL(50, 700, 8), // "setLight"
QT_MOC_LITERAL(51, 709, 5), // "light"
QT_MOC_LITERAL(52, 715, 11), // "setMidlight"
QT_MOC_LITERAL(53, 727, 8), // "midlight"
QT_MOC_LITERAL(54, 736, 7), // "setDark"
QT_MOC_LITERAL(55, 744, 4), // "dark"
QT_MOC_LITERAL(56, 749, 6), // "setMid"
QT_MOC_LITERAL(57, 756, 3), // "mid"
QT_MOC_LITERAL(58, 760, 9), // "setShadow"
QT_MOC_LITERAL(59, 770, 6), // "shadow"
QT_MOC_LITERAL(60, 777, 12), // "setHighlight"
QT_MOC_LITERAL(61, 790, 9), // "highlight"
QT_MOC_LITERAL(62, 800, 18), // "setHighlightedText"
QT_MOC_LITERAL(63, 819, 15), // "highlightedText"
QT_MOC_LITERAL(64, 835, 7), // "setLink"
QT_MOC_LITERAL(65, 843, 4), // "link"
QT_MOC_LITERAL(66, 848, 14), // "setLinkVisited"
QT_MOC_LITERAL(67, 863, 11), // "linkVisited"
QT_MOC_LITERAL(68, 875, 17), // "setItemBackground"
QT_MOC_LITERAL(69, 893, 14), // "itemBackground"
QT_MOC_LITERAL(70, 908, 12), // "setTextTitle"
QT_MOC_LITERAL(71, 921, 9), // "textTitle"
QT_MOC_LITERAL(72, 931, 11), // "setTextTips"
QT_MOC_LITERAL(73, 943, 8), // "textTips"
QT_MOC_LITERAL(74, 952, 14), // "setTextWarning"
QT_MOC_LITERAL(75, 967, 11), // "textWarning"
QT_MOC_LITERAL(76, 979, 13), // "setTextLively"
QT_MOC_LITERAL(77, 993, 10), // "textLively"
QT_MOC_LITERAL(78, 1004, 14), // "setLightLively"
QT_MOC_LITERAL(79, 1019, 11), // "lightLively"
QT_MOC_LITERAL(80, 1031, 13), // "setDarkLively"
QT_MOC_LITERAL(81, 1045, 10), // "darkLively"
QT_MOC_LITERAL(82, 1056, 14), // "setFrameBorder"
QT_MOC_LITERAL(83, 1071, 11), // "frameBorder"
QT_MOC_LITERAL(84, 1083, 11), // "dotsPerInch"
QT_MOC_LITERAL(85, 1095, 10), // "screenName"
QT_MOC_LITERAL(86, 1106, 14), // "setDotsPerInch"
QT_MOC_LITERAL(87, 1121, 3), // "dpi"
QT_MOC_LITERAL(88, 1125, 15), // "setWindowRadius"
QT_MOC_LITERAL(89, 1141, 12), // "windowRadius"
QT_MOC_LITERAL(90, 1154, 25), // "_q_onThemePropertyChanged"
QT_MOC_LITERAL(91, 1180, 4), // "name"
QT_MOC_LITERAL(92, 1185, 5) // "value"

    },
    "Dtk::Gui::DXCBPlatformInterface\0"
    "setCursorBlinkTime\0\0cursorBlinkTime\0"
    "setCursorBlinkTimeout\0cursorBlinkTimeout\0"
    "setCursorBlink\0cursorBlink\0"
    "setDoubleClickDistance\0doubleClickDistance\0"
    "setDoubleClickTime\0doubleClickTime\0"
    "setDndDragThreshold\0dndDragThreshold\0"
    "setThemeName\0themeName\0setIconThemeName\0"
    "iconThemeName\0setSoundThemeName\0"
    "soundThemeName\0setFontName\0fontName\0"
    "setMonoFontName\0monoFontName\0"
    "setFontPointSize\0fontPointSize\0"
    "setGtkFontName\0setActiveColor\0activeColor\0"
    "setDarkActiveColor\0setWindow\0window\0"
    "setWindowText\0windowText\0setBase\0base\0"
    "setAlternateBase\0alternateBase\0"
    "setToolTipBase\0toolTipBase\0setToolTipText\0"
    "toolTipText\0setText\0text\0setButton\0"
    "button\0setButtonText\0buttonText\0"
    "setBrightText\0brightText\0setLight\0"
    "light\0setMidlight\0midlight\0setDark\0"
    "dark\0setMid\0mid\0setShadow\0shadow\0"
    "setHighlight\0highlight\0setHighlightedText\0"
    "highlightedText\0setLink\0link\0"
    "setLinkVisited\0linkVisited\0setItemBackground\0"
    "itemBackground\0setTextTitle\0textTitle\0"
    "setTextTips\0textTips\0setTextWarning\0"
    "textWarning\0setTextLively\0textLively\0"
    "setLightLively\0lightLively\0setDarkLively\0"
    "darkLively\0setFrameBorder\0frameBorder\0"
    "dotsPerInch\0screenName\0setDotsPerInch\0"
    "dpi\0setWindowRadius\0windowRadius\0"
    "_q_onThemePropertyChanged\0name\0value"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DXCBPlatformInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      47,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  249,    2, 0x0a /* Public */,
       4,    1,  252,    2, 0x0a /* Public */,
       6,    1,  255,    2, 0x0a /* Public */,
       8,    1,  258,    2, 0x0a /* Public */,
      10,    1,  261,    2, 0x0a /* Public */,
      12,    1,  264,    2, 0x0a /* Public */,
      14,    1,  267,    2, 0x0a /* Public */,
      16,    1,  270,    2, 0x0a /* Public */,
      18,    1,  273,    2, 0x0a /* Public */,
      20,    1,  276,    2, 0x0a /* Public */,
      22,    1,  279,    2, 0x0a /* Public */,
      24,    1,  282,    2, 0x0a /* Public */,
      26,    1,  285,    2, 0x0a /* Public */,
      27,    1,  288,    2, 0x0a /* Public */,
      29,    1,  291,    2, 0x0a /* Public */,
      30,    1,  294,    2, 0x0a /* Public */,
      32,    1,  297,    2, 0x0a /* Public */,
      34,    1,  300,    2, 0x0a /* Public */,
      36,    1,  303,    2, 0x0a /* Public */,
      38,    1,  306,    2, 0x0a /* Public */,
      40,    1,  309,    2, 0x0a /* Public */,
      42,    1,  312,    2, 0x0a /* Public */,
      44,    1,  315,    2, 0x0a /* Public */,
      46,    1,  318,    2, 0x0a /* Public */,
      48,    1,  321,    2, 0x0a /* Public */,
      50,    1,  324,    2, 0x0a /* Public */,
      52,    1,  327,    2, 0x0a /* Public */,
      54,    1,  330,    2, 0x0a /* Public */,
      56,    1,  333,    2, 0x0a /* Public */,
      58,    1,  336,    2, 0x0a /* Public */,
      60,    1,  339,    2, 0x0a /* Public */,
      62,    1,  342,    2, 0x0a /* Public */,
      64,    1,  345,    2, 0x0a /* Public */,
      66,    1,  348,    2, 0x0a /* Public */,
      68,    1,  351,    2, 0x0a /* Public */,
      70,    1,  354,    2, 0x0a /* Public */,
      72,    1,  357,    2, 0x0a /* Public */,
      74,    1,  360,    2, 0x0a /* Public */,
      76,    1,  363,    2, 0x0a /* Public */,
      78,    1,  366,    2, 0x0a /* Public */,
      80,    1,  369,    2, 0x0a /* Public */,
      82,    1,  372,    2, 0x0a /* Public */,
      84,    1,  375,    2, 0x0a /* Public */,
      84,    0,  378,    2, 0x2a /* Public | MethodCloned */,
      86,    2,  379,    2, 0x0a /* Public */,
      88,    1,  384,    2, 0x0a /* Public */,
      90,    2,  387,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::QByteArray,   15,
    QMetaType::Void, QMetaType::QByteArray,   17,
    QMetaType::Void, QMetaType::QByteArray,   19,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::QByteArray,   23,
    QMetaType::Void, QMetaType::QReal,   25,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::QColor,   28,
    QMetaType::Void, QMetaType::QColor,   28,
    QMetaType::Void, QMetaType::QColor,   31,
    QMetaType::Void, QMetaType::QColor,   33,
    QMetaType::Void, QMetaType::QColor,   35,
    QMetaType::Void, QMetaType::QColor,   37,
    QMetaType::Void, QMetaType::QColor,   39,
    QMetaType::Void, QMetaType::QColor,   41,
    QMetaType::Void, QMetaType::QColor,   43,
    QMetaType::Void, QMetaType::QColor,   45,
    QMetaType::Void, QMetaType::QColor,   47,
    QMetaType::Void, QMetaType::QColor,   49,
    QMetaType::Void, QMetaType::QColor,   51,
    QMetaType::Void, QMetaType::QColor,   53,
    QMetaType::Void, QMetaType::QColor,   55,
    QMetaType::Void, QMetaType::QColor,   57,
    QMetaType::Void, QMetaType::QColor,   59,
    QMetaType::Void, QMetaType::QColor,   61,
    QMetaType::Void, QMetaType::QColor,   63,
    QMetaType::Void, QMetaType::QColor,   65,
    QMetaType::Void, QMetaType::QColor,   67,
    QMetaType::Void, QMetaType::QColor,   69,
    QMetaType::Void, QMetaType::QColor,   71,
    QMetaType::Void, QMetaType::QColor,   73,
    QMetaType::Void, QMetaType::QColor,   75,
    QMetaType::Void, QMetaType::QColor,   77,
    QMetaType::Void, QMetaType::QColor,   79,
    QMetaType::Void, QMetaType::QColor,   81,
    QMetaType::Void, QMetaType::QColor,   83,
    QMetaType::Int, QMetaType::QString,   85,
    QMetaType::Int,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   85,   87,
    QMetaType::Void, QMetaType::Int,   89,
    QMetaType::Void, QMetaType::QByteArray, QMetaType::QVariant,   91,   92,

       0        // eod
};

void Dtk::Gui::DXCBPlatformInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DXCBPlatformInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setCursorBlinkTime((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setCursorBlinkTimeout((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->setCursorBlink((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->setDoubleClickDistance((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->setDoubleClickTime((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->setDndDragThreshold((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->setThemeName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 7: _t->setIconThemeName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 8: _t->setSoundThemeName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 9: _t->setFontName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 10: _t->setMonoFontName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 11: _t->setFontPointSize((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 12: _t->setGtkFontName((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 13: _t->setActiveColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 14: _t->setDarkActiveColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 15: _t->setWindow((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 16: _t->setWindowText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 17: _t->setBase((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 18: _t->setAlternateBase((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 19: _t->setToolTipBase((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 20: _t->setToolTipText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 21: _t->setText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 22: _t->setButton((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 23: _t->setButtonText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 24: _t->setBrightText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 25: _t->setLight((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 26: _t->setMidlight((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 27: _t->setDark((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 28: _t->setMid((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 29: _t->setShadow((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 30: _t->setHighlight((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 31: _t->setHighlightedText((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 32: _t->setLink((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 33: _t->setLinkVisited((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 34: _t->setItemBackground((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 35: _t->setTextTitle((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 36: _t->setTextTips((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 37: _t->setTextWarning((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 38: _t->setTextLively((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 39: _t->setLightLively((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 40: _t->setDarkLively((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 41: _t->setFrameBorder((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 42: { int _r = _t->dotsPerInch((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 43: { int _r = _t->dotsPerInch();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 44: _t->setDotsPerInch((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 45: _t->setWindowRadius((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 46: _t->d_func()->_q_onThemePropertyChanged((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DXCBPlatformInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DXCBPlatformInterface.data,
    qt_meta_data_Dtk__Gui__DXCBPlatformInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DXCBPlatformInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DXCBPlatformInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DXCBPlatformInterface.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DPlatformInterface"))
        return static_cast< DPlatformInterface*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DXCBPlatformInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 47;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
