/* Generated by wayland-scanner 1.25.0 */

/*
 * SPDX-FileCopyrightText: 2024 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface treeland_personalization_appearance_context_v1_interface;
extern const struct wl_interface treeland_personalization_cursor_context_v1_interface;
extern const struct wl_interface treeland_personalization_font_context_v1_interface;
extern const struct wl_interface treeland_personalization_wallpaper_context_v1_interface;
extern const struct wl_interface treeland_personalization_window_context_v1_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *treeland_personalization_manager_v1_types[] = {
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&treeland_personalization_window_context_v1_interface,
	&wl_surface_interface,
	&treeland_personalization_wallpaper_context_v1_interface,
	&treeland_personalization_cursor_context_v1_interface,
	&treeland_personalization_font_context_v1_interface,
	&treeland_personalization_appearance_context_v1_interface,
};

static const struct wl_message treeland_personalization_manager_v1_requests[] = {
	{ "get_window_context", "no", treeland_personalization_manager_v1_types + 7 },
	{ "get_wallpaper_context", "n", treeland_personalization_manager_v1_types + 9 },
	{ "get_cursor_context", "n", treeland_personalization_manager_v1_types + 10 },
	{ "get_font_context", "n", treeland_personalization_manager_v1_types + 11 },
	{ "get_appearance_context", "n", treeland_personalization_manager_v1_types + 12 },
	{ "destroy", "2", treeland_personalization_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_personalization_manager_v1_interface = {
	"treeland_personalization_manager_v1", 2,
	6, treeland_personalization_manager_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_personalization_wallpaper_context_v1_requests[] = {
	{ "set_fd", "hs", treeland_personalization_manager_v1_types + 0 },
	{ "set_identifier", "s", treeland_personalization_manager_v1_types + 0 },
	{ "set_output", "s", treeland_personalization_manager_v1_types + 0 },
	{ "set_on", "u", treeland_personalization_manager_v1_types + 0 },
	{ "set_isdark", "u", treeland_personalization_manager_v1_types + 0 },
	{ "commit", "", treeland_personalization_manager_v1_types + 0 },
	{ "get_metadata", "", treeland_personalization_manager_v1_types + 0 },
	{ "destroy", "", treeland_personalization_manager_v1_types + 0 },
};

static const struct wl_message treeland_personalization_wallpaper_context_v1_events[] = {
	{ "metadata", "s", treeland_personalization_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_personalization_wallpaper_context_v1_interface = {
	"treeland_personalization_wallpaper_context_v1", 1,
	8, treeland_personalization_wallpaper_context_v1_requests,
	1, treeland_personalization_wallpaper_context_v1_events,
};

static const struct wl_message treeland_personalization_cursor_context_v1_requests[] = {
	{ "set_theme", "s", treeland_personalization_manager_v1_types + 0 },
	{ "get_theme", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_size", "u", treeland_personalization_manager_v1_types + 0 },
	{ "get_size", "", treeland_personalization_manager_v1_types + 0 },
	{ "commit", "", treeland_personalization_manager_v1_types + 0 },
	{ "destroy", "", treeland_personalization_manager_v1_types + 0 },
};

static const struct wl_message treeland_personalization_cursor_context_v1_events[] = {
	{ "verfity", "i", treeland_personalization_manager_v1_types + 0 },
	{ "theme", "s", treeland_personalization_manager_v1_types + 0 },
	{ "size", "u", treeland_personalization_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_personalization_cursor_context_v1_interface = {
	"treeland_personalization_cursor_context_v1", 1,
	6, treeland_personalization_cursor_context_v1_requests,
	3, treeland_personalization_cursor_context_v1_events,
};

static const struct wl_message treeland_personalization_window_context_v1_requests[] = {
	{ "set_blend_mode", "i", treeland_personalization_manager_v1_types + 0 },
	{ "set_round_corner_radius", "i", treeland_personalization_manager_v1_types + 0 },
	{ "set_shadow", "iiiiiii", treeland_personalization_manager_v1_types + 0 },
	{ "set_border", "iiiii", treeland_personalization_manager_v1_types + 0 },
	{ "set_titlebar", "i", treeland_personalization_manager_v1_types + 0 },
	{ "destroy", "", treeland_personalization_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_personalization_window_context_v1_interface = {
	"treeland_personalization_window_context_v1", 1,
	6, treeland_personalization_window_context_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_personalization_font_context_v1_requests[] = {
	{ "set_font_size", "u", treeland_personalization_manager_v1_types + 0 },
	{ "get_font_size", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_font", "s", treeland_personalization_manager_v1_types + 0 },
	{ "get_font", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_monospace_font", "s", treeland_personalization_manager_v1_types + 0 },
	{ "get_monospace_font", "", treeland_personalization_manager_v1_types + 0 },
	{ "destroy", "", treeland_personalization_manager_v1_types + 0 },
};

static const struct wl_message treeland_personalization_font_context_v1_events[] = {
	{ "font", "s", treeland_personalization_manager_v1_types + 0 },
	{ "monospace_font", "s", treeland_personalization_manager_v1_types + 0 },
	{ "font_size", "u", treeland_personalization_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_personalization_font_context_v1_interface = {
	"treeland_personalization_font_context_v1", 1,
	7, treeland_personalization_font_context_v1_requests,
	3, treeland_personalization_font_context_v1_events,
};

static const struct wl_message treeland_personalization_appearance_context_v1_requests[] = {
	{ "set_round_corner_radius", "i", treeland_personalization_manager_v1_types + 0 },
	{ "get_round_corner_radius", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_icon_theme", "s", treeland_personalization_manager_v1_types + 0 },
	{ "get_icon_theme", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_active_color", "s", treeland_personalization_manager_v1_types + 0 },
	{ "get_active_color", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_window_opacity", "u", treeland_personalization_manager_v1_types + 0 },
	{ "get_window_opacity", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_window_theme_type", "u", treeland_personalization_manager_v1_types + 0 },
	{ "get_window_theme_type", "", treeland_personalization_manager_v1_types + 0 },
	{ "set_window_titlebar_height", "u", treeland_personalization_manager_v1_types + 0 },
	{ "get_window_titlebar_height", "", treeland_personalization_manager_v1_types + 0 },
	{ "destroy", "", treeland_personalization_manager_v1_types + 0 },
};

static const struct wl_message treeland_personalization_appearance_context_v1_events[] = {
	{ "round_corner_radius", "i", treeland_personalization_manager_v1_types + 0 },
	{ "icon_theme", "s", treeland_personalization_manager_v1_types + 0 },
	{ "active_color", "s", treeland_personalization_manager_v1_types + 0 },
	{ "window_opacity", "u", treeland_personalization_manager_v1_types + 0 },
	{ "window_theme_type", "u", treeland_personalization_manager_v1_types + 0 },
	{ "window_titlebar_height", "u", treeland_personalization_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_personalization_appearance_context_v1_interface = {
	"treeland_personalization_appearance_context_v1", 1,
	13, treeland_personalization_appearance_context_v1_requests,
	6, treeland_personalization_appearance_context_v1_events,
};

