/****************************************************************************
** Meta object code from reading C++ file 'xeventmonitor_interface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/dbus/xeventmonitor_interface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xeventmonitor_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ComDeepinApiXEventMonitorInterface_t {
    QByteArrayData data[22];
    char stringdata0[269];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ComDeepinApiXEventMonitorInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ComDeepinApiXEventMonitorInterface_t qt_meta_stringdata_ComDeepinApiXEventMonitorInterface = {
    {
QT_MOC_LITERAL(0, 0, 34), // "ComDeepinApiXEventMonitorInte..."
QT_MOC_LITERAL(1, 35, 11), // "ButtonPress"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 3), // "in0"
QT_MOC_LITERAL(4, 52, 3), // "in1"
QT_MOC_LITERAL(5, 56, 3), // "in2"
QT_MOC_LITERAL(6, 60, 3), // "in3"
QT_MOC_LITERAL(7, 64, 13), // "ButtonRelease"
QT_MOC_LITERAL(8, 78, 13), // "CancelAllArea"
QT_MOC_LITERAL(9, 92, 10), // "CursorInto"
QT_MOC_LITERAL(10, 103, 10), // "CursorMove"
QT_MOC_LITERAL(11, 114, 9), // "CursorOut"
QT_MOC_LITERAL(12, 124, 8), // "KeyPress"
QT_MOC_LITERAL(13, 133, 10), // "KeyRelease"
QT_MOC_LITERAL(14, 144, 12), // "RegisterArea"
QT_MOC_LITERAL(15, 157, 26), // "QDBusPendingReply<QString>"
QT_MOC_LITERAL(16, 184, 3), // "in4"
QT_MOC_LITERAL(17, 188, 13), // "RegisterAreas"
QT_MOC_LITERAL(18, 202, 8), // "AreaList"
QT_MOC_LITERAL(19, 211, 18), // "RegisterFullScreen"
QT_MOC_LITERAL(20, 230, 14), // "UnregisterArea"
QT_MOC_LITERAL(21, 245, 23) // "QDBusPendingReply<bool>"

    },
    "ComDeepinApiXEventMonitorInterface\0"
    "ButtonPress\0\0in0\0in1\0in2\0in3\0ButtonRelease\0"
    "CancelAllArea\0CursorInto\0CursorMove\0"
    "CursorOut\0KeyPress\0KeyRelease\0"
    "RegisterArea\0QDBusPendingReply<QString>\0"
    "in4\0RegisterAreas\0AreaList\0"
    "RegisterFullScreen\0UnregisterArea\0"
    "QDBusPendingReply<bool>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ComDeepinApiXEventMonitorInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   74,    2, 0x06 /* Public */,
       7,    4,   83,    2, 0x06 /* Public */,
       8,    0,   92,    2, 0x06 /* Public */,
       9,    3,   93,    2, 0x06 /* Public */,
      10,    3,  100,    2, 0x06 /* Public */,
      11,    3,  107,    2, 0x06 /* Public */,
      12,    4,  114,    2, 0x06 /* Public */,
      13,    4,  123,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    5,  132,    2, 0x0a /* Public */,
      17,    2,  143,    2, 0x0a /* Public */,
      19,    0,  148,    2, 0x0a /* Public */,
      20,    1,  149,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,    6,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,    6,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int, QMetaType::QString,    3,    4,    5,    6,

 // slots: parameters
    0x80000000 | 15, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    4,    5,    6,   16,
    0x80000000 | 15, 0x80000000 | 18, QMetaType::Int,    3,    4,
    0x80000000 | 15,
    0x80000000 | 21, QMetaType::QString,    3,

       0        // eod
};

void ComDeepinApiXEventMonitorInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComDeepinApiXEventMonitorInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ButtonPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 1: _t->ButtonRelease((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 2: _t->CancelAllArea(); break;
        case 3: _t->CursorInto((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 4: _t->CursorMove((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 5: _t->CursorOut((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 6: _t->KeyPress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 7: _t->KeyRelease((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 8: { QDBusPendingReply<QString> _r = _t->RegisterArea((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->RegisterAreas((*reinterpret_cast< AreaList(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QString> _r = _t->RegisterFullScreen();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<bool> _r = _t->UnregisterArea((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(int , int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::ButtonPress)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(int , int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::ButtonRelease)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::CancelAllArea)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::CursorInto)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::CursorMove)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::CursorOut)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(const QString & , int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::KeyPress)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ComDeepinApiXEventMonitorInterface::*)(const QString & , int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComDeepinApiXEventMonitorInterface::KeyRelease)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ComDeepinApiXEventMonitorInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_ComDeepinApiXEventMonitorInterface.data,
    qt_meta_data_ComDeepinApiXEventMonitorInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ComDeepinApiXEventMonitorInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComDeepinApiXEventMonitorInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ComDeepinApiXEventMonitorInterface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int ComDeepinApiXEventMonitorInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ComDeepinApiXEventMonitorInterface::ButtonPress(int _t1, int _t2, int _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ComDeepinApiXEventMonitorInterface::ButtonRelease(int _t1, int _t2, int _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ComDeepinApiXEventMonitorInterface::CancelAllArea()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ComDeepinApiXEventMonitorInterface::CursorInto(int _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ComDeepinApiXEventMonitorInterface::CursorMove(int _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ComDeepinApiXEventMonitorInterface::CursorOut(int _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void ComDeepinApiXEventMonitorInterface::KeyPress(const QString & _t1, int _t2, int _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ComDeepinApiXEventMonitorInterface::KeyRelease(const QString & _t1, int _t2, int _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
