/****************************************************************************
** Meta object code from reading C++ file 'dguiapplicationhelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/kernel/dguiapplicationhelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dguiapplicationhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DGuiApplicationHelper_t {
    QByteArrayData data[53];
    char stringdata0[777];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DGuiApplicationHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DGuiApplicationHelper_t qt_meta_stringdata_Dtk__Gui__DGuiApplicationHelper = {
    {
QT_MOC_LITERAL(0, 0, 31), // "Dtk::Gui::DGuiApplicationHelper"
QT_MOC_LITERAL(1, 32, 16), // "themeTypeChanged"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 9), // "ColorType"
QT_MOC_LITERAL(4, 60, 9), // "themeType"
QT_MOC_LITERAL(5, 70, 18), // "paletteTypeChanged"
QT_MOC_LITERAL(6, 89, 11), // "paletteType"
QT_MOC_LITERAL(7, 101, 18), // "newProcessInstance"
QT_MOC_LITERAL(8, 120, 3), // "pid"
QT_MOC_LITERAL(9, 124, 9), // "arguments"
QT_MOC_LITERAL(10, 134, 11), // "fontChanged"
QT_MOC_LITERAL(11, 146, 4), // "font"
QT_MOC_LITERAL(12, 151, 25), // "applicationPaletteChanged"
QT_MOC_LITERAL(13, 177, 15), // "sizeModeChanged"
QT_MOC_LITERAL(14, 193, 31), // "DGuiApplicationHelper::SizeMode"
QT_MOC_LITERAL(15, 225, 8), // "sizeMode"
QT_MOC_LITERAL(16, 234, 12), // "setThemeType"
QT_MOC_LITERAL(17, 247, 14), // "setPaletteType"
QT_MOC_LITERAL(18, 262, 16), // "handleHelpAction"
QT_MOC_LITERAL(19, 279, 7), // "openUrl"
QT_MOC_LITERAL(20, 287, 3), // "url"
QT_MOC_LITERAL(21, 291, 11), // "setSizeMode"
QT_MOC_LITERAL(22, 303, 4), // "mode"
QT_MOC_LITERAL(23, 308, 13), // "resetSizeMode"
QT_MOC_LITERAL(24, 322, 13), // "isCompactMode"
QT_MOC_LITERAL(25, 336, 23), // "_q_initApplicationTheme"
QT_MOC_LITERAL(26, 360, 18), // "_q_sizeModeChanged"
QT_MOC_LITERAL(27, 379, 13), // "hasUserManual"
QT_MOC_LITERAL(28, 393, 11), // "UnknownType"
QT_MOC_LITERAL(29, 405, 9), // "LightType"
QT_MOC_LITERAL(30, 415, 8), // "DarkType"
QT_MOC_LITERAL(31, 424, 11), // "SingleScope"
QT_MOC_LITERAL(32, 436, 9), // "UserScope"
QT_MOC_LITERAL(33, 446, 10), // "GroupScope"
QT_MOC_LITERAL(34, 457, 10), // "WorldScope"
QT_MOC_LITERAL(35, 468, 8), // "SizeMode"
QT_MOC_LITERAL(36, 477, 10), // "NormalMode"
QT_MOC_LITERAL(37, 488, 11), // "CompactMode"
QT_MOC_LITERAL(38, 500, 9), // "Attribute"
QT_MOC_LITERAL(39, 510, 21), // "UseInactiveColorGroup"
QT_MOC_LITERAL(40, 532, 16), // "ColorCompositing"
QT_MOC_LITERAL(41, 549, 24), // "DontSaveApplicationTheme"
QT_MOC_LITERAL(42, 574, 13), // "ReadOnlyLimit"
QT_MOC_LITERAL(43, 588, 21), // "IsDeepinPlatformTheme"
QT_MOC_LITERAL(44, 610, 14), // "IsDXcbPlatform"
QT_MOC_LITERAL(45, 625, 17), // "IsXWindowPlatform"
QT_MOC_LITERAL(46, 643, 18), // "IsTableEnvironment"
QT_MOC_LITERAL(47, 662, 19), // "IsDeepinEnvironment"
QT_MOC_LITERAL(48, 682, 27), // "IsSpecialEffectsEnvironment"
QT_MOC_LITERAL(49, 710, 17), // "IsWaylandPlatform"
QT_MOC_LITERAL(50, 728, 18), // "IsTreelandPlatform"
QT_MOC_LITERAL(51, 747, 13), // "HasAnimations"
QT_MOC_LITERAL(52, 761, 15) // "HasInWindowBlur"

    },
    "Dtk::Gui::DGuiApplicationHelper\0"
    "themeTypeChanged\0\0ColorType\0themeType\0"
    "paletteTypeChanged\0paletteType\0"
    "newProcessInstance\0pid\0arguments\0"
    "fontChanged\0font\0applicationPaletteChanged\0"
    "sizeModeChanged\0DGuiApplicationHelper::SizeMode\0"
    "sizeMode\0setThemeType\0setPaletteType\0"
    "handleHelpAction\0openUrl\0url\0setSizeMode\0"
    "mode\0resetSizeMode\0isCompactMode\0"
    "_q_initApplicationTheme\0_q_sizeModeChanged\0"
    "hasUserManual\0UnknownType\0LightType\0"
    "DarkType\0SingleScope\0UserScope\0"
    "GroupScope\0WorldScope\0SizeMode\0"
    "NormalMode\0CompactMode\0Attribute\0"
    "UseInactiveColorGroup\0ColorCompositing\0"
    "DontSaveApplicationTheme\0ReadOnlyLimit\0"
    "IsDeepinPlatformTheme\0IsDXcbPlatform\0"
    "IsXWindowPlatform\0IsTableEnvironment\0"
    "IsDeepinEnvironment\0IsSpecialEffectsEnvironment\0"
    "IsWaylandPlatform\0IsTreelandPlatform\0"
    "HasAnimations\0HasInWindowBlur"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DGuiApplicationHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       3,  134, // properties
       4,  146, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,
       5,    1,   97,    2, 0x06 /* Public */,
       7,    2,  100,    2, 0x06 /* Public */,
      10,    1,  105,    2, 0x06 /* Public */,
      12,    0,  108,    2, 0x06 /* Public */,
      13,    1,  109,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,  112,    2, 0x0a /* Public */,
      17,    1,  115,    2, 0x0a /* Public */,
      18,    0,  118,    2, 0x0a /* Public */,
      19,    1,  119,    2, 0x0a /* Public */,
      15,    0,  122,    2, 0x0a /* Public */,
      21,    1,  123,    2, 0x0a /* Public */,
      23,    0,  126,    2, 0x0a /* Public */,
      24,    0,  127,    2, 0x0a /* Public */,
      25,    1,  128,    2, 0x08 /* Private */,
      26,    1,  131,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    6,
    QMetaType::Void, QMetaType::LongLong, QMetaType::QStringList,    8,    9,
    QMetaType::Void, QMetaType::QFont,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    6,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   20,
    0x80000000 | 14,
    QMetaType::Void, 0x80000000 | 14,   22,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x00495009,
       6, 0x80000000 | 3, 0x0049510b,
      27, QMetaType::Bool, 0x00095001,

 // properties: notify_signal_id
       0,
       1,
       0,

 // enums: name, alias, flags, count, data
       3,    3, 0x0,    3,  166,
      31,   31, 0x0,    3,  172,
      35,   35, 0x0,    2,  178,
      38,   38, 0x0,   14,  182,

 // enum data: key, value
      28, uint(Dtk::Gui::DGuiApplicationHelper::UnknownType),
      29, uint(Dtk::Gui::DGuiApplicationHelper::LightType),
      30, uint(Dtk::Gui::DGuiApplicationHelper::DarkType),
      32, uint(Dtk::Gui::DGuiApplicationHelper::UserScope),
      33, uint(Dtk::Gui::DGuiApplicationHelper::GroupScope),
      34, uint(Dtk::Gui::DGuiApplicationHelper::WorldScope),
      36, uint(Dtk::Gui::DGuiApplicationHelper::NormalMode),
      37, uint(Dtk::Gui::DGuiApplicationHelper::CompactMode),
      39, uint(Dtk::Gui::DGuiApplicationHelper::UseInactiveColorGroup),
      40, uint(Dtk::Gui::DGuiApplicationHelper::ColorCompositing),
      41, uint(Dtk::Gui::DGuiApplicationHelper::DontSaveApplicationTheme),
      42, uint(Dtk::Gui::DGuiApplicationHelper::ReadOnlyLimit),
      43, uint(Dtk::Gui::DGuiApplicationHelper::IsDeepinPlatformTheme),
      44, uint(Dtk::Gui::DGuiApplicationHelper::IsDXcbPlatform),
      45, uint(Dtk::Gui::DGuiApplicationHelper::IsXWindowPlatform),
      46, uint(Dtk::Gui::DGuiApplicationHelper::IsTableEnvironment),
      47, uint(Dtk::Gui::DGuiApplicationHelper::IsDeepinEnvironment),
      48, uint(Dtk::Gui::DGuiApplicationHelper::IsSpecialEffectsEnvironment),
      49, uint(Dtk::Gui::DGuiApplicationHelper::IsWaylandPlatform),
      50, uint(Dtk::Gui::DGuiApplicationHelper::IsTreelandPlatform),
      51, uint(Dtk::Gui::DGuiApplicationHelper::HasAnimations),
      52, uint(Dtk::Gui::DGuiApplicationHelper::HasInWindowBlur),

       0        // eod
};

void Dtk::Gui::DGuiApplicationHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DGuiApplicationHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->themeTypeChanged((*reinterpret_cast< ColorType(*)>(_a[1]))); break;
        case 1: _t->paletteTypeChanged((*reinterpret_cast< ColorType(*)>(_a[1]))); break;
        case 2: _t->newProcessInstance((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 3: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 4: _t->applicationPaletteChanged(); break;
        case 5: _t->sizeModeChanged((*reinterpret_cast< DGuiApplicationHelper::SizeMode(*)>(_a[1]))); break;
        case 6: _t->setThemeType((*reinterpret_cast< ColorType(*)>(_a[1]))); break;
        case 7: _t->setPaletteType((*reinterpret_cast< ColorType(*)>(_a[1]))); break;
        case 8: _t->handleHelpAction(); break;
        case 9: _t->openUrl((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: { DGuiApplicationHelper::SizeMode _r = _t->sizeMode();
            if (_a[0]) *reinterpret_cast< DGuiApplicationHelper::SizeMode*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setSizeMode((*reinterpret_cast< const DGuiApplicationHelper::SizeMode(*)>(_a[1]))); break;
        case 12: _t->resetSizeMode(); break;
        case 13: { bool _r = _t->isCompactMode();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->d_func()->_q_initApplicationTheme((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->d_func()->_q_sizeModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DGuiApplicationHelper::*)(ColorType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DGuiApplicationHelper::themeTypeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DGuiApplicationHelper::*)(ColorType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DGuiApplicationHelper::paletteTypeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DGuiApplicationHelper::*)(qint64 , const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DGuiApplicationHelper::newProcessInstance)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DGuiApplicationHelper::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DGuiApplicationHelper::fontChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DGuiApplicationHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DGuiApplicationHelper::applicationPaletteChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DGuiApplicationHelper::*)(DGuiApplicationHelper::SizeMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DGuiApplicationHelper::sizeModeChanged)) {
                *result = 5;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DGuiApplicationHelper *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< ColorType*>(_v) = _t->themeType(); break;
        case 1: *reinterpret_cast< ColorType*>(_v) = _t->paletteType(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->hasUserManual(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DGuiApplicationHelper *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setPaletteType(*reinterpret_cast< ColorType*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DGuiApplicationHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DGuiApplicationHelper.data,
    qt_meta_data_Dtk__Gui__DGuiApplicationHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DGuiApplicationHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DGuiApplicationHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DGuiApplicationHelper.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DGuiApplicationHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DGuiApplicationHelper::themeTypeChanged(ColorType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Dtk::Gui::DGuiApplicationHelper::paletteTypeChanged(ColorType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Dtk::Gui::DGuiApplicationHelper::newProcessInstance(qint64 _t1, const QStringList & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Dtk::Gui::DGuiApplicationHelper::fontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Dtk::Gui::DGuiApplicationHelper::applicationPaletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Dtk::Gui::DGuiApplicationHelper::sizeModeChanged(DGuiApplicationHelper::SizeMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
