/****************************************************************************
** Meta object code from reading C++ file 'dfontmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/util/dfontmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfontmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DFontManager::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DFontManager",
        "fontChanged",
        "",
        "fontPixelSize",
        "SizeType",
        "type",
        "setFontPixelSize",
        "size",
        "QFont",
        "font",
        "get",
        "pixelSize",
        "base",
        "t1",
        "t2",
        "t3",
        "t4",
        "t5",
        "t6",
        "t7",
        "t8",
        "t9",
        "t10",
        "baseFont",
        "T1",
        "T2",
        "T3",
        "T4",
        "T5",
        "T6",
        "T7",
        "T8",
        "T9",
        "T10",
        "T11",
        "NSizeTypes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'fontPixelSize'
        QtMocHelpers::MethodData<int(enum SizeType) const>(3, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Method 'setFontPixelSize'
        QtMocHelpers::MethodData<void(enum SizeType, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 7 },
        }}),
        // Method 'fontPixelSize'
        QtMocHelpers::MethodData<int(const QFont &)>(3, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'get'
        QtMocHelpers::MethodData<QFont(int, const QFont &)>(10, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::Int, 11 }, { 0x80000000 | 8, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 't1'
        QtMocHelpers::PropertyData<QFont>(13, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't2'
        QtMocHelpers::PropertyData<QFont>(14, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't3'
        QtMocHelpers::PropertyData<QFont>(15, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't4'
        QtMocHelpers::PropertyData<QFont>(16, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't5'
        QtMocHelpers::PropertyData<QFont>(17, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't6'
        QtMocHelpers::PropertyData<QFont>(18, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't7'
        QtMocHelpers::PropertyData<QFont>(19, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't8'
        QtMocHelpers::PropertyData<QFont>(20, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't9'
        QtMocHelpers::PropertyData<QFont>(21, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 't10'
        QtMocHelpers::PropertyData<QFont>(22, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'baseFont'
        QtMocHelpers::PropertyData<QFont>(23, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SizeType'
        QtMocHelpers::EnumData<enum SizeType>(4, 4, QMC::EnumFlags{}).add({
            {   24, SizeType::T1 },
            {   25, SizeType::T2 },
            {   26, SizeType::T3 },
            {   27, SizeType::T4 },
            {   28, SizeType::T5 },
            {   29, SizeType::T6 },
            {   30, SizeType::T7 },
            {   31, SizeType::T8 },
            {   32, SizeType::T9 },
            {   33, SizeType::T10 },
            {   34, SizeType::T11 },
            {   35, SizeType::NSizeTypes },
        }),
    };
    return QtMocHelpers::metaObjectData<DFontManager, qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DFontManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DFontManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFontManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fontChanged(); break;
        case 1: { int _r = _t->fontPixelSize((*reinterpret_cast<std::add_pointer_t<enum SizeType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->setFontPixelSize((*reinterpret_cast<std::add_pointer_t<enum SizeType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: { int _r = _t->fontPixelSize((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { QFont _r = _t->get((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QFont>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QFont*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFontManager::*)()>(_a, &DFontManager::fontChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QFont*>(_v) = _t->t1(); break;
        case 1: *reinterpret_cast<QFont*>(_v) = _t->t2(); break;
        case 2: *reinterpret_cast<QFont*>(_v) = _t->t3(); break;
        case 3: *reinterpret_cast<QFont*>(_v) = _t->t4(); break;
        case 4: *reinterpret_cast<QFont*>(_v) = _t->t5(); break;
        case 5: *reinterpret_cast<QFont*>(_v) = _t->t6(); break;
        case 6: *reinterpret_cast<QFont*>(_v) = _t->t7(); break;
        case 7: *reinterpret_cast<QFont*>(_v) = _t->t8(); break;
        case 8: *reinterpret_cast<QFont*>(_v) = _t->t9(); break;
        case 9: *reinterpret_cast<QFont*>(_v) = _t->t10(); break;
        case 10: *reinterpret_cast<QFont*>(_v) = _t->baseFont(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 10: _t->setBaseFont(*reinterpret_cast<QFont*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 10: _t->resetBaseFont(); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DFontManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DFontManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui12DFontManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DFontManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DFontManager::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
