/****************************************************************************
** Meta object code from reading C++ file 'ddciiconplayer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/util/ddciiconplayer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ddciiconplayer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DDciIconImagePlayer_t {
    QByteArrayData data[17];
    char stringdata0[199];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DDciIconImagePlayer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DDciIconImagePlayer_t qt_meta_stringdata_Dtk__Gui__DDciIconImagePlayer = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Dtk::Gui::DDciIconImagePlayer"
QT_MOC_LITERAL(1, 30, 7), // "started"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 7), // "updated"
QT_MOC_LITERAL(4, 47, 8), // "finished"
QT_MOC_LITERAL(5, 56, 12), // "stateChanged"
QT_MOC_LITERAL(6, 69, 5), // "start"
QT_MOC_LITERAL(7, 75, 5), // "speed"
QT_MOC_LITERAL(8, 81, 31), // "Gui::DDciIconImagePlayer::Flags"
QT_MOC_LITERAL(9, 113, 5), // "flags"
QT_MOC_LITERAL(10, 119, 4), // "stop"
QT_MOC_LITERAL(11, 124, 5), // "state"
QT_MOC_LITERAL(12, 130, 31), // "Gui::DDciIconImagePlayer::State"
QT_MOC_LITERAL(13, 162, 5), // "State"
QT_MOC_LITERAL(14, 168, 10), // "NotRunning"
QT_MOC_LITERAL(15, 179, 11), // "WaitingRead"
QT_MOC_LITERAL(16, 191, 7) // "Running"

    },
    "Dtk::Gui::DDciIconImagePlayer\0started\0"
    "\0updated\0finished\0stateChanged\0start\0"
    "speed\0Gui::DDciIconImagePlayer::Flags\0"
    "flags\0stop\0state\0Gui::DDciIconImagePlayer::State\0"
    "State\0NotRunning\0WaitingRead\0Running"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DDciIconImagePlayer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       1,   68, // properties
       1,   72, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,
       3,    0,   55,    2, 0x06 /* Public */,
       4,    0,   56,    2, 0x06 /* Public */,
       5,    0,   57,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   58,    2, 0x0a /* Public */,
       6,    1,   63,    2, 0x2a /* Public | MethodCloned */,
       6,    0,   66,    2, 0x2a /* Public | MethodCloned */,
      10,    0,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Bool, QMetaType::Float, 0x80000000 | 8,    7,    9,
    QMetaType::Bool, QMetaType::Float,    7,
    QMetaType::Bool,
    QMetaType::Void,

 // properties: name, type, flags
      11, 0x80000000 | 12, 0x00495009,

 // properties: notify_signal_id
       3,

 // enums: name, alias, flags, count, data
      13,   13, 0x0,    3,   77,

 // enum data: key, value
      14, uint(Dtk::Gui::DDciIconImagePlayer::NotRunning),
      15, uint(Dtk::Gui::DDciIconImagePlayer::WaitingRead),
      16, uint(Dtk::Gui::DDciIconImagePlayer::Running),

       0        // eod
};

void Dtk::Gui::DDciIconImagePlayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DDciIconImagePlayer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->updated(); break;
        case 2: _t->finished(); break;
        case 3: _t->stateChanged(); break;
        case 4: { bool _r = _t->start((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< Gui::DDciIconImagePlayer::Flags(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->start((*reinterpret_cast< float(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->start();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->stop(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DDciIconImagePlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconImagePlayer::started)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DDciIconImagePlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconImagePlayer::updated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DDciIconImagePlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconImagePlayer::finished)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DDciIconImagePlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconImagePlayer::stateChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DDciIconImagePlayer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Gui::DDciIconImagePlayer::State*>(_v) = _t->state(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DDciIconImagePlayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DDciIconImagePlayer.data,
    qt_meta_data_Dtk__Gui__DDciIconImagePlayer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DDciIconImagePlayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DDciIconImagePlayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DDciIconImagePlayer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DDciIconImagePlayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DDciIconImagePlayer::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DDciIconImagePlayer::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DDciIconImagePlayer::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Dtk::Gui::DDciIconImagePlayer::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
struct qt_meta_stringdata_Dtk__Gui__DDciIconPlayer_t {
    QByteArrayData data[12];
    char stringdata0[123];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DDciIconPlayer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DDciIconPlayer_t qt_meta_stringdata_Dtk__Gui__DDciIconPlayer = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Dtk::Gui::DDciIconPlayer"
QT_MOC_LITERAL(1, 25, 12), // "stateChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 7), // "updated"
QT_MOC_LITERAL(4, 47, 11), // "modeChanged"
QT_MOC_LITERAL(5, 59, 14), // "DDciIcon::Mode"
QT_MOC_LITERAL(6, 74, 7), // "oldMode"
QT_MOC_LITERAL(7, 82, 7), // "newMode"
QT_MOC_LITERAL(8, 90, 16), // "_q_playFromQueue"
QT_MOC_LITERAL(9, 107, 5), // "State"
QT_MOC_LITERAL(10, 113, 4), // "Idle"
QT_MOC_LITERAL(11, 118, 4) // "Busy"

    },
    "Dtk::Gui::DDciIconPlayer\0stateChanged\0"
    "\0updated\0modeChanged\0DDciIcon::Mode\0"
    "oldMode\0newMode\0_q_playFromQueue\0State\0"
    "Idle\0Busy"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DDciIconPlayer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       1,   44, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,
       3,    0,   35,    2, 0x06 /* Public */,
       4,    2,   36,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   41,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5, 0x80000000 | 5,    6,    7,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,

 // enums: name, alias, flags, count, data
       9,    9, 0x0,    2,   49,

 // enum data: key, value
      10, uint(Dtk::Gui::DDciIconPlayer::Idle),
      11, uint(Dtk::Gui::DDciIconPlayer::Busy),

       0        // eod
};

void Dtk::Gui::DDciIconPlayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DDciIconPlayer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChanged(); break;
        case 1: _t->updated(); break;
        case 2: _t->modeChanged((*reinterpret_cast< DDciIcon::Mode(*)>(_a[1])),(*reinterpret_cast< DDciIcon::Mode(*)>(_a[2]))); break;
        case 3: _t->d_func()->_q_playFromQueue((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DDciIconPlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconPlayer::stateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DDciIconPlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconPlayer::updated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DDciIconPlayer::*)(DDciIcon::Mode , DDciIcon::Mode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDciIconPlayer::modeChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DDciIconPlayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DDciIconPlayer.data,
    qt_meta_data_Dtk__Gui__DDciIconPlayer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DDciIconPlayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DDciIconPlayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DDciIconPlayer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DDciIconPlayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DDciIconPlayer::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DDciIconPlayer::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DDciIconPlayer::modeChanged(DDciIcon::Mode _t1, DDciIcon::Mode _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
