/****************************************************************************
** Meta object code from reading C++ file 'dregionmonitor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/kernel/dregionmonitor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dregionmonitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DRegionMonitor_t {
    QByteArrayData data[35];
    char stringdata0[450];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DRegionMonitor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DRegionMonitor_t qt_meta_stringdata_Dtk__Gui__DRegionMonitor = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Dtk::Gui::DRegionMonitor"
QT_MOC_LITERAL(1, 25, 11), // "buttonPress"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 1), // "p"
QT_MOC_LITERAL(4, 40, 4), // "flag"
QT_MOC_LITERAL(5, 45, 13), // "buttonRelease"
QT_MOC_LITERAL(6, 59, 10), // "cursorMove"
QT_MOC_LITERAL(7, 70, 11), // "cursorEnter"
QT_MOC_LITERAL(8, 82, 11), // "cursorLeave"
QT_MOC_LITERAL(9, 94, 8), // "keyPress"
QT_MOC_LITERAL(10, 103, 7), // "keyname"
QT_MOC_LITERAL(11, 111, 10), // "keyRelease"
QT_MOC_LITERAL(12, 122, 21), // "registerdFlagsChanged"
QT_MOC_LITERAL(13, 144, 14), // "RegisterdFlags"
QT_MOC_LITERAL(14, 159, 5), // "flags"
QT_MOC_LITERAL(15, 165, 21), // "coordinateTypeChanged"
QT_MOC_LITERAL(16, 187, 14), // "CoordinateType"
QT_MOC_LITERAL(17, 202, 4), // "type"
QT_MOC_LITERAL(18, 207, 14), // "registerRegion"
QT_MOC_LITERAL(19, 222, 6), // "region"
QT_MOC_LITERAL(20, 229, 16), // "unregisterRegion"
QT_MOC_LITERAL(21, 246, 16), // "setWatchedRegion"
QT_MOC_LITERAL(22, 263, 16), // "setRegisterFlags"
QT_MOC_LITERAL(23, 280, 17), // "setCoordinateType"
QT_MOC_LITERAL(24, 298, 14), // "_q_ButtonPress"
QT_MOC_LITERAL(25, 313, 16), // "_q_ButtonRelease"
QT_MOC_LITERAL(26, 330, 13), // "_q_CursorMove"
QT_MOC_LITERAL(27, 344, 14), // "_q_CursorEnter"
QT_MOC_LITERAL(28, 359, 14), // "_q_CursorLeave"
QT_MOC_LITERAL(29, 374, 11), // "_q_KeyPress"
QT_MOC_LITERAL(30, 386, 13), // "_q_KeyRelease"
QT_MOC_LITERAL(31, 400, 14), // "coordinateType"
QT_MOC_LITERAL(32, 415, 14), // "registerdFlags"
QT_MOC_LITERAL(33, 430, 10), // "ScaleRatio"
QT_MOC_LITERAL(34, 441, 8) // "Original"

    },
    "Dtk::Gui::DRegionMonitor\0buttonPress\0"
    "\0p\0flag\0buttonRelease\0cursorMove\0"
    "cursorEnter\0cursorLeave\0keyPress\0"
    "keyname\0keyRelease\0registerdFlagsChanged\0"
    "RegisterdFlags\0flags\0coordinateTypeChanged\0"
    "CoordinateType\0type\0registerRegion\0"
    "region\0unregisterRegion\0setWatchedRegion\0"
    "setRegisterFlags\0setCoordinateType\0"
    "_q_ButtonPress\0_q_ButtonRelease\0"
    "_q_CursorMove\0_q_CursorEnter\0"
    "_q_CursorLeave\0_q_KeyPress\0_q_KeyRelease\0"
    "coordinateType\0registerdFlags\0ScaleRatio\0"
    "Original"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DRegionMonitor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       2,  226, // properties
       1,  234, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  124,    2, 0x06 /* Public */,
       5,    2,  129,    2, 0x06 /* Public */,
       6,    1,  134,    2, 0x06 /* Public */,
       7,    1,  137,    2, 0x06 /* Public */,
       8,    1,  140,    2, 0x06 /* Public */,
       9,    1,  143,    2, 0x06 /* Public */,
      11,    1,  146,    2, 0x06 /* Public */,
      12,    1,  149,    2, 0x06 /* Public */,
      15,    1,  152,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  155,    2, 0x0a /* Public */,
      18,    1,  156,    2, 0x0a /* Public */,
      20,    0,  159,    2, 0x0a /* Public */,
      21,    1,  160,    2, 0x0a /* Public */,
      22,    1,  163,    2, 0x0a /* Public */,
      23,    1,  166,    2, 0x0a /* Public */,
      24,    4,  169,    2, 0x08 /* Private */,
      25,    4,  178,    2, 0x08 /* Private */,
      26,    3,  187,    2, 0x08 /* Private */,
      27,    3,  194,    2, 0x08 /* Private */,
      28,    3,  201,    2, 0x08 /* Private */,
      29,    4,  208,    2, 0x08 /* Private */,
      30,    4,  217,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QPoint, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QPoint, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QPoint,    3,
    QMetaType::Void, QMetaType::QPoint,    3,
    QMetaType::Void, QMetaType::QPoint,    3,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 16,   17,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRegion,   19,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRegion,   19,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,    2,

 // properties: name, type, flags
      31, 0x80000000 | 16, 0x0049510b,
      32, 0x80000000 | 13, 0x0049500b,

 // properties: notify_signal_id
       8,
       7,

 // enums: name, alias, flags, count, data
      16,   16, 0x0,    2,  239,

 // enum data: key, value
      33, uint(Dtk::Gui::DRegionMonitor::ScaleRatio),
      34, uint(Dtk::Gui::DRegionMonitor::Original),

       0        // eod
};

void Dtk::Gui::DRegionMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DRegionMonitor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->buttonPress((*reinterpret_cast< const QPoint(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        case 1: _t->buttonRelease((*reinterpret_cast< const QPoint(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        case 2: _t->cursorMove((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 3: _t->cursorEnter((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 4: _t->cursorLeave((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 5: _t->keyPress((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->keyRelease((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->registerdFlagsChanged((*reinterpret_cast< RegisterdFlags(*)>(_a[1]))); break;
        case 8: _t->coordinateTypeChanged((*reinterpret_cast< CoordinateType(*)>(_a[1]))); break;
        case 9: _t->registerRegion(); break;
        case 10: _t->registerRegion((*reinterpret_cast< const QRegion(*)>(_a[1]))); break;
        case 11: _t->unregisterRegion(); break;
        case 12: _t->setWatchedRegion((*reinterpret_cast< const QRegion(*)>(_a[1]))); break;
        case 13: _t->setRegisterFlags((*reinterpret_cast< RegisterdFlags(*)>(_a[1]))); break;
        case 14: _t->setCoordinateType((*reinterpret_cast< CoordinateType(*)>(_a[1]))); break;
        case 15: _t->d_func()->_q_ButtonPress((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 16: _t->d_func()->_q_ButtonRelease((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 17: _t->d_func()->_q_CursorMove((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 18: _t->d_func()->_q_CursorEnter((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 19: _t->d_func()->_q_CursorLeave((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 20: _t->d_func()->_q_KeyPress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 21: _t->d_func()->_q_KeyRelease((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DRegionMonitor::*)(const QPoint & , const int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::buttonPress)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(const QPoint & , const int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::buttonRelease)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(const QPoint & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::cursorMove)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(const QPoint & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::cursorEnter)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(const QPoint & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::cursorLeave)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::keyPress)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::keyRelease)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(RegisterdFlags ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::registerdFlagsChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DRegionMonitor::*)(CoordinateType ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DRegionMonitor::coordinateTypeChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DRegionMonitor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< CoordinateType*>(_v) = _t->coordinateType(); break;
        case 1: *reinterpret_cast< RegisterdFlags*>(_v) = _t->registerFlags(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DRegionMonitor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCoordinateType(*reinterpret_cast< CoordinateType*>(_v)); break;
        case 1: _t->setRegisterFlags(*reinterpret_cast< RegisterdFlags*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DRegionMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DRegionMonitor.data,
    qt_meta_data_Dtk__Gui__DRegionMonitor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DRegionMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DRegionMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DRegionMonitor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DRegionMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 22;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DRegionMonitor::buttonPress(const QPoint & _t1, const int _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Dtk::Gui::DRegionMonitor::buttonRelease(const QPoint & _t1, const int _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Dtk::Gui::DRegionMonitor::cursorMove(const QPoint & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Dtk::Gui::DRegionMonitor::cursorEnter(const QPoint & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Dtk::Gui::DRegionMonitor::cursorLeave(const QPoint & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Dtk::Gui::DRegionMonitor::keyPress(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Dtk::Gui::DRegionMonitor::keyRelease(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Dtk::Gui::DRegionMonitor::registerdFlagsChanged(RegisterdFlags _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Dtk::Gui::DRegionMonitor::coordinateTypeChanged(CoordinateType _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Gui::DRegionMonitor *>(this), &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
