/****************************************************************************
** Meta object code from reading C++ file 'dxcbplatforminterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/plugins/platform/xcb/dxcbplatforminterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dxcbplatforminterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DXCBPlatformInterface::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DXCBPlatformInterface",
        "setCursorBlinkTime",
        "",
        "cursorBlinkTime",
        "setCursorBlinkTimeout",
        "cursorBlinkTimeout",
        "setCursorBlink",
        "cursorBlink",
        "setDoubleClickDistance",
        "doubleClickDistance",
        "setDoubleClickTime",
        "doubleClickTime",
        "setDndDragThreshold",
        "dndDragThreshold",
        "setThemeName",
        "themeName",
        "setIconThemeName",
        "iconThemeName",
        "setSoundThemeName",
        "soundThemeName",
        "setFontName",
        "fontName",
        "setMonoFontName",
        "monoFontName",
        "setFontPointSize",
        "fontPointSize",
        "setGtkFontName",
        "setActiveColor",
        "QColor",
        "activeColor",
        "setDarkActiveColor",
        "dotsPerInch",
        "screenName",
        "setDotsPerInch",
        "dpi",
        "setWindowRadius",
        "windowRadius",
        "_q_onThemePropertyChanged",
        "name",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setCursorBlinkTime'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setCursorBlinkTimeout'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setCursorBlink'
        QtMocHelpers::SlotData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setDoubleClickDistance'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'setDoubleClickTime'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'setDndDragThreshold'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'setThemeName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Slot 'setIconThemeName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 17 },
        }}),
        // Slot 'setSoundThemeName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 19 },
        }}),
        // Slot 'setFontName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Slot 'setMonoFontName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 23 },
        }}),
        // Slot 'setFontPointSize'
        QtMocHelpers::SlotData<void(qreal)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Slot 'setGtkFontName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Slot 'setActiveColor'
        QtMocHelpers::SlotData<void(const QColor)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'setDarkActiveColor'
        QtMocHelpers::SlotData<void(const QColor &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'dotsPerInch'
        QtMocHelpers::SlotData<int(const QString &) const>(31, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 32 },
        }}),
        // Slot 'dotsPerInch'
        QtMocHelpers::SlotData<int() const>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int),
        // Slot 'setDotsPerInch'
        QtMocHelpers::SlotData<void(const QString &, int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 }, { QMetaType::Int, 34 },
        }}),
        // Slot 'setWindowRadius'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 36 },
        }}),
        // Slot '_q_onThemePropertyChanged'
        QtMocHelpers::SlotData<void(const QByteArray &, const QVariant &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 38 }, { 0x80000000 | 39, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DXCBPlatformInterface, qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DXCBPlatformInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DXCBPlatformInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DXCBPlatformInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setCursorBlinkTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setCursorBlinkTimeout((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setCursorBlink((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->setDoubleClickDistance((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setDoubleClickTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setDndDragThreshold((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setThemeName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 7: _t->setIconThemeName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 8: _t->setSoundThemeName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 9: _t->setFontName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 10: _t->setMonoFontName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 11: _t->setFontPointSize((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 12: _t->setGtkFontName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 13: _t->setActiveColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->setDarkActiveColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: { int _r = _t->dotsPerInch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: { int _r = _t->dotsPerInch();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setDotsPerInch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 18: _t->setWindowRadius((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->d_func()->_q_onThemePropertyChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *Dtk::Gui::DXCBPlatformInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DXCBPlatformInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui21DXCBPlatformInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DPlatformInterface"))
        return static_cast< DPlatformInterface*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DXCBPlatformInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
