/****************************************************************************
** Meta object code from reading C++ file 'dplatformhandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/kernel/dplatformhandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dplatformhandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DPlatformHandle_t {
    QByteArrayData data[71];
    char stringdata0[1164];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DPlatformHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DPlatformHandle_t qt_meta_stringdata_Dtk__Gui__DPlatformHandle = {
    {
QT_MOC_LITERAL(0, 0, 25), // "Dtk::Gui::DPlatformHandle"
QT_MOC_LITERAL(1, 26, 19), // "frameMarginsChanged"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 19), // "windowRadiusChanged"
QT_MOC_LITERAL(4, 67, 18), // "borderWidthChanged"
QT_MOC_LITERAL(5, 86, 18), // "borderColorChanged"
QT_MOC_LITERAL(6, 105, 19), // "shadowRadiusChanged"
QT_MOC_LITERAL(7, 125, 19), // "shadowOffsetChanged"
QT_MOC_LITERAL(8, 145, 18), // "shadowColorChanged"
QT_MOC_LITERAL(9, 164, 19), // "windowEffectChanged"
QT_MOC_LITERAL(10, 184, 26), // "windowStartUpEffectChanged"
QT_MOC_LITERAL(11, 211, 15), // "clipPathChanged"
QT_MOC_LITERAL(12, 227, 16), // "frameMaskChanged"
QT_MOC_LITERAL(13, 244, 28), // "translucentBackgroundChanged"
QT_MOC_LITERAL(14, 273, 25), // "enableSystemResizeChanged"
QT_MOC_LITERAL(15, 299, 23), // "enableSystemMoveChanged"
QT_MOC_LITERAL(16, 323, 23), // "enableBlurWindowChanged"
QT_MOC_LITERAL(17, 347, 30), // "autoInputMaskByClipPathChanged"
QT_MOC_LITERAL(18, 378, 15), // "setWindowRadius"
QT_MOC_LITERAL(19, 394, 12), // "windowRadius"
QT_MOC_LITERAL(20, 407, 14), // "setBorderWidth"
QT_MOC_LITERAL(21, 422, 11), // "borderWidth"
QT_MOC_LITERAL(22, 434, 14), // "setBorderColor"
QT_MOC_LITERAL(23, 449, 11), // "borderColor"
QT_MOC_LITERAL(24, 461, 15), // "setWindowEffect"
QT_MOC_LITERAL(25, 477, 12), // "EffectScenes"
QT_MOC_LITERAL(26, 490, 11), // "effectScene"
QT_MOC_LITERAL(27, 502, 22), // "setWindowStartUpEffect"
QT_MOC_LITERAL(28, 525, 11), // "EffectTypes"
QT_MOC_LITERAL(29, 537, 10), // "effectType"
QT_MOC_LITERAL(30, 548, 15), // "setShadowRadius"
QT_MOC_LITERAL(31, 564, 12), // "shadowRadius"
QT_MOC_LITERAL(32, 577, 15), // "setShadowOffset"
QT_MOC_LITERAL(33, 593, 12), // "shadowOffset"
QT_MOC_LITERAL(34, 606, 14), // "setShadowColor"
QT_MOC_LITERAL(35, 621, 11), // "shadowColor"
QT_MOC_LITERAL(36, 633, 11), // "setClipPath"
QT_MOC_LITERAL(37, 645, 12), // "QPainterPath"
QT_MOC_LITERAL(38, 658, 8), // "clipPath"
QT_MOC_LITERAL(39, 667, 12), // "setFrameMask"
QT_MOC_LITERAL(40, 680, 9), // "frameMask"
QT_MOC_LITERAL(41, 690, 24), // "setTranslucentBackground"
QT_MOC_LITERAL(42, 715, 21), // "translucentBackground"
QT_MOC_LITERAL(43, 737, 21), // "setEnableSystemResize"
QT_MOC_LITERAL(44, 759, 18), // "enableSystemResize"
QT_MOC_LITERAL(45, 778, 19), // "setEnableSystemMove"
QT_MOC_LITERAL(46, 798, 16), // "enableSystemMove"
QT_MOC_LITERAL(47, 815, 19), // "setEnableBlurWindow"
QT_MOC_LITERAL(48, 835, 16), // "enableBlurWindow"
QT_MOC_LITERAL(49, 852, 26), // "setAutoInputMaskByClipPath"
QT_MOC_LITERAL(50, 879, 23), // "autoInputMaskByClipPath"
QT_MOC_LITERAL(51, 903, 12), // "windowEffect"
QT_MOC_LITERAL(52, 916, 11), // "EffectScene"
QT_MOC_LITERAL(53, 928, 19), // "windowStartUpEffect"
QT_MOC_LITERAL(54, 948, 10), // "EffectType"
QT_MOC_LITERAL(55, 959, 12), // "frameMargins"
QT_MOC_LITERAL(56, 972, 8), // "QMargins"
QT_MOC_LITERAL(57, 981, 12), // "realWindowId"
QT_MOC_LITERAL(58, 994, 3), // "WId"
QT_MOC_LITERAL(59, 998, 14), // "EffectNoRadius"
QT_MOC_LITERAL(60, 1013, 14), // "EffectNoShadow"
QT_MOC_LITERAL(61, 1028, 14), // "EffectNoBorder"
QT_MOC_LITERAL(62, 1043, 13), // "EffectNoStart"
QT_MOC_LITERAL(63, 1057, 13), // "EffectNoClose"
QT_MOC_LITERAL(64, 1071, 16), // "EffectNoMaximize"
QT_MOC_LITERAL(65, 1088, 16), // "EffectNoMinimize"
QT_MOC_LITERAL(66, 1105, 12), // "EffectNormal"
QT_MOC_LITERAL(67, 1118, 12), // "EffectCursor"
QT_MOC_LITERAL(68, 1131, 9), // "EffectTop"
QT_MOC_LITERAL(69, 1141, 12), // "EffectBottom"
QT_MOC_LITERAL(70, 1154, 9) // "EffectOut"

    },
    "Dtk::Gui::DPlatformHandle\0frameMarginsChanged\0"
    "\0windowRadiusChanged\0borderWidthChanged\0"
    "borderColorChanged\0shadowRadiusChanged\0"
    "shadowOffsetChanged\0shadowColorChanged\0"
    "windowEffectChanged\0windowStartUpEffectChanged\0"
    "clipPathChanged\0frameMaskChanged\0"
    "translucentBackgroundChanged\0"
    "enableSystemResizeChanged\0"
    "enableSystemMoveChanged\0enableBlurWindowChanged\0"
    "autoInputMaskByClipPathChanged\0"
    "setWindowRadius\0windowRadius\0"
    "setBorderWidth\0borderWidth\0setBorderColor\0"
    "borderColor\0setWindowEffect\0EffectScenes\0"
    "effectScene\0setWindowStartUpEffect\0"
    "EffectTypes\0effectType\0setShadowRadius\0"
    "shadowRadius\0setShadowOffset\0shadowOffset\0"
    "setShadowColor\0shadowColor\0setClipPath\0"
    "QPainterPath\0clipPath\0setFrameMask\0"
    "frameMask\0setTranslucentBackground\0"
    "translucentBackground\0setEnableSystemResize\0"
    "enableSystemResize\0setEnableSystemMove\0"
    "enableSystemMove\0setEnableBlurWindow\0"
    "enableBlurWindow\0setAutoInputMaskByClipPath\0"
    "autoInputMaskByClipPath\0windowEffect\0"
    "EffectScene\0windowStartUpEffect\0"
    "EffectType\0frameMargins\0QMargins\0"
    "realWindowId\0WId\0EffectNoRadius\0"
    "EffectNoShadow\0EffectNoBorder\0"
    "EffectNoStart\0EffectNoClose\0"
    "EffectNoMaximize\0EffectNoMinimize\0"
    "EffectNormal\0EffectCursor\0EffectTop\0"
    "EffectBottom\0EffectOut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DPlatformHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      31,   14, // methods
      17,  230, // properties
       4,  298, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  169,    2, 0x06 /* Public */,
       3,    0,  170,    2, 0x06 /* Public */,
       4,    0,  171,    2, 0x06 /* Public */,
       5,    0,  172,    2, 0x06 /* Public */,
       6,    0,  173,    2, 0x06 /* Public */,
       7,    0,  174,    2, 0x06 /* Public */,
       8,    0,  175,    2, 0x06 /* Public */,
       9,    0,  176,    2, 0x06 /* Public */,
      10,    0,  177,    2, 0x06 /* Public */,
      11,    0,  178,    2, 0x06 /* Public */,
      12,    0,  179,    2, 0x06 /* Public */,
      13,    0,  180,    2, 0x06 /* Public */,
      14,    0,  181,    2, 0x06 /* Public */,
      15,    0,  182,    2, 0x06 /* Public */,
      16,    0,  183,    2, 0x06 /* Public */,
      17,    0,  184,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    1,  185,    2, 0x0a /* Public */,
      20,    1,  188,    2, 0x0a /* Public */,
      22,    1,  191,    2, 0x0a /* Public */,
      24,    1,  194,    2, 0x0a /* Public */,
      27,    1,  197,    2, 0x0a /* Public */,
      30,    1,  200,    2, 0x0a /* Public */,
      32,    1,  203,    2, 0x0a /* Public */,
      34,    1,  206,    2, 0x0a /* Public */,
      36,    1,  209,    2, 0x0a /* Public */,
      39,    1,  212,    2, 0x0a /* Public */,
      41,    1,  215,    2, 0x0a /* Public */,
      43,    1,  218,    2, 0x0a /* Public */,
      45,    1,  221,    2, 0x0a /* Public */,
      47,    1,  224,    2, 0x0a /* Public */,
      49,    1,  227,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::QColor,   23,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, QMetaType::Int,   31,
    QMetaType::Void, QMetaType::QPoint,   33,
    QMetaType::Void, QMetaType::QColor,   35,
    QMetaType::Void, 0x80000000 | 37,   38,
    QMetaType::Void, QMetaType::QRegion,   40,
    QMetaType::Void, QMetaType::Bool,   42,
    QMetaType::Void, QMetaType::Bool,   44,
    QMetaType::Void, QMetaType::Bool,   46,
    QMetaType::Void, QMetaType::Bool,   48,
    QMetaType::Void, QMetaType::Bool,   50,

 // properties: name, type, flags
      19, QMetaType::Int, 0x00495103,
      21, QMetaType::Int, 0x00495103,
      23, QMetaType::QColor, 0x00495103,
      31, QMetaType::Int, 0x00495103,
      33, QMetaType::QPoint, 0x00495103,
      35, QMetaType::QColor, 0x00495103,
      51, 0x80000000 | 52, 0x0049510b,
      53, 0x80000000 | 54, 0x0049510b,
      38, 0x80000000 | 37, 0x0049510b,
      40, QMetaType::QRegion, 0x00495103,
      55, 0x80000000 | 56, 0x00495009,
      42, QMetaType::Bool, 0x00495103,
      44, QMetaType::Bool, 0x00495103,
      46, QMetaType::Bool, 0x00495103,
      48, QMetaType::Bool, 0x00495103,
      50, QMetaType::Bool, 0x00495103,
      57, 0x80000000 | 58, 0x00095409,

 // properties: notify_signal_id
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
       0,
      11,
      12,
      13,
      14,
      15,
       0,

 // enums: name, alias, flags, count, data
      52,   52, 0x0,    7,  318,
      25,   52, 0x0,    7,  332,
      54,   54, 0x0,    5,  346,
      28,   54, 0x0,    5,  356,

 // enum data: key, value
      59, uint(Dtk::Gui::DPlatformHandle::EffectNoRadius),
      60, uint(Dtk::Gui::DPlatformHandle::EffectNoShadow),
      61, uint(Dtk::Gui::DPlatformHandle::EffectNoBorder),
      62, uint(Dtk::Gui::DPlatformHandle::EffectNoStart),
      63, uint(Dtk::Gui::DPlatformHandle::EffectNoClose),
      64, uint(Dtk::Gui::DPlatformHandle::EffectNoMaximize),
      65, uint(Dtk::Gui::DPlatformHandle::EffectNoMinimize),
      59, uint(Dtk::Gui::DPlatformHandle::EffectNoRadius),
      60, uint(Dtk::Gui::DPlatformHandle::EffectNoShadow),
      61, uint(Dtk::Gui::DPlatformHandle::EffectNoBorder),
      62, uint(Dtk::Gui::DPlatformHandle::EffectNoStart),
      63, uint(Dtk::Gui::DPlatformHandle::EffectNoClose),
      64, uint(Dtk::Gui::DPlatformHandle::EffectNoMaximize),
      65, uint(Dtk::Gui::DPlatformHandle::EffectNoMinimize),
      66, uint(Dtk::Gui::DPlatformHandle::EffectNormal),
      67, uint(Dtk::Gui::DPlatformHandle::EffectCursor),
      68, uint(Dtk::Gui::DPlatformHandle::EffectTop),
      69, uint(Dtk::Gui::DPlatformHandle::EffectBottom),
      70, uint(Dtk::Gui::DPlatformHandle::EffectOut),
      66, uint(Dtk::Gui::DPlatformHandle::EffectNormal),
      67, uint(Dtk::Gui::DPlatformHandle::EffectCursor),
      68, uint(Dtk::Gui::DPlatformHandle::EffectTop),
      69, uint(Dtk::Gui::DPlatformHandle::EffectBottom),
      70, uint(Dtk::Gui::DPlatformHandle::EffectOut),

       0        // eod
};

void Dtk::Gui::DPlatformHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DPlatformHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->frameMarginsChanged(); break;
        case 1: _t->windowRadiusChanged(); break;
        case 2: _t->borderWidthChanged(); break;
        case 3: _t->borderColorChanged(); break;
        case 4: _t->shadowRadiusChanged(); break;
        case 5: _t->shadowOffsetChanged(); break;
        case 6: _t->shadowColorChanged(); break;
        case 7: _t->windowEffectChanged(); break;
        case 8: _t->windowStartUpEffectChanged(); break;
        case 9: _t->clipPathChanged(); break;
        case 10: _t->frameMaskChanged(); break;
        case 11: _t->translucentBackgroundChanged(); break;
        case 12: _t->enableSystemResizeChanged(); break;
        case 13: _t->enableSystemMoveChanged(); break;
        case 14: _t->enableBlurWindowChanged(); break;
        case 15: _t->autoInputMaskByClipPathChanged(); break;
        case 16: _t->setWindowRadius((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->setBorderWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->setBorderColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 19: _t->setWindowEffect((*reinterpret_cast< EffectScenes(*)>(_a[1]))); break;
        case 20: _t->setWindowStartUpEffect((*reinterpret_cast< EffectTypes(*)>(_a[1]))); break;
        case 21: _t->setShadowRadius((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->setShadowOffset((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 23: _t->setShadowColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 24: _t->setClipPath((*reinterpret_cast< const QPainterPath(*)>(_a[1]))); break;
        case 25: _t->setFrameMask((*reinterpret_cast< const QRegion(*)>(_a[1]))); break;
        case 26: _t->setTranslucentBackground((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->setEnableSystemResize((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->setEnableSystemMove((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 29: _t->setEnableBlurWindow((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->setAutoInputMaskByClipPath((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPainterPath >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::frameMarginsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::windowRadiusChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::borderWidthChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::borderColorChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::shadowRadiusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::shadowOffsetChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::shadowColorChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::windowEffectChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::windowStartUpEffectChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::clipPathChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::frameMaskChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::translucentBackgroundChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::enableSystemResizeChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::enableSystemMoveChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::enableBlurWindowChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (DPlatformHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DPlatformHandle::autoInputMaskByClipPathChanged)) {
                *result = 15;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMargins >(); break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPainterPath >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DPlatformHandle *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->windowRadius(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->borderWidth(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->borderColor(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->shadowRadius(); break;
        case 4: *reinterpret_cast< QPoint*>(_v) = _t->shadowOffset(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->shadowColor(); break;
        case 6: *reinterpret_cast< EffectScene*>(_v) = _t->windowEffect(); break;
        case 7: *reinterpret_cast< EffectType*>(_v) = _t->windowStartUpEffect(); break;
        case 8: *reinterpret_cast< QPainterPath*>(_v) = _t->clipPath(); break;
        case 9: *reinterpret_cast< QRegion*>(_v) = _t->frameMask(); break;
        case 10: *reinterpret_cast< QMargins*>(_v) = _t->frameMargins(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->translucentBackground(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->enableSystemResize(); break;
        case 13: *reinterpret_cast< bool*>(_v) = _t->enableSystemMove(); break;
        case 14: *reinterpret_cast< bool*>(_v) = _t->enableBlurWindow(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->autoInputMaskByClipPath(); break;
        case 16: *reinterpret_cast< WId*>(_v) = _t->realWindowId(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DPlatformHandle *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWindowRadius(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setBorderWidth(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setShadowRadius(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setShadowOffset(*reinterpret_cast< QPoint*>(_v)); break;
        case 5: _t->setShadowColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setWindowEffect(*reinterpret_cast< EffectScene*>(_v)); break;
        case 7: _t->setWindowStartUpEffect(*reinterpret_cast< EffectType*>(_v)); break;
        case 8: _t->setClipPath(*reinterpret_cast< QPainterPath*>(_v)); break;
        case 9: _t->setFrameMask(*reinterpret_cast< QRegion*>(_v)); break;
        case 11: _t->setTranslucentBackground(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setEnableSystemResize(*reinterpret_cast< bool*>(_v)); break;
        case 13: _t->setEnableSystemMove(*reinterpret_cast< bool*>(_v)); break;
        case 14: _t->setEnableBlurWindow(*reinterpret_cast< bool*>(_v)); break;
        case 15: _t->setAutoInputMaskByClipPath(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DPlatformHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DPlatformHandle.data,
    qt_meta_data_Dtk__Gui__DPlatformHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DPlatformHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DPlatformHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DPlatformHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DPlatformHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 17;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DPlatformHandle::frameMarginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DPlatformHandle::windowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DPlatformHandle::borderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Dtk::Gui::DPlatformHandle::borderColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Dtk::Gui::DPlatformHandle::shadowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Dtk::Gui::DPlatformHandle::shadowOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Dtk::Gui::DPlatformHandle::shadowColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Dtk::Gui::DPlatformHandle::windowEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Dtk::Gui::DPlatformHandle::windowStartUpEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Dtk::Gui::DPlatformHandle::clipPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Dtk::Gui::DPlatformHandle::frameMaskChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Dtk::Gui::DPlatformHandle::translucentBackgroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Dtk::Gui::DPlatformHandle::enableSystemResizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void Dtk::Gui::DPlatformHandle::enableSystemMoveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Dtk::Gui::DPlatformHandle::enableBlurWindowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Dtk::Gui::DPlatformHandle::autoInputMaskByClipPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
