/****************************************************************************
** Meta object code from reading C++ file 'dwindowmanagerhelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/kernel/dwindowmanagerhelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dwindowmanagerhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DWindowManagerHelper_t {
    QByteArrayData data[53];
    char stringdata0[729];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DWindowManagerHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DWindowManagerHelper_t qt_meta_stringdata_Dtk__Gui__DWindowManagerHelper = {
    {
QT_MOC_LITERAL(0, 0, 30), // "Dtk::Gui::DWindowManagerHelper"
QT_MOC_LITERAL(1, 31, 20), // "windowManagerChanged"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 20), // "hasBlurWindowChanged"
QT_MOC_LITERAL(4, 74, 19), // "hasCompositeChanged"
QT_MOC_LITERAL(5, 94, 20), // "hasNoTitlebarChanged"
QT_MOC_LITERAL(6, 115, 25), // "hasWallpaperEffectChanged"
QT_MOC_LITERAL(7, 141, 17), // "windowListChanged"
QT_MOC_LITERAL(8, 159, 25), // "windowMotifWMHintsChanged"
QT_MOC_LITERAL(9, 185, 5), // "winId"
QT_MOC_LITERAL(10, 191, 13), // "hasBlurWindow"
QT_MOC_LITERAL(11, 205, 12), // "hasComposite"
QT_MOC_LITERAL(12, 218, 13), // "hasNoTitlebar"
QT_MOC_LITERAL(13, 232, 18), // "hasWallpaperEffect"
QT_MOC_LITERAL(14, 251, 13), // "MotifFunction"
QT_MOC_LITERAL(15, 265, 11), // "FUNC_RESIZE"
QT_MOC_LITERAL(16, 277, 9), // "FUNC_MOVE"
QT_MOC_LITERAL(17, 287, 13), // "FUNC_MINIMIZE"
QT_MOC_LITERAL(18, 301, 13), // "FUNC_MAXIMIZE"
QT_MOC_LITERAL(19, 315, 10), // "FUNC_CLOSE"
QT_MOC_LITERAL(20, 326, 8), // "FUNC_ALL"
QT_MOC_LITERAL(21, 335, 14), // "MotifFunctions"
QT_MOC_LITERAL(22, 350, 15), // "MotifDecoration"
QT_MOC_LITERAL(23, 366, 12), // "DECOR_BORDER"
QT_MOC_LITERAL(24, 379, 13), // "DECOR_RESIZEH"
QT_MOC_LITERAL(25, 393, 11), // "DECOR_TITLE"
QT_MOC_LITERAL(26, 405, 10), // "DECOR_MENU"
QT_MOC_LITERAL(27, 416, 14), // "DECOR_MINIMIZE"
QT_MOC_LITERAL(28, 431, 14), // "DECOR_MAXIMIZE"
QT_MOC_LITERAL(29, 446, 9), // "DECOR_ALL"
QT_MOC_LITERAL(30, 456, 16), // "MotifDecorations"
QT_MOC_LITERAL(31, 473, 6), // "WMName"
QT_MOC_LITERAL(32, 480, 7), // "OtherWM"
QT_MOC_LITERAL(33, 488, 8), // "DeepinWM"
QT_MOC_LITERAL(34, 497, 6), // "KWinWM"
QT_MOC_LITERAL(35, 504, 12), // "WmWindowType"
QT_MOC_LITERAL(36, 517, 16), // "UnknowWindowType"
QT_MOC_LITERAL(37, 534, 10), // "NormalType"
QT_MOC_LITERAL(38, 545, 11), // "DesktopType"
QT_MOC_LITERAL(39, 557, 8), // "DockType"
QT_MOC_LITERAL(40, 566, 11), // "ToolbarType"
QT_MOC_LITERAL(41, 578, 8), // "MenuType"
QT_MOC_LITERAL(42, 587, 11), // "UtilityType"
QT_MOC_LITERAL(43, 599, 10), // "SplashType"
QT_MOC_LITERAL(44, 610, 10), // "DialogType"
QT_MOC_LITERAL(45, 621, 16), // "DropDownMenuType"
QT_MOC_LITERAL(46, 638, 13), // "PopupMenuType"
QT_MOC_LITERAL(47, 652, 11), // "TooltipType"
QT_MOC_LITERAL(48, 664, 16), // "NotificationType"
QT_MOC_LITERAL(49, 681, 9), // "ComboType"
QT_MOC_LITERAL(50, 691, 7), // "DndType"
QT_MOC_LITERAL(51, 699, 15), // "KdeOverrideType"
QT_MOC_LITERAL(52, 715, 13) // "WmWindowTypes"

    },
    "Dtk::Gui::DWindowManagerHelper\0"
    "windowManagerChanged\0\0hasBlurWindowChanged\0"
    "hasCompositeChanged\0hasNoTitlebarChanged\0"
    "hasWallpaperEffectChanged\0windowListChanged\0"
    "windowMotifWMHintsChanged\0winId\0"
    "hasBlurWindow\0hasComposite\0hasNoTitlebar\0"
    "hasWallpaperEffect\0MotifFunction\0"
    "FUNC_RESIZE\0FUNC_MOVE\0FUNC_MINIMIZE\0"
    "FUNC_MAXIMIZE\0FUNC_CLOSE\0FUNC_ALL\0"
    "MotifFunctions\0MotifDecoration\0"
    "DECOR_BORDER\0DECOR_RESIZEH\0DECOR_TITLE\0"
    "DECOR_MENU\0DECOR_MINIMIZE\0DECOR_MAXIMIZE\0"
    "DECOR_ALL\0MotifDecorations\0WMName\0"
    "OtherWM\0DeepinWM\0KWinWM\0WmWindowType\0"
    "UnknowWindowType\0NormalType\0DesktopType\0"
    "DockType\0ToolbarType\0MenuType\0UtilityType\0"
    "SplashType\0DialogType\0DropDownMenuType\0"
    "PopupMenuType\0TooltipType\0NotificationType\0"
    "ComboType\0DndType\0KdeOverrideType\0"
    "WmWindowTypes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DWindowManagerHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       4,   58, // properties
       7,   74, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,
       4,    0,   51,    2, 0x06 /* Public */,
       5,    0,   52,    2, 0x06 /* Public */,
       6,    0,   53,    2, 0x06 /* Public */,
       7,    0,   54,    2, 0x06 /* Public */,
       8,    1,   55,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt,    9,

 // properties: name, type, flags
      10, QMetaType::Bool, 0x00495001,
      11, QMetaType::Bool, 0x00495001,
      12, QMetaType::Bool, 0x00495001,
      13, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      14,   14, 0x0,    6,  109,
      21,   14, 0x1,    6,  121,
      22,   22, 0x0,    7,  133,
      30,   22, 0x1,    7,  147,
      31,   31, 0x0,    3,  161,
      35,   35, 0x0,   16,  167,
      52,   35, 0x1,   16,  199,

 // enum data: key, value
      15, uint(Dtk::Gui::DWindowManagerHelper::FUNC_RESIZE),
      16, uint(Dtk::Gui::DWindowManagerHelper::FUNC_MOVE),
      17, uint(Dtk::Gui::DWindowManagerHelper::FUNC_MINIMIZE),
      18, uint(Dtk::Gui::DWindowManagerHelper::FUNC_MAXIMIZE),
      19, uint(Dtk::Gui::DWindowManagerHelper::FUNC_CLOSE),
      20, uint(Dtk::Gui::DWindowManagerHelper::FUNC_ALL),
      15, uint(Dtk::Gui::DWindowManagerHelper::FUNC_RESIZE),
      16, uint(Dtk::Gui::DWindowManagerHelper::FUNC_MOVE),
      17, uint(Dtk::Gui::DWindowManagerHelper::FUNC_MINIMIZE),
      18, uint(Dtk::Gui::DWindowManagerHelper::FUNC_MAXIMIZE),
      19, uint(Dtk::Gui::DWindowManagerHelper::FUNC_CLOSE),
      20, uint(Dtk::Gui::DWindowManagerHelper::FUNC_ALL),
      23, uint(Dtk::Gui::DWindowManagerHelper::DECOR_BORDER),
      24, uint(Dtk::Gui::DWindowManagerHelper::DECOR_RESIZEH),
      25, uint(Dtk::Gui::DWindowManagerHelper::DECOR_TITLE),
      26, uint(Dtk::Gui::DWindowManagerHelper::DECOR_MENU),
      27, uint(Dtk::Gui::DWindowManagerHelper::DECOR_MINIMIZE),
      28, uint(Dtk::Gui::DWindowManagerHelper::DECOR_MAXIMIZE),
      29, uint(Dtk::Gui::DWindowManagerHelper::DECOR_ALL),
      23, uint(Dtk::Gui::DWindowManagerHelper::DECOR_BORDER),
      24, uint(Dtk::Gui::DWindowManagerHelper::DECOR_RESIZEH),
      25, uint(Dtk::Gui::DWindowManagerHelper::DECOR_TITLE),
      26, uint(Dtk::Gui::DWindowManagerHelper::DECOR_MENU),
      27, uint(Dtk::Gui::DWindowManagerHelper::DECOR_MINIMIZE),
      28, uint(Dtk::Gui::DWindowManagerHelper::DECOR_MAXIMIZE),
      29, uint(Dtk::Gui::DWindowManagerHelper::DECOR_ALL),
      32, uint(Dtk::Gui::DWindowManagerHelper::OtherWM),
      33, uint(Dtk::Gui::DWindowManagerHelper::DeepinWM),
      34, uint(Dtk::Gui::DWindowManagerHelper::KWinWM),
      36, uint(Dtk::Gui::DWindowManagerHelper::UnknowWindowType),
      37, uint(Dtk::Gui::DWindowManagerHelper::NormalType),
      38, uint(Dtk::Gui::DWindowManagerHelper::DesktopType),
      39, uint(Dtk::Gui::DWindowManagerHelper::DockType),
      40, uint(Dtk::Gui::DWindowManagerHelper::ToolbarType),
      41, uint(Dtk::Gui::DWindowManagerHelper::MenuType),
      42, uint(Dtk::Gui::DWindowManagerHelper::UtilityType),
      43, uint(Dtk::Gui::DWindowManagerHelper::SplashType),
      44, uint(Dtk::Gui::DWindowManagerHelper::DialogType),
      45, uint(Dtk::Gui::DWindowManagerHelper::DropDownMenuType),
      46, uint(Dtk::Gui::DWindowManagerHelper::PopupMenuType),
      47, uint(Dtk::Gui::DWindowManagerHelper::TooltipType),
      48, uint(Dtk::Gui::DWindowManagerHelper::NotificationType),
      49, uint(Dtk::Gui::DWindowManagerHelper::ComboType),
      50, uint(Dtk::Gui::DWindowManagerHelper::DndType),
      51, uint(Dtk::Gui::DWindowManagerHelper::KdeOverrideType),
      36, uint(Dtk::Gui::DWindowManagerHelper::UnknowWindowType),
      37, uint(Dtk::Gui::DWindowManagerHelper::NormalType),
      38, uint(Dtk::Gui::DWindowManagerHelper::DesktopType),
      39, uint(Dtk::Gui::DWindowManagerHelper::DockType),
      40, uint(Dtk::Gui::DWindowManagerHelper::ToolbarType),
      41, uint(Dtk::Gui::DWindowManagerHelper::MenuType),
      42, uint(Dtk::Gui::DWindowManagerHelper::UtilityType),
      43, uint(Dtk::Gui::DWindowManagerHelper::SplashType),
      44, uint(Dtk::Gui::DWindowManagerHelper::DialogType),
      45, uint(Dtk::Gui::DWindowManagerHelper::DropDownMenuType),
      46, uint(Dtk::Gui::DWindowManagerHelper::PopupMenuType),
      47, uint(Dtk::Gui::DWindowManagerHelper::TooltipType),
      48, uint(Dtk::Gui::DWindowManagerHelper::NotificationType),
      49, uint(Dtk::Gui::DWindowManagerHelper::ComboType),
      50, uint(Dtk::Gui::DWindowManagerHelper::DndType),
      51, uint(Dtk::Gui::DWindowManagerHelper::KdeOverrideType),

       0        // eod
};

void Dtk::Gui::DWindowManagerHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DWindowManagerHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->windowManagerChanged(); break;
        case 1: _t->hasBlurWindowChanged(); break;
        case 2: _t->hasCompositeChanged(); break;
        case 3: _t->hasNoTitlebarChanged(); break;
        case 4: _t->hasWallpaperEffectChanged(); break;
        case 5: _t->windowListChanged(); break;
        case 6: _t->windowMotifWMHintsChanged((*reinterpret_cast< quint32(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DWindowManagerHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::windowManagerChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DWindowManagerHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::hasBlurWindowChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DWindowManagerHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::hasCompositeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DWindowManagerHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::hasNoTitlebarChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DWindowManagerHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::hasWallpaperEffectChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DWindowManagerHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::windowListChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DWindowManagerHelper::*)(quint32 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DWindowManagerHelper::windowMotifWMHintsChanged)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DWindowManagerHelper *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->hasBlurWindow(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->hasComposite(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->hasNoTitlebar(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->hasWallpaperEffect(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DWindowManagerHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Gui__DWindowManagerHelper.data,
    qt_meta_data_Dtk__Gui__DWindowManagerHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Gui::DWindowManagerHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DWindowManagerHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Gui__DWindowManagerHelper.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DWindowManagerHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DWindowManagerHelper::windowManagerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Dtk::Gui::DWindowManagerHelper::hasBlurWindowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Dtk::Gui::DWindowManagerHelper::hasCompositeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Dtk::Gui::DWindowManagerHelper::hasNoTitlebarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Dtk::Gui::DWindowManagerHelper::hasWallpaperEffectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Dtk::Gui::DWindowManagerHelper::windowListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Dtk::Gui::DWindowManagerHelper::windowMotifWMHintsChanged(quint32 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
