/****************************************************************************
** Meta object code from reading C++ file 'dplatformtheme.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/kernel/dplatformtheme.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dplatformtheme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DPlatformTheme::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DPlatformTheme",
        "cursorBlinkTimeChanged",
        "",
        "cursorBlinkTime",
        "cursorBlinkTimeoutChanged",
        "cursorBlinkTimeout",
        "cursorBlinkChanged",
        "cursorBlink",
        "doubleClickDistanceChanged",
        "doubleClickDistance",
        "doubleClickTimeChanged",
        "doubleClickTime",
        "dndDragThresholdChanged",
        "dndDragThreshold",
        "themeNameChanged",
        "themeName",
        "iconThemeNameChanged",
        "iconThemeName",
        "soundThemeNameChanged",
        "soundThemeName",
        "fontNameChanged",
        "fontName",
        "monoFontNameChanged",
        "monoFontName",
        "fontPointSizeChanged",
        "fontPointSize",
        "gtkFontNameChanged",
        "activeColorChanged",
        "QColor",
        "activeColor",
        "darkActiveColorChanged",
        "paletteChanged",
        "DPalette",
        "palette",
        "dotsPerInchChanged",
        "screen",
        "dpi",
        "windowRadiusChanged",
        "r",
        "sizeModeChanged",
        "sizeMode",
        "scrollBarPolicyChanged",
        "scrollBarPolicy",
        "setCursorBlinkTime",
        "setCursorBlinkTimeout",
        "setCursorBlink",
        "setDoubleClickDistance",
        "setDoubleClickTime",
        "setDndDragThreshold",
        "setThemeName",
        "setIconThemeName",
        "setSoundThemeName",
        "setFontName",
        "setMonoFontName",
        "setFontPointSize",
        "setGtkFontName",
        "setActiveColor",
        "setDarkActiveColor",
        "dotsPerInch",
        "screenName",
        "setDotsPerInch",
        "setWindowRadius",
        "windowRadius",
        "gtkFontName",
        "darkActiveColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cursorBlinkTimeChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'cursorBlinkTimeoutChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'cursorBlinkChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'doubleClickDistanceChanged'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'doubleClickTimeChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'dndDragThresholdChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Signal 'themeNameChanged'
        QtMocHelpers::SignalData<void(QByteArray)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Signal 'iconThemeNameChanged'
        QtMocHelpers::SignalData<void(QByteArray)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 17 },
        }}),
        // Signal 'soundThemeNameChanged'
        QtMocHelpers::SignalData<void(QByteArray)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 19 },
        }}),
        // Signal 'fontNameChanged'
        QtMocHelpers::SignalData<void(QByteArray)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Signal 'monoFontNameChanged'
        QtMocHelpers::SignalData<void(QByteArray)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 23 },
        }}),
        // Signal 'fontPointSizeChanged'
        QtMocHelpers::SignalData<void(qreal)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Signal 'gtkFontNameChanged'
        QtMocHelpers::SignalData<void(QByteArray)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Signal 'activeColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'darkActiveColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'paletteChanged'
        QtMocHelpers::SignalData<void(DPalette)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Signal 'dotsPerInchChanged'
        QtMocHelpers::SignalData<void(const QString &, int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { QMetaType::Int, 36 },
        }}),
        // Signal 'windowRadiusChanged'
        QtMocHelpers::SignalData<void(int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 38 },
        }}),
        // Signal 'sizeModeChanged'
        QtMocHelpers::SignalData<void(int)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 40 },
        }}),
        // Signal 'scrollBarPolicyChanged'
        QtMocHelpers::SignalData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 42 },
        }}),
        // Slot 'setCursorBlinkTime'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setCursorBlinkTimeout'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setCursorBlink'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setDoubleClickDistance'
        QtMocHelpers::SlotData<void(int)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'setDoubleClickTime'
        QtMocHelpers::SlotData<void(int)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'setDndDragThreshold'
        QtMocHelpers::SlotData<void(int)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'setThemeName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 15 },
        }}),
        // Slot 'setIconThemeName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 17 },
        }}),
        // Slot 'setSoundThemeName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 19 },
        }}),
        // Slot 'setFontName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Slot 'setMonoFontName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 23 },
        }}),
        // Slot 'setFontPointSize'
        QtMocHelpers::SlotData<void(qreal)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Slot 'setGtkFontName'
        QtMocHelpers::SlotData<void(const QByteArray &)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
        // Slot 'setActiveColor'
        QtMocHelpers::SlotData<void(const QColor)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'setDarkActiveColor'
        QtMocHelpers::SlotData<void(const QColor &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'dotsPerInch'
        QtMocHelpers::SlotData<int(const QString &) const>(58, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 59 },
        }}),
        // Slot 'dotsPerInch'
        QtMocHelpers::SlotData<int() const>(58, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int),
        // Slot 'setDotsPerInch'
        QtMocHelpers::SlotData<void(const QString &, int)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 59 }, { QMetaType::Int, 36 },
        }}),
        // Slot 'setWindowRadius'
        QtMocHelpers::SlotData<void(int)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 62 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'cursorBlinkTime'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'cursorBlinkTimeout'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'cursorBlink'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'doubleClickDistance'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'doubleClickTime'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'dndDragThreshold'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'windowRadius'
        QtMocHelpers::PropertyData<int>(62, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'themeName'
        QtMocHelpers::PropertyData<QByteArray>(15, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'iconThemeName'
        QtMocHelpers::PropertyData<QByteArray>(17, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'soundThemeName'
        QtMocHelpers::PropertyData<QByteArray>(19, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'fontName'
        QtMocHelpers::PropertyData<QByteArray>(21, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'monoFontName'
        QtMocHelpers::PropertyData<QByteArray>(23, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'fontPointSize'
        QtMocHelpers::PropertyData<qreal>(25, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'gtkFontName'
        QtMocHelpers::PropertyData<QByteArray>(63, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'activeColor'
        QtMocHelpers::PropertyData<QColor>(29, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 13),
        // property 'darkActiveColor'
        QtMocHelpers::PropertyData<QColor>(64, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'sizeMode'
        QtMocHelpers::PropertyData<int>(40, QMetaType::Int, QMC::DefaultPropertyFlags, 18),
        // property 'scrollBarPolicy'
        QtMocHelpers::PropertyData<int>(42, QMetaType::Int, QMC::DefaultPropertyFlags, 19),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DPlatformTheme, qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DPlatformTheme::staticMetaObject = { {
    QMetaObject::SuperData::link<DNativeSettings::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DPlatformTheme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DPlatformTheme *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cursorBlinkTimeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->cursorBlinkTimeoutChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->cursorBlinkChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->doubleClickDistanceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->doubleClickTimeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->dndDragThresholdChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->themeNameChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 7: _t->iconThemeNameChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 8: _t->soundThemeNameChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 9: _t->fontNameChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 10: _t->monoFontNameChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 11: _t->fontPointSizeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 12: _t->gtkFontNameChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 13: _t->activeColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->darkActiveColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->paletteChanged((*reinterpret_cast<std::add_pointer_t<DPalette>>(_a[1]))); break;
        case 16: _t->dotsPerInchChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->windowRadiusChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->sizeModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->scrollBarPolicyChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->setCursorBlinkTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->setCursorBlinkTimeout((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->setCursorBlink((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->setDoubleClickDistance((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->setDoubleClickTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->setDndDragThreshold((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->setThemeName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 27: _t->setIconThemeName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 28: _t->setSoundThemeName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 29: _t->setFontName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 30: _t->setMonoFontName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 31: _t->setFontPointSize((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 32: _t->setGtkFontName((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 33: _t->setActiveColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 34: _t->setDarkActiveColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 35: { int _r = _t->dotsPerInch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 36: { int _r = _t->dotsPerInch();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->setDotsPerInch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 38: _t->setWindowRadius((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::cursorBlinkTimeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::cursorBlinkTimeoutChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(bool )>(_a, &DPlatformTheme::cursorBlinkChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::doubleClickDistanceChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::doubleClickTimeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::dndDragThresholdChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QByteArray )>(_a, &DPlatformTheme::themeNameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QByteArray )>(_a, &DPlatformTheme::iconThemeNameChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QByteArray )>(_a, &DPlatformTheme::soundThemeNameChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QByteArray )>(_a, &DPlatformTheme::fontNameChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QByteArray )>(_a, &DPlatformTheme::monoFontNameChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(qreal )>(_a, &DPlatformTheme::fontPointSizeChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QByteArray )>(_a, &DPlatformTheme::gtkFontNameChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QColor )>(_a, &DPlatformTheme::activeColorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(QColor )>(_a, &DPlatformTheme::darkActiveColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(DPalette )>(_a, &DPlatformTheme::paletteChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(const QString & , int )>(_a, &DPlatformTheme::dotsPerInchChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::windowRadiusChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::sizeModeChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (DPlatformTheme::*)(int )>(_a, &DPlatformTheme::scrollBarPolicyChanged, 19))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->cursorBlinkTime(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->cursorBlinkTimeout(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->cursorBlink(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->doubleClickDistance(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->doubleClickTime(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->dndDragThreshold(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->windowRadius(); break;
        case 7: *reinterpret_cast<QByteArray*>(_v) = _t->themeName(); break;
        case 8: *reinterpret_cast<QByteArray*>(_v) = _t->iconThemeName(); break;
        case 9: *reinterpret_cast<QByteArray*>(_v) = _t->soundThemeName(); break;
        case 10: *reinterpret_cast<QByteArray*>(_v) = _t->fontName(); break;
        case 11: *reinterpret_cast<QByteArray*>(_v) = _t->monoFontName(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->fontPointSize(); break;
        case 13: *reinterpret_cast<QByteArray*>(_v) = _t->gtkFontName(); break;
        case 14: *reinterpret_cast<QColor*>(_v) = _t->activeColor(); break;
        case 15: *reinterpret_cast<QColor*>(_v) = _t->darkActiveColor(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->sizeMode(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->scrollBarPolicy(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCursorBlinkTime(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setCursorBlinkTimeout(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setCursorBlink(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setDoubleClickDistance(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setDoubleClickTime(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setDndDragThreshold(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setWindowRadius(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setThemeName(*reinterpret_cast<QByteArray*>(_v)); break;
        case 8: _t->setIconThemeName(*reinterpret_cast<QByteArray*>(_v)); break;
        case 9: _t->setSoundThemeName(*reinterpret_cast<QByteArray*>(_v)); break;
        case 10: _t->setFontName(*reinterpret_cast<QByteArray*>(_v)); break;
        case 11: _t->setMonoFontName(*reinterpret_cast<QByteArray*>(_v)); break;
        case 12: _t->setFontPointSize(*reinterpret_cast<qreal*>(_v)); break;
        case 13: _t->setGtkFontName(*reinterpret_cast<QByteArray*>(_v)); break;
        case 14: _t->setActiveColor(*reinterpret_cast<QColor*>(_v)); break;
        case 15: _t->setDarkActiveColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DPlatformTheme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DPlatformTheme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui14DPlatformThemeE_t>.strings))
        return static_cast<void*>(this);
    return DNativeSettings::qt_metacast(_clname);
}

int Dtk::Gui::DPlatformTheme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DNativeSettings::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 39;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DPlatformTheme::cursorBlinkTimeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Dtk::Gui::DPlatformTheme::cursorBlinkTimeoutChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Dtk::Gui::DPlatformTheme::cursorBlinkChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Dtk::Gui::DPlatformTheme::doubleClickDistanceChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Dtk::Gui::DPlatformTheme::doubleClickTimeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Dtk::Gui::DPlatformTheme::dndDragThresholdChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Dtk::Gui::DPlatformTheme::themeNameChanged(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Dtk::Gui::DPlatformTheme::iconThemeNameChanged(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Dtk::Gui::DPlatformTheme::soundThemeNameChanged(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Dtk::Gui::DPlatformTheme::fontNameChanged(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Dtk::Gui::DPlatformTheme::monoFontNameChanged(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Dtk::Gui::DPlatformTheme::fontPointSizeChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Dtk::Gui::DPlatformTheme::gtkFontNameChanged(QByteArray _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Dtk::Gui::DPlatformTheme::activeColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Dtk::Gui::DPlatformTheme::darkActiveColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Dtk::Gui::DPlatformTheme::paletteChanged(DPalette _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Dtk::Gui::DPlatformTheme::dotsPerInchChanged(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void Dtk::Gui::DPlatformTheme::windowRadiusChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Dtk::Gui::DPlatformTheme::sizeModeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Dtk::Gui::DPlatformTheme::scrollBarPolicyChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}
QT_WARNING_POP
