/****************************************************************************
** Meta object code from reading C++ file 'orgdeepindtkpreference.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../orgdeepindtkpreference.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'orgdeepindtkpreference.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgDeepinDTKPreference_t {
    QByteArrayData data[60];
    char stringdata0[1365];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgDeepinDTKPreference_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgDeepinDTKPreference_t qt_meta_stringdata_OrgDeepinDTKPreference = {
    {
QT_MOC_LITERAL(0, 0, 22), // "OrgDeepinDTKPreference"
QT_MOC_LITERAL(1, 23, 14), // "DConfigKeyList"
QT_MOC_LITERAL(2, 38, 197), // "autoDisplayFeature;colorMode;..."
QT_MOC_LITERAL(3, 236, 15), // "DConfigFileName"
QT_MOC_LITERAL(4, 252, 25), // "org.deepin.dtk.preference"
QT_MOC_LITERAL(5, 278, 18), // "DConfigFileVersion"
QT_MOC_LITERAL(6, 297, 3), // "1.0"
QT_MOC_LITERAL(7, 301, 22), // "configInitializeFailed"
QT_MOC_LITERAL(8, 324, 0), // ""
QT_MOC_LITERAL(9, 325, 23), // "configInitializeSucceed"
QT_MOC_LITERAL(10, 349, 19), // "Dtk::Core::DConfig*"
QT_MOC_LITERAL(11, 369, 6), // "config"
QT_MOC_LITERAL(12, 376, 12), // "valueChanged"
QT_MOC_LITERAL(13, 389, 3), // "key"
QT_MOC_LITERAL(14, 393, 5), // "value"
QT_MOC_LITERAL(15, 399, 25), // "autoDisplayFeatureChanged"
QT_MOC_LITERAL(16, 425, 16), // "colorModeChanged"
QT_MOC_LITERAL(17, 442, 23), // "defaultColorModeChanged"
QT_MOC_LITERAL(18, 466, 26), // "disableInWindowBlurChanged"
QT_MOC_LITERAL(19, 493, 26), // "enableDtkAnimationsChanged"
QT_MOC_LITERAL(20, 520, 21), // "featureUpdatedChanged"
QT_MOC_LITERAL(21, 542, 29), // "keyboardsearchDisabledChanged"
QT_MOC_LITERAL(22, 572, 12), // "rulesChanged"
QT_MOC_LITERAL(23, 585, 22), // "scrollBarPolicyChanged"
QT_MOC_LITERAL(24, 608, 15), // "sizeModeChanged"
QT_MOC_LITERAL(25, 624, 16), // "themeTypeChanged"
QT_MOC_LITERAL(26, 641, 21), // "titlebarHeightChanged"
QT_MOC_LITERAL(27, 663, 24), // "underlineShortcutChanged"
QT_MOC_LITERAL(28, 688, 19), // "isInitializeSucceed"
QT_MOC_LITERAL(29, 708, 21), // "isInitializeSucceeded"
QT_MOC_LITERAL(30, 730, 18), // "isInitializeFailed"
QT_MOC_LITERAL(31, 749, 14), // "isInitializing"
QT_MOC_LITERAL(32, 764, 7), // "keyList"
QT_MOC_LITERAL(33, 772, 14), // "isDefaultValue"
QT_MOC_LITERAL(34, 787, 32), // "autoDisplayFeatureIsDefaultValue"
QT_MOC_LITERAL(35, 820, 23), // "colorModeIsDefaultValue"
QT_MOC_LITERAL(36, 844, 30), // "defaultColorModeIsDefaultValue"
QT_MOC_LITERAL(37, 875, 33), // "disableInWindowBlurIsDefaultV..."
QT_MOC_LITERAL(38, 909, 33), // "enableDtkAnimationsIsDefaultV..."
QT_MOC_LITERAL(39, 943, 28), // "featureUpdatedIsDefaultValue"
QT_MOC_LITERAL(40, 972, 36), // "keyboardsearchDisabledIsDefau..."
QT_MOC_LITERAL(41, 1009, 19), // "rulesIsDefaultValue"
QT_MOC_LITERAL(42, 1029, 29), // "scrollBarPolicyIsDefaultValue"
QT_MOC_LITERAL(43, 1059, 22), // "sizeModeIsDefaultValue"
QT_MOC_LITERAL(44, 1082, 23), // "themeTypeIsDefaultValue"
QT_MOC_LITERAL(45, 1106, 28), // "titlebarHeightIsDefaultValue"
QT_MOC_LITERAL(46, 1135, 31), // "underlineShortcutIsDefaultValue"
QT_MOC_LITERAL(47, 1167, 18), // "autoDisplayFeature"
QT_MOC_LITERAL(48, 1186, 9), // "colorMode"
QT_MOC_LITERAL(49, 1196, 16), // "defaultColorMode"
QT_MOC_LITERAL(50, 1213, 19), // "disableInWindowBlur"
QT_MOC_LITERAL(51, 1233, 19), // "enableDtkAnimations"
QT_MOC_LITERAL(52, 1253, 14), // "featureUpdated"
QT_MOC_LITERAL(53, 1268, 22), // "keyboardsearchDisabled"
QT_MOC_LITERAL(54, 1291, 5), // "rules"
QT_MOC_LITERAL(55, 1297, 15), // "scrollBarPolicy"
QT_MOC_LITERAL(56, 1313, 8), // "sizeMode"
QT_MOC_LITERAL(57, 1322, 9), // "themeType"
QT_MOC_LITERAL(58, 1332, 14), // "titlebarHeight"
QT_MOC_LITERAL(59, 1347, 17) // "underlineShortcut"

    },
    "OrgDeepinDTKPreference\0DConfigKeyList\0"
    "autoDisplayFeature;colorMode;defaultColorMode;disableInWindowBlur;enab"
    "leDtkAnimations;featureUpdated;keyboardsearchDisabled;rules;scrollBarP"
    "olicy;sizeMode;themeType;titlebarHeight;underlineShortcut\0"
    "DConfigFileName\0org.deepin.dtk.preference\0"
    "DConfigFileVersion\0""1.0\0configInitializeFailed\0"
    "\0configInitializeSucceed\0Dtk::Core::DConfig*\0"
    "config\0valueChanged\0key\0value\0"
    "autoDisplayFeatureChanged\0colorModeChanged\0"
    "defaultColorModeChanged\0"
    "disableInWindowBlurChanged\0"
    "enableDtkAnimationsChanged\0"
    "featureUpdatedChanged\0"
    "keyboardsearchDisabledChanged\0"
    "rulesChanged\0scrollBarPolicyChanged\0"
    "sizeModeChanged\0themeTypeChanged\0"
    "titlebarHeightChanged\0underlineShortcutChanged\0"
    "isInitializeSucceed\0isInitializeSucceeded\0"
    "isInitializeFailed\0isInitializing\0"
    "keyList\0isDefaultValue\0"
    "autoDisplayFeatureIsDefaultValue\0"
    "colorModeIsDefaultValue\0"
    "defaultColorModeIsDefaultValue\0"
    "disableInWindowBlurIsDefaultValue\0"
    "enableDtkAnimationsIsDefaultValue\0"
    "featureUpdatedIsDefaultValue\0"
    "keyboardsearchDisabledIsDefaultValue\0"
    "rulesIsDefaultValue\0scrollBarPolicyIsDefaultValue\0"
    "sizeModeIsDefaultValue\0themeTypeIsDefaultValue\0"
    "titlebarHeightIsDefaultValue\0"
    "underlineShortcutIsDefaultValue\0"
    "autoDisplayFeature\0colorMode\0"
    "defaultColorMode\0disableInWindowBlur\0"
    "enableDtkAnimations\0featureUpdated\0"
    "keyboardsearchDisabled\0rules\0"
    "scrollBarPolicy\0sizeMode\0themeType\0"
    "titlebarHeight\0underlineShortcut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgDeepinDTKPreference[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
      36,   20, // methods
      13,  244, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,  200,    8, 0x06 /* Public */,
       9,    1,  201,    8, 0x06 /* Public */,
      12,    2,  204,    8, 0x06 /* Public */,
      15,    0,  209,    8, 0x06 /* Public */,
      16,    0,  210,    8, 0x06 /* Public */,
      17,    0,  211,    8, 0x06 /* Public */,
      18,    0,  212,    8, 0x06 /* Public */,
      19,    0,  213,    8, 0x06 /* Public */,
      20,    0,  214,    8, 0x06 /* Public */,
      21,    0,  215,    8, 0x06 /* Public */,
      22,    0,  216,    8, 0x06 /* Public */,
      23,    0,  217,    8, 0x06 /* Public */,
      24,    0,  218,    8, 0x06 /* Public */,
      25,    0,  219,    8, 0x06 /* Public */,
      26,    0,  220,    8, 0x06 /* Public */,
      27,    0,  221,    8, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      11,    0,  222,    8, 0x02 /* Public */,
      28,    0,  223,    8, 0x02 /* Public */,
      29,    0,  224,    8, 0x02 /* Public */,
      30,    0,  225,    8, 0x02 /* Public */,
      31,    0,  226,    8, 0x02 /* Public */,
      32,    0,  227,    8, 0x02 /* Public */,
      33,    1,  228,    8, 0x02 /* Public */,
      34,    0,  231,    8, 0x02 /* Public */,
      35,    0,  232,    8, 0x02 /* Public */,
      36,    0,  233,    8, 0x02 /* Public */,
      37,    0,  234,    8, 0x02 /* Public */,
      38,    0,  235,    8, 0x02 /* Public */,
      39,    0,  236,    8, 0x02 /* Public */,
      40,    0,  237,    8, 0x02 /* Public */,
      41,    0,  238,    8, 0x02 /* Public */,
      42,    0,  239,    8, 0x02 /* Public */,
      43,    0,  240,    8, 0x02 /* Public */,
      44,    0,  241,    8, 0x02 /* Public */,
      45,    0,  242,    8, 0x02 /* Public */,
      46,    0,  243,    8, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariant,   13,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    0x80000000 | 10,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::QStringList,
    QMetaType::Bool, QMetaType::QString,   13,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,

 // properties: name, type, flags
      47, QMetaType::Bool, 0x00495107,
      48, QMetaType::QString, 0x00495107,
      49, QMetaType::QString, 0x00495107,
      50, QMetaType::Bool, 0x00495107,
      51, QMetaType::Bool, 0x00495107,
      52, QMetaType::Bool, 0x00495107,
      53, QMetaType::Bool, 0x00495107,
      54, QMetaType::QString, 0x00495107,
      55, QMetaType::LongLong, 0x00495107,
      56, QMetaType::LongLong, 0x00495107,
      57, QMetaType::LongLong, 0x00495107,
      58, QMetaType::LongLong, 0x00495107,
      59, QMetaType::Bool, 0x00495107,

 // properties: notify_signal_id
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,

       0        // eod
};

void OrgDeepinDTKPreference::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgDeepinDTKPreference *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast< Dtk::Core::DConfig*(*)>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 3: _t->autoDisplayFeatureChanged(); break;
        case 4: _t->colorModeChanged(); break;
        case 5: _t->defaultColorModeChanged(); break;
        case 6: _t->disableInWindowBlurChanged(); break;
        case 7: _t->enableDtkAnimationsChanged(); break;
        case 8: _t->featureUpdatedChanged(); break;
        case 9: _t->keyboardsearchDisabledChanged(); break;
        case 10: _t->rulesChanged(); break;
        case 11: _t->scrollBarPolicyChanged(); break;
        case 12: _t->sizeModeChanged(); break;
        case 13: _t->themeTypeChanged(); break;
        case 14: _t->titlebarHeightChanged(); break;
        case 15: _t->underlineShortcutChanged(); break;
        case 16: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast< Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->isDefaultValue((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->autoDisplayFeatureIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->colorModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->defaultColorModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->disableInWindowBlurIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->enableDtkAnimationsIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->featureUpdatedIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->keyboardsearchDisabledIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->rulesIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->scrollBarPolicyIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->sizeModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->themeTypeIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->titlebarHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->underlineShortcutIsDefaultValue();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::configInitializeFailed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)(Dtk::Core::DConfig * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::configInitializeSucceed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)(const QString & , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::valueChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::autoDisplayFeatureChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::colorModeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::defaultColorModeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::disableInWindowBlurChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::enableDtkAnimationsChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::featureUpdatedChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::keyboardsearchDisabledChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::rulesChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::scrollBarPolicyChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::sizeModeChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::themeTypeChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::titlebarHeightChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (OrgDeepinDTKPreference::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgDeepinDTKPreference::underlineShortcutChanged)) {
                *result = 15;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<OrgDeepinDTKPreference *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->autoDisplayFeature(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->colorMode(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->defaultColorMode(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->disableInWindowBlur(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->enableDtkAnimations(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->featureUpdated(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->keyboardsearchDisabled(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->rules(); break;
        case 8: *reinterpret_cast< qlonglong*>(_v) = _t->scrollBarPolicy(); break;
        case 9: *reinterpret_cast< qlonglong*>(_v) = _t->sizeMode(); break;
        case 10: *reinterpret_cast< qlonglong*>(_v) = _t->themeType(); break;
        case 11: *reinterpret_cast< qlonglong*>(_v) = _t->titlebarHeight(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->underlineShortcut(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<OrgDeepinDTKPreference *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAutoDisplayFeature(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setColorMode(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setDefaultColorMode(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setDisableInWindowBlur(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setEnableDtkAnimations(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setFeatureUpdated(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setKeyboardsearchDisabled(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setRules(*reinterpret_cast< QString*>(_v)); break;
        case 8: _t->setScrollBarPolicy(*reinterpret_cast< qlonglong*>(_v)); break;
        case 9: _t->setSizeMode(*reinterpret_cast< qlonglong*>(_v)); break;
        case 10: _t->setThemeType(*reinterpret_cast< qlonglong*>(_v)); break;
        case 11: _t->setTitlebarHeight(*reinterpret_cast< qlonglong*>(_v)); break;
        case 12: _t->setUnderlineShortcut(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        OrgDeepinDTKPreference *_t = static_cast<OrgDeepinDTKPreference *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetAutoDisplayFeature(); break;
        case 1: _t->resetColorMode(); break;
        case 2: _t->resetDefaultColorMode(); break;
        case 3: _t->resetDisableInWindowBlur(); break;
        case 4: _t->resetEnableDtkAnimations(); break;
        case 5: _t->resetFeatureUpdated(); break;
        case 6: _t->resetKeyboardsearchDisabled(); break;
        case 7: _t->resetRules(); break;
        case 8: _t->resetScrollBarPolicy(); break;
        case 9: _t->resetSizeMode(); break;
        case 10: _t->resetThemeType(); break;
        case 11: _t->resetTitlebarHeight(); break;
        case 12: _t->resetUnderlineShortcut(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject OrgDeepinDTKPreference::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_OrgDeepinDTKPreference.data,
    qt_meta_data_OrgDeepinDTKPreference,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgDeepinDTKPreference::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgDeepinDTKPreference::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgDeepinDTKPreference.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OrgDeepinDTKPreference::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 13;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void OrgDeepinDTKPreference::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgDeepinDTKPreference::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void OrgDeepinDTKPreference::valueChanged(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void OrgDeepinDTKPreference::autoDisplayFeatureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OrgDeepinDTKPreference::colorModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OrgDeepinDTKPreference::defaultColorModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OrgDeepinDTKPreference::disableInWindowBlurChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OrgDeepinDTKPreference::enableDtkAnimationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OrgDeepinDTKPreference::featureUpdatedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OrgDeepinDTKPreference::keyboardsearchDisabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void OrgDeepinDTKPreference::rulesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OrgDeepinDTKPreference::scrollBarPolicyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void OrgDeepinDTKPreference::sizeModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void OrgDeepinDTKPreference::themeTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void OrgDeepinDTKPreference::titlebarHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void OrgDeepinDTKPreference::underlineShortcutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
