/****************************************************************************
** Meta object code from reading C++ file 'dpalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/kernel/dpalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Gui__DPalette_t {
    QByteArrayData data[15];
    char stringdata0[192];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Gui__DPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Gui__DPalette_t qt_meta_stringdata_Dtk__Gui__DPalette = {
    {
QT_MOC_LITERAL(0, 0, 18), // "Dtk::Gui::DPalette"
QT_MOC_LITERAL(1, 19, 9), // "ColorType"
QT_MOC_LITERAL(2, 29, 6), // "NoType"
QT_MOC_LITERAL(3, 36, 14), // "ItemBackground"
QT_MOC_LITERAL(4, 51, 9), // "TextTitle"
QT_MOC_LITERAL(5, 61, 8), // "TextTips"
QT_MOC_LITERAL(6, 70, 11), // "TextWarning"
QT_MOC_LITERAL(7, 82, 10), // "TextLively"
QT_MOC_LITERAL(8, 93, 11), // "LightLively"
QT_MOC_LITERAL(9, 105, 10), // "DarkLively"
QT_MOC_LITERAL(10, 116, 11), // "FrameBorder"
QT_MOC_LITERAL(11, 128, 15), // "PlaceholderText"
QT_MOC_LITERAL(12, 144, 17), // "FrameShadowBorder"
QT_MOC_LITERAL(13, 162, 17), // "ObviousBackground"
QT_MOC_LITERAL(14, 180, 11) // "NColorTypes"

    },
    "Dtk::Gui::DPalette\0ColorType\0NoType\0"
    "ItemBackground\0TextTitle\0TextTips\0"
    "TextWarning\0TextLively\0LightLively\0"
    "DarkLively\0FrameBorder\0PlaceholderText\0"
    "FrameShadowBorder\0ObviousBackground\0"
    "NColorTypes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Gui__DPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,   13,   19,

 // enum data: key, value
       2, uint(Dtk::Gui::DPalette::NoType),
       3, uint(Dtk::Gui::DPalette::ItemBackground),
       4, uint(Dtk::Gui::DPalette::TextTitle),
       5, uint(Dtk::Gui::DPalette::TextTips),
       6, uint(Dtk::Gui::DPalette::TextWarning),
       7, uint(Dtk::Gui::DPalette::TextLively),
       8, uint(Dtk::Gui::DPalette::LightLively),
       9, uint(Dtk::Gui::DPalette::DarkLively),
      10, uint(Dtk::Gui::DPalette::FrameBorder),
      11, uint(Dtk::Gui::DPalette::PlaceholderText),
      12, uint(Dtk::Gui::DPalette::FrameShadowBorder),
      13, uint(Dtk::Gui::DPalette::ObviousBackground),
      14, uint(Dtk::Gui::DPalette::NColorTypes),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject Dtk::Gui::DPalette::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QPalette>::value(),
    qt_meta_stringdata_Dtk__Gui__DPalette.data,
    qt_meta_data_Dtk__Gui__DPalette,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
