/****************************************************************************
** Meta object code from reading C++ file 'dguiapplicationhelper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/kernel/dguiapplicationhelper.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dguiapplicationhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DGuiApplicationHelper::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DGuiApplicationHelper",
        "themeTypeChanged",
        "",
        "ColorType",
        "themeType",
        "paletteTypeChanged",
        "paletteType",
        "newProcessInstance",
        "pid",
        "arguments",
        "fontChanged",
        "QFont",
        "font",
        "applicationPaletteChanged",
        "sizeModeChanged",
        "DGuiApplicationHelper::SizeMode",
        "sizeMode",
        "setPaletteType",
        "handleHelpAction",
        "openUrl",
        "url",
        "setSizeMode",
        "mode",
        "resetSizeMode",
        "isCompactMode",
        "_q_initApplicationTheme",
        "_q_sizeModeChanged",
        "hasUserManual",
        "UnknownType",
        "LightType",
        "DarkType",
        "SingleScope",
        "UserScope",
        "GroupScope",
        "WorldScope",
        "SizeMode",
        "NormalMode",
        "CompactMode",
        "Attribute",
        "UseInactiveColorGroup",
        "ColorCompositing",
        "DontSaveApplicationTheme",
        "ReadOnlyLimit",
        "IsDeepinPlatformTheme",
        "IsDXcbPlatform",
        "IsXWindowPlatform",
        "IsTableEnvironment",
        "IsDeepinEnvironment",
        "IsSpecialEffectsEnvironment",
        "IsWaylandPlatform",
        "IsTreelandPlatform",
        "HasAnimations",
        "HasInWindowBlur"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'themeTypeChanged'
        QtMocHelpers::SignalData<void(enum ColorType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'paletteTypeChanged'
        QtMocHelpers::SignalData<void(enum ColorType)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'newProcessInstance'
        QtMocHelpers::SignalData<void(qint64, const QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 }, { QMetaType::QStringList, 9 },
        }}),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'applicationPaletteChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sizeModeChanged'
        QtMocHelpers::SignalData<void(DGuiApplicationHelper::SizeMode)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setPaletteType'
        QtMocHelpers::SlotData<void(enum ColorType)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Slot 'handleHelpAction'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'sizeMode'
        QtMocHelpers::SlotData<DGuiApplicationHelper::SizeMode() const>(16, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'setSizeMode'
        QtMocHelpers::SlotData<void(const DGuiApplicationHelper::SizeMode)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 22 },
        }}),
        // Slot 'resetSizeMode'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'isCompactMode'
        QtMocHelpers::SlotData<bool()>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot '_q_initApplicationTheme'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot '_q_sizeModeChanged'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'themeType'
        QtMocHelpers::PropertyData<enum ColorType>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'paletteType'
        QtMocHelpers::PropertyData<enum ColorType>(6, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'hasUserManual'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ColorType'
        QtMocHelpers::EnumData<enum ColorType>(3, 3, QMC::EnumFlags{}).add({
            {   28, ColorType::UnknownType },
            {   29, ColorType::LightType },
            {   30, ColorType::DarkType },
        }),
        // enum 'SingleScope'
        QtMocHelpers::EnumData<enum SingleScope>(31, 31, QMC::EnumFlags{}).add({
            {   32, SingleScope::UserScope },
            {   33, SingleScope::GroupScope },
            {   34, SingleScope::WorldScope },
        }),
        // enum 'SizeMode'
        QtMocHelpers::EnumData<enum SizeMode>(35, 35, QMC::EnumFlags{}).add({
            {   36, SizeMode::NormalMode },
            {   37, SizeMode::CompactMode },
        }),
        // enum 'Attribute'
        QtMocHelpers::EnumData<enum Attribute>(38, 38, QMC::EnumFlags{}).add({
            {   39, Attribute::UseInactiveColorGroup },
            {   40, Attribute::ColorCompositing },
            {   41, Attribute::DontSaveApplicationTheme },
            {   42, Attribute::ReadOnlyLimit },
            {   43, Attribute::IsDeepinPlatformTheme },
            {   44, Attribute::IsDXcbPlatform },
            {   45, Attribute::IsXWindowPlatform },
            {   46, Attribute::IsTableEnvironment },
            {   47, Attribute::IsDeepinEnvironment },
            {   48, Attribute::IsSpecialEffectsEnvironment },
            {   49, Attribute::IsWaylandPlatform },
            {   50, Attribute::IsTreelandPlatform },
            {   51, Attribute::HasAnimations },
            {   52, Attribute::HasInWindowBlur },
        }),
    };
    return QtMocHelpers::metaObjectData<DGuiApplicationHelper, qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DGuiApplicationHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::DGuiApplicationHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DGuiApplicationHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->themeTypeChanged((*reinterpret_cast<std::add_pointer_t<enum ColorType>>(_a[1]))); break;
        case 1: _t->paletteTypeChanged((*reinterpret_cast<std::add_pointer_t<enum ColorType>>(_a[1]))); break;
        case 2: _t->newProcessInstance((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 3: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 4: _t->applicationPaletteChanged(); break;
        case 5: _t->sizeModeChanged((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::SizeMode>>(_a[1]))); break;
        case 6: _t->setPaletteType((*reinterpret_cast<std::add_pointer_t<enum ColorType>>(_a[1]))); break;
        case 7: _t->handleHelpAction(); break;
        case 8: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: { DGuiApplicationHelper::SizeMode _r = _t->sizeMode();
            if (_a[0]) *reinterpret_cast<DGuiApplicationHelper::SizeMode*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setSizeMode((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::SizeMode>>(_a[1]))); break;
        case 11: _t->resetSizeMode(); break;
        case 12: { bool _r = _t->isCompactMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->d_func()->_q_initApplicationTheme((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->d_func()->_q_sizeModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DGuiApplicationHelper::*)(ColorType )>(_a, &DGuiApplicationHelper::themeTypeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DGuiApplicationHelper::*)(ColorType )>(_a, &DGuiApplicationHelper::paletteTypeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DGuiApplicationHelper::*)(qint64 , const QStringList & )>(_a, &DGuiApplicationHelper::newProcessInstance, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DGuiApplicationHelper::*)(const QFont & )>(_a, &DGuiApplicationHelper::fontChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DGuiApplicationHelper::*)()>(_a, &DGuiApplicationHelper::applicationPaletteChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DGuiApplicationHelper::*)(DGuiApplicationHelper::SizeMode )>(_a, &DGuiApplicationHelper::sizeModeChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum ColorType*>(_v) = _t->themeType(); break;
        case 1: *reinterpret_cast<enum ColorType*>(_v) = _t->paletteType(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasUserManual(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setPaletteType(*reinterpret_cast<enum ColorType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Dtk::Gui::DGuiApplicationHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::DGuiApplicationHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui21DGuiApplicationHelperE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Core::DObject"))
        return static_cast< Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Gui::DGuiApplicationHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Gui::DGuiApplicationHelper::themeTypeChanged(ColorType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Dtk::Gui::DGuiApplicationHelper::paletteTypeChanged(ColorType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Dtk::Gui::DGuiApplicationHelper::newProcessInstance(qint64 _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Dtk::Gui::DGuiApplicationHelper::fontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Dtk::Gui::DGuiApplicationHelper::applicationPaletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Dtk::Gui::DGuiApplicationHelper::sizeModeChanged(DGuiApplicationHelper::SizeMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
